import React from 'react';
import LayoutCard, { LayoutIframeCard } from './LayoutCard';
import { jrequest } from 'utils/request';
import { Responsive, WidthProvider } from 'react-grid-layout';
import { Card, Button, Modal, Select, Input, message } from 'antd';
import '../../../node_modules/react-grid-layout/css/styles.css';
import "../../../node_modules/react-resizable/css/styles.css";
import StoreRank from './NormalComponent/StoreRank';
import InfoBar from './NormalComponent/InfoBar';
import StaticChinaMap from './NormalComponent/StaticChinaMap';
import BirdGrid from "../../components/Grid/BirdGrid";

import Editor from 'react-simple-code-editor';
import { highlight, languages } from 'prismjs/components/prism-core';
import 'prismjs/components/prism-clike';
import 'prismjs/components/prism-javascript';

const ResponsiveGridLayout = WidthProvider(Responsive);

const splitStr = "^^split^^";

export default class LayoutBox extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            visible:false,
            layouts: props.layouts ? props.layouts : [] ,
            
            //TODO 验证测试用的假数据,后面会改回来
            selectKinds: ["普通组件", "图表组件"],
            isNormal:true,
            selectItems: [],
            showBirdGrid:false, //动态birdGrid是否显示
            birdgird_option_code:"", //使用了Editor的要记录的code
        }
        this.staticData = {
            selectedItem: null,
            layouts: [],
            normalList:{
                "门店销售情况": <StoreRank/>,
                "动态门店销售情况":"",
                "消息栏": <InfoBar/>,
                "动态消息栏":"",
                "用户分布(示例)": <StaticChinaMap/>,
                "动态BirdGrid": "",
            },
            birdGridOption:"", //动态BirdGrid的option
            birdGridTitle:"", //动态BirdGrid的name
        }
    }

    renderLayouts = () => {
        let layoutDoms = this.state.layouts.map((item,index) => {
            var isIframe = item.isIframe;
            //是否可动
            item.static = !this.props.active;
            if(isIframe) {
                return (
                    <div data-grid={item} key={item.i} >
                        <LayoutIframeCard key={item.i} id={item.i} title={item.title} url={item.content} active={this.props.active} onDel={() => this.onDelCard(item.i)} />
                    </div>
                );
            } else {
                let idStr = item.i;
                let strs = idStr.split(splitStr)
                if(strs.length == 1) {
                    //非动态BirdGrid的组件数据
                    let content = this.staticData.normalList[item.i];
                    return (
                        <div data-grid={item} key={item.i}>
                            <LayoutCard title={item.i} content={content} active={this.props.active} onDel={() => this.onDelCard(item.i)}/>
                        </div>
                    );
                } else if(strs[0].startsWith("动态消息栏")) {
                    //动态消息栏的组件
                    let optionStr = strs[1];
                    let evalObj1;
                    let option;
                    //解返回来的配置信息
                    
                    try{
                        option = JSON.parse(optionStr,function(k,v){
                            if(v.indexOf&&v.indexOf('function')>-1){
                               return eval("(function(){return "+v+" })()")
                            }
                            return v;
                          });
                    } catch(err) {
                        optionStr = "evalObj1 = " + optionStr;
                        eval(optionStr);
                    }
                    
                    let content;
                    if(option) {
                        content = <InfoBar option={option} />;
                    } else {
                        content = <InfoBar option={evalObj1} />;
                    }
                    
                    return (
                        <div data-grid={item} key={strs[0]}>
                            <LayoutCard title={item.title} content={content} active={this.props.active} onDel={() => this.onDelCard(strs[0])}/>
                        </div>
                    );

                } else if(strs[0].startsWith("动态门店销售情况")) {
                    //动态消息栏的组件
                    let optionStr = strs[1];
                    let evalObj2;
                    let option;
                    //解返回来的配置信息
                    
                    try{
                        option = JSON.parse(optionStr,function(k,v){
                            if(v.indexOf&&v.indexOf('function')>-1){
                               return eval("(function(){return "+v+" })()")
                            }
                            return v;
                          });
                    } catch(err) {
                        optionStr = "evalObj2 = " + optionStr;
                        eval(optionStr);
                    }
                    
                    let content;
                    if(option) {
                        content = <StoreRank option={option} />;
                    } else {
                        content = <StoreRank option={evalObj2} />;
                    }
                    
                    return (
                        <div data-grid={item} key={strs[0]}>
                            <LayoutCard title={item.title} content={content} active={this.props.active} onDel={() => this.onDelCard(strs[0])}/>
                        </div>
                    );
                } else {
                    //动态BirdGrid的组件
                    let optionStr = strs[1];
                    let evalObj;
                    let option;
                    //解返回来的配置信息
                    
                    try{
                        option = JSON.parse(optionStr,function(k,v){
                            if(v.indexOf&&v.indexOf('function')>-1){
                               return eval("(function(){return "+v+" })()")
                            }
                            return v;
                          });
                    } catch(err) {
                        optionStr = "evalObj = " + optionStr;
                        eval(optionStr);
                    }
                    
                    let content;
                    if(option) {
                        content = <BirdGrid gridOption={option} />;
                    } else {
                        content = <BirdGrid gridOption={evalObj} />;
                    }
                    
                    return (
                        <div data-grid={item} key={strs[0]}>
                            <LayoutCard title={item.title} content={content} active={this.props.active} onDel={() => this.onDelCard(strs[0])}/>
                        </div>
                    );
                } 
            }
        });

        let gridLaout = (
            <ResponsiveGridLayout className="layout"
                rowHeight={50}
                // breakpoints={{lg: 1200, md: 996, sm: 768, xs: 480, xxs: 0}}
                // cols={{lg: 12, md: 10, sm: 6, xs: 4, xxs: 2}} 
                breakpoints={{lg: 1200}}
                cols={{lg: 12}} 
                onLayoutChange={this.onLayoutChange}
                onResize={this.onResize}
                onDragStart={this.onDragStart}>

                {layoutDoms}
            </ResponsiveGridLayout >
        );
        return gridLaout;
    }

    onResize = (layout, oldItem, newItem, placeholder, event, element) => {
        let id = "if"+newItem.i;
        let ifm = document.getElementById(id);
        if(ifm) {
            console.log("found");
            // ifm.height=document.documentElement.clientHeight;
        }
    }

    //阻挡LayoutWindow的事件
    onDragStart = (layout, oldItem, newItem, placeholder, event, element) => {
        event.stopPropagation();
    }

    //TODO 改变layout后的回调
    onLayoutChange = (layout, layouts) => {
        let fromState = this.state.layouts;

        for(let j = 0; j< layout.length; j++) {
            let item = layout[j];
            for(let i = 0;i<fromState.length; i++) {
                let fitem = fromState[i];
                let iis = fitem.i.split(splitStr);
                if(item.i === fitem.i || (iis.length == 2 && iis[0] === item.i)) {
                    fitem.x = item.x;
                    fitem.y = item.y;
                    fitem.h = item.h;
                    fitem.w = item.w;
                    break;
                }
            }
            //TODO 序列化动态BirdGrid的content
        }
        this.setState({
            layouts: fromState
        });
        this.props.transferFromChild(fromState);
    }

    //删除下一级的卡片
    onDelCard = (key) => {
        let layouts = this.state.layouts;
        let index = -1;
        for(let i=0; i<layouts.length; i++) {
            let ii = layouts[i].i;
            let iis = ii.split(splitStr);

            if(layouts[i].i === key || (iis.length == 2 && iis[0] === key)) {
                index = i;
                break;
            }
        }
        if(index !== -1) {
            layouts.splice(index,1);
            this.setState({layouts:layouts});
        }
    }

    getExtra = () => {
        let closeDom = <Button type="danger" size="small" shape="circle" icon="close" onClick={this.props.onDel}></Button>;
        if(!this.props.active) {
            closeDom = <div></div>;
        }
        let textplaceholder = "内容配置";
        if(this.staticData.selectedItem === "动态消息栏") {
            textplaceholder = 
            `例如：{
                bars:[{
                    className:"qq",
                    id:"qq",
                    type:"qq",
                    count:"99+",
                    name:"官网",
                }],
                countUrl:"",
                sql:"" //查询的sql
            }
            `
        } else if(this.staticData.selectedItem === "动态门店销售情况") {
            textplaceholder = 
            `例如：{
                ranks:[{
                    title:"亮健1号店",
                    total: 12345
                }],
                rankUrl:"",
                sql:"", //查询的sql
            }
            `
        }
        let activeDom = (
            <div>
                <Button type="primary" onClick={this.showAddModal}>添加窗口</Button>
                <Modal
                    visible={this.state.visible}
                    onOk={this.onAddOk}
                    onCancel={this.onAddCancel} >
                    
                    {/* 添加组件的类型 */}
                    <Select style={{width:120}} defaultValue="请选择" onChange={this.selectKind}>
                        {this.state.selectKinds.map(item => {
                            return (<Select.Option key={item} value={item}>{item}</Select.Option>);
                        })}
                    </Select>
                    <Select defaultValue="请选择" onChange={this.selectItem} style={{width:"250px"}}>
                        {this.state.selectItems.map(item => {
                            let key = item.id;
                            item.key = item.id;
                            return (<Select.Option key={key} value={key}>{item.name}</Select.Option>);
                        })}
                    </Select>
                    <div>
                        {this.getCodeStyle()}
                        {this.state.showBirdGrid ? 
                            <Input id="birdgird_title" placeholder={"窗口名称"} defaultValue="" 
                            autoComplete={"off"}
                            onChange={(e) => {
                                this.staticData.birdGridTitle = e.target.value;
                            }}
                            onMouseDown={(e) => {
                                e.stopPropagation();
                                document.getElementById("birdgird_title").focus();
                            }}
                            // onMouseOver={() => {
                            //     document.getElementById("birdgird_title").focus();
                            // }}
                            /> : null}
                        {this.state.showBirdGrid ? 
                            // <Input.TextArea id="birdgird_option" placeholder={textplaceholder} defaultValue="" 
                            // onChange={(e) => {
                            //     this.staticData.birdGridOption = e.target.value
                            // }}
                            // onMouseDown={(e) => {
                            //     e.stopPropagation();
                            //     document.getElementById("birdgird_option").focus();
                            // }}
                            // // onMouseOver={() => {
                            // //     document.getElementById("birdgird_option").focus();
                            // // }} 
                            // rows={10} /> 
                            <Editor
                                id = "birdgird_option"
                                value = {this.state.birdgird_option_code}
                                placeholder={textplaceholder}
                                onValueChange={code => {
                                    this.setState({ birdgird_option_code: code }, () => {
                                        this.staticData.birdGridOption = code;
                                    });
                                }}
                                highlight={code => {
                                    let h = highlight(code, languages.js)
                                    return h;
                                }}
                                padding={10}
                                style={{
                                    fontFamily: '"Fira code", "Fira Mono", monospace',
                                    fontSize: 12,
                                    minHeight:"200px",
                                }}
                                onMouseDown={(e) => {
                                    e.stopPropagation();
                                    document.getElementById("birdgird_option").focus();
                                }}
                            />
                            : null } 
                            
                    </div>
                </Modal>
                {closeDom}
            </div>
        );
        let staticDom = <div></div>;
        if(this.props.active) {
            return activeDom;
        } else {
            return staticDom;
        }
    }

    showAddModal = () => {
        this.setState({visible:true});
    }

    //选择类型
    selectKind = (value) => {
        if(value === "普通组件") {
            let list = Object.keys(this.staticData.normalList).map((k) => {
                return {
                    name:k,
                    id:k
                };
            });
            this.setState({
                //TODO 暂时未定普通的组件结构
                selectItems: list,
                isNormal: true,
            });
        } else if (value === "图表组件") {
            jrequest({
                url:"/feature/features",
                method:"get",
                data:{page:1, size:1000}
            }).then((result) => {
                let list = result.content;
                
                this.setState({
                    selectItems: list,
                    isNormal: false,
                });
            });
        }
    }

    //选择组件
    selectItem = (value) => {
        if(value !== "动态BirdGrid" && value !== "动态门店销售情况" && value !== "动态消息栏") {
            let fromState = this.state.selectItems;
            for(let i=0;i<fromState.length; i++) {
                let item = fromState[i];
                if(item.id === value) {
                    this.staticData.selectedItem = item;
                    break;
                }
            }
            this.setState({showBirdGrid : false});
        } else {
            //显示那个配置动态BirdGrid的框
            this.setState({showBirdGrid : true});
            // this.staticData.selectedItem = "动态BirdGrid";
            this.staticData.selectedItem = value;
        }
    }
    
    //选择完成后添加
    onAddOk = () => {
        if(this.state.isNormal) {
            if(this.staticData.selectedItem !== "动态BirdGrid" && this.staticData.selectedItem !== "动态门店销售情况" && this.staticData.selectedItem !== "动态消息栏"){
                let layouts = this.state.layouts;
                let title = this.staticData.selectedItem.name;
                let content = this.staticData.selectedItem.content;
                let layout = {i: this.staticData.selectedItem.id,x: 0, y: 0, w: 2, h: 2, title:title, content: content};
                layouts.push(layout);
                this.setState({
                    layouts: layouts,
                    visible:false,
                });
            } else {
                let header = this.staticData.selectedItem === "动态BirdGrid" ? "" : this.staticData.selectedItem
                let layouts = this.state.layouts;
                let title = this.staticData.birdGridTitle;
                let optionStr = this.staticData.birdGridOption;
                let i = header + (new Date()).getTime() + splitStr + optionStr

                let evalOption;
                let evalOptionStr = "evalOption = " + optionStr;
                let evalSuccess = false;
                try {
                    eval(evalOptionStr);
                    evalSuccess = true;
                } catch (err) {
                    message.error("input error : " + err.message,15);
                    evalSuccess = false;
                }
                if (evalSuccess) {
                    let layout = {i: i,x: 0, y: 0, w: 2, h: 2, title:title};
                    layouts.push(layout);
                    this.setState({
                        layouts: layouts,
                        visible:false,
                        showBirdGrid:false,
                    });
                }
            }
        } else {
            let layouts = this.state.layouts;
            let title = this.staticData.selectedItem.name;
            let layout = {i: this.staticData.selectedItem.id, x: 0, y: 0, w: 2, h: 2, title:title, content: this.staticData.selectedItem.url, isIframe:true};
            layouts.push(layout);
            this.setState({
                layouts: layouts,
                visible:false,
            });
        }
    }

    onAddCancel = () => {
        this.setState({visible:false});
    }

    render() {
        let backgroundColor = this.props.active ? "#dddfe6" : "white";

        let titleOpt = (
            <div>
                <span style={{marginRight:"15px",float:"left"}}>{this.props.title}</span>
                {this.getExtra()}
            </div>
        );

        return (
            <Card 
                size="small"
                title={titleOpt}
                // title={this.props.title} 
                // extra={this.getExtra()} 
                style={{height:"100%", width:"100%", marginBottom:"5px"}}
                headStyle={{backgroundColor:"#fafafa"}}  
                bodyStyle={{height:"100%", width:"100%", padding:"0px",backgroundColor: backgroundColor}}>
                {this.renderLayouts()}
            </Card>
        );
    }

    //不知道为什么，导入css总是会失败，所以干脆就直接写style了
    getCodeStyle = () => {
        return (
            <style>{`
                html {
                    box-sizing: border-box;
                }
                
                *,
                *:before,
                *:after {
                    box-sizing: inherit;
                }
                
                html,
                body,
                #root {
                    height: 100%;
                    width: 100%;
                }
                
                body {
                    font-family: 'Fira code', 'Fira Mono', Consolas, Menlo, Courier, monospace;
                    font-size: 14px;
                    line-height: 1.5;
                    margin: 0;
                    padding: 0;
                }
                
                /* App */
                .container {
                    display: flex;
                    align-items: center;
                    justify-content: center;
                    height: 100%;
                    width: 100%;
                }
                
                .container__content {
                    width: 440px;
                    max-width: 100%;
                    padding: 10px;
                    text-align: center;
                }
                
                .container_editor_area {
                    tab-size: 4ch;
                    max-height: 400px;
                    overflow: auto;
                    margin: 1.67em 0;
                }
                
                .container__editor {
                    font-size: 12px;
                    font-variant-ligatures: common-ligatures;
                    background-color: #fafafa;
                    border-radius: 3px;
                }
                
                .container__editor textarea {
                    outline: 0;
                }
                
                .button {
                    display: inline-block;
                    padding: 0 6px;
                    text-decoration: none;
                    background: #000;
                    color: #fff;
                }
                
                .button:hover {
                    background: linear-gradient(45deg, #E42B66, #E2433F);
                }
                
                /* Syntax highlighting */
                .token.comment,
                .token.prolog,
                .token.doctype,
                .token.cdata {
                    color: #90a4ae;
                }
                .token.punctuation {
                    color: #9e9e9e;
                }
                .namespace {
                    opacity: 0.7;
                }
                .token.property,
                .token.tag,
                .token.boolean,
                .token.number,
                .token.constant,
                .token.symbol,
                .token.deleted {
                    color: #e91e63;
                }
                .token.selector,
                .token.attr-name,
                .token.string,
                .token.char,
                .token.builtin,
                .token.inserted {
                    color: #4caf50;
                }
                .token.operator,
                .token.entity,
                .token.url,
                .language-css .token.string,
                .style .token.string {
                    color: #795548;
                }
                .token.atrule,
                .token.attr-value,
                .token.keyword {
                    color: #3f51b5;
                }
                .token.function {
                    color: #f44336;
                }
                .token.regex,
                .token.important,
                .token.variable {
                    color: #ff9800;
                }
                .token.important,
                .token.bold {
                    font-weight: bold;
                }
                .token.italic {
                    font-style: italic;
                }
                .token.entity {
                    cursor: help;
                }
            `}</style>
        );
    }
}