import React from 'react';
import { Responsive, WidthProvider } from 'react-grid-layout';
import { Card, Button, Modal, Select, Input, Row, message } from 'antd';
import { jrequest } from 'utils/request';
import router from 'umi/router';
import '../../../node_modules/react-grid-layout/css/styles.css';
import "../../../node_modules/react-resizable/css/styles.css";
import LayoutBox from './LayoutBox';

const ResponsiveGridLayout = WidthProvider(Responsive);

export default class LayoutWindow extends React.Component {
    constructor(props){
        super(props);
        this.state = {
            visible:false,
            info: {},
            layouts: [],
        }
        this.staticData = {
            boxName : "",
        }

        let query = props.query ? props.query : "";
        jrequest({
            url:"/layout/layout"+query,
            method:"get"
        }).then((result) => {
            let layoutInfos = result.data.layoutInfo;
            this.setState({
                info: result.data,
                layouts: layoutInfos
            });
        });
    }

    //子布局在改变了布局后传递上来信息
    transferFromChild = (id,param) => {
        let fromState = this.state.layouts;
        for(let i=0;i<fromState.length; i++) {
            let fitem = fromState[i];
            if (id === fitem.i) {
                fitem.children = param;
                break;
            }
        }
        this.setState({
            layouts: fromState
        });
    }

    renderLayouts = () => {
        let layoutDoms = this.state.layouts.map((item) => {
            //是否可动
            item.static = !this.props.active;
            return (
                <div data-grid={item} key={item.i}>
                    <LayoutBox 
                        title={item.title} 
                        layouts={item.children ? item.children : [] } 
                        active={this.props.active}
                        onDel={() => this.onDelWindow(item.i)}
                        transferFromChild={(param) => this.transferFromChild(item.i, param)}/>
                </div>
            );
        });
        let gridLaout = (
            <ResponsiveGridLayout className="layout"
                rowHeight={50}
                // breakpoints={{lg: 1200, md: 996, sm: 768, xs: 480, xxs: 0}}
                // cols={{lg: 12, md: 10, sm: 6, xs: 4, xxs: 2}} 
                breakpoints={{lg: 1200}}
                cols={{lg: 12}} 
                onLayoutChange={this.onLayoutChange}>
            
                {layoutDoms}
            </ResponsiveGridLayout >
        );
        return gridLaout;
    }

    //改变layout后的回调, 暂时只用lg的
    onLayoutChange = (layout, layouts) => {

        let curr = layouts["lg"];
        let fromState = this.state.layouts;
        for(let j = 0; j<curr.length; j++) {
            let item = layout[j];
            for(let i = 0;i<fromState.length; i++) {
                let fitem = fromState[i];
                if(item.i === fitem.i) {
                    fitem.x = item.x;
                    fitem.y = item.y;
                    fitem.h = item.h;
                    fitem.w = item.w;
                    break;
                }
            }
        }
        this.setState({
            layouts: fromState
        });
    }

    getExtra = () => {
        let activeDom = (
            <div>
                <Button type="primary" onClick={this.showAddModal}>添加布局</Button>
                <Modal
                    visible={this.state.visible}
                    onOk={this.onAddOk}
                    onCancel={this.onAddCancel} >

                    <Input placeholder="输入窗口名称" onChange={(e) => this.staticData.boxName = e.target.value} />
                </Modal>
            </div>
        );
        let staticDom = <div></div>;
        if(this.props.active) {
            return activeDom;
        } else {
            return staticDom;
        }
    }

    //删除下一级的卡片
    onDelWindow = (key) => {
        let layouts = this.state.layouts;
        let index = -1;
        for(let i=0; i<layouts.length; i++) {
            if(layouts[i].i === key) {
                index = i;
                break;
            }
        }
        if(index !== -1) {
            layouts.splice(index,1);
            this.setState({layouts:layouts});
        }
    }

    showAddModal = () => {
        this.setState({visible:true});
    }

    //生成guid
    guid = () => {
        return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function(c) {
            var r = Math.random()*16|0, v = c == 'x' ? r : (r&0x3|0x8);
            return v.toString(16);
        });
    }
    
    //添加窗口
    onAddOk = () => {
        let layouts = this.state.layouts;
        let children = [];
        let boxName = this.staticData.boxName;
        let layout = {i: this.guid(), x: 0, y: 0, w: 10, h: 3, title:boxName, children:children };
        layouts.push(layout);
        this.setState({
            layouts: layouts,
            visible:false,
        });
    }

    onAddCancel = () => {
        this.setState({visible:false});
    }

    //保存布局
    saveAll = () => {
        let fromState = this.state.layouts;
        let info = this.state.info;
        info.layoutInfo = fromState;
        if(this.props.query) {
            info.name = this.props.query.replace("?name=","");
            console.log(info);
        }

        jrequest({
            url:"/layout/layout",
            method:"put",
            data:info
        }).then((result) => {
            if(result.success) {
                message.success("保存成功");
            } else {
                message.success("保存失败");
            }
        }).catch((error) => {
            message.error(error);
        })
    }

    render() {
        let buttonBar = (
            <Row gutter={8}>
                <Button type="primary" onClick={this.saveAll} style={{float:"left"}}>保存</Button>
                <Button type="primary" onClick={() => router.goBack()} style={{float:"right"}} >取消</Button>
            </Row>
        );
        if(!this.props.active) {
            buttonBar = <div></div>;
        }
        let backgroundColor = this.props.active ? "#c9d6de" : "white";

        let titleOpt = (
            <div>
                <span style={{marginRight:"15px",float:"left"}}>{this.props.title}</span>
                {this.getExtra()}
            </div>
        );
        return (
            <div>
                {buttonBar}
                <Row gutter={8} style={{marginBottom: "10px"}}>
                    <Card
                        title={titleOpt}
                        // title={this.props.title} 
                        size="small"
                        // extra={this.getExtra()} 
                        style={{height:"100%", width:"100%"}}  
                        headStyle={{backgroundColor:"#fafafa"}}
                        bodyStyle={{height:"100%", width:"100%", padding:"0px",backgroundColor: backgroundColor} }>
                            {this.renderLayouts()}
                    </Card>
                </Row>
            </div>
        );
    }
}