import React from "react";
import { Row, Col } from "antd";
import Panel from 'components/Panel';
import Icon from 'components/Icon';
import './../NormalComponent/index.less';
import ReactEcharts from 'echarts-for-react';

import {
    request,
    config,
    util,
    permission
} from 'utils';

const {
    APIV3
} = config;

export default class StaticChinaMap extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            opMemProv:{},
        }

        let opMemProv = {
            title: {
                text: '各省用户',
                subtext: '',
                left: 'center'
            },
            tooltip: {
                trigger: 'item'
            },
            legend: {
                orient: 'vertical',
                left: 'left',
                data: ['member']
            },
            visualMap: {
                min: 0,
                max: 1500000,
                left: 'left',
                top: 'bottom',
                text: ['高', '低'],       // 文本，默认为数值文本
                calculable: true
            },
            toolbox: {
                show: true,
                orient: 'vertical',
                left: 'right',
                top: 'center',
                feature: {
                    dataView: { readOnly: false },
                    restore: {},
                    saveAsImage: {}
                }
            },
            series: [
                {
                    name: 'member',
                    type: 'map',
                    mapType: 'china',
                    roam: false,
                    label: {
                        normal: {
                            show: true
                        },
                        emphasis: {
                            show: true
                        }
                    },
                    data: [
                        //静态假数据
                        { name: '北京', value: 0 },
                        { name: '天津', value: 0 },
                        { name: '上海', value: 0 },
                        { name: '重庆', value: 0 },
                        { name: '河北', value: 0 },
                        { name: '河南', value: 0 },
                        { name: '云南', value: 0 },
                        { name: '辽宁', value: 0 },
                        { name: '黑龙江', value: 0 },
                        { name: '湖南', value: 0 },
                        { name: '安徽', value: 0 },
                        { name: '山东', value: 0 },
                        { name: '新疆', value: 0 },
                        { name: '江苏', value: 0 },
                        { name: '浙江', value: 0 },
                        { name: '江西', value: 0 },
                        { name: '湖北', value: 0 },
                        { name: '广西', value: 0 },
                        { name: '甘肃', value: 0 },
                        { name: '山西', value: 0 },
                        { name: '内蒙古', value: 0 },
                        { name: '陕西', value: 0 },
                        { name: '吉林', value: 0 },
                        { name: '福建', value: 0 },
                        { name: '贵州', value: 0 },
                        { name: '广东', value: 0 },
                        { name: '青海', value: 0 },
                        { name: '西藏', value: 0 },
                        { name: '四川', value: 0 },
                        { name: '宁夏', value: 0 },
                        { name: '海南', value: 0 },
                        { name: '台湾', value: 0 },
                        { name: '香港', value: 0 },
                        { name: '澳门', value: 0 }
                    ]
                }]
        }
        //获取数据，修正数据
        request({
            url: APIV3 + '/web/value/memberProvince',
            method: "post"
        }).then((resp) => {
            let arr = [];
            for (var i = 0; i < resp.length; i++) {
                arr.push({
                    name: resp[i][0],
                    value: resp[i][1]
                });
            }
            opMemProv.series[0].data = arr.sort(function (a, b) {
                return a.value - b.value;
            })
            opMemProv.visualMap.max = opMemProv.series[0].data[arr.length - 1].value;
            this.setState({
                opMemProv: opMemProv
            },() => console.log("赋值了"));
        });
    }

    render() {
        return (
            <ReactEcharts
                option={this.state.opMemProv}
                style={{height: '500px', width: '100%'}}
                className='react_for_echarts' />
        );
    }
}