import React from 'react';
import { jrequest } from 'utils/request';
import { Input, Modal, Button, Select, Collapse,Row, Col, message, Table } from 'antd';

const { TextArea } = Input;

class EditUpdateLog extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            visible: false,
            confirmLoading:false,

            infoId: props.infoId,
            tableData: [],
            total:0,
            current:1,
            pageSize:5,
        }
        this.staticData = {
            version:"",
            desc:""
        }

        jrequest({
            url:"/feature/log",
            method:"get",
            data:{
                infoId:this.state.infoId,
                page:this.state.current,
                size:this.state.pageSize
            }
        }).then((result) => {
            this.setState({
                tableData: result.content,
                total: result.totalElements,
            });
        }).catch((error) => {

        })
    }

    componentWillReceiveProps(props) {
        this.setState({infoId:props.infoId});
        this.staticData = {
            version:"",
            desc:""
        }
    }

    getColumnDef = () => {
        let columnDef = [{
            title:"版本号",
            dataIndex:"version",
            key:"version",
        },{
            title:"更新内容",
            dataIndex:"updateDesc",
            key:"updateDesc"
        },{
            title:"更新时间",
            dataIndex:"updateDate",
            key:"updateDate"
        },{
            title:"操作人",
            dataIndex:"operator",
            key:"operator"
        }];
        return columnDef;
    }

    changePage = (current) => {
        jrequest({
            url:"/feature/log",
            method:"get",
            data:{
                infoId:this.state.infoId,
                page:current,
                size:this.state.pageSize
            }
        }).then((result) => {
            this.setState({
                tableData: result.content,
                total: result.totalElements,
                current: current
            });
        }).catch((error) => {

        })
    }

    changePageSize = (pageSize,current) => {
        this.setState({pageSize: pageSize}, 
            () => this.refresh());
    }

    refresh = () => {
        jrequest({
            url:"/feature/log",
            method:"get",
            data:{
                infoId:this.state.infoId,
                page:this.state.current,
                size:this.state.pageSize
            }
        }).then((result) => {
            this.setState({
                tableData: result.content,
                total: result.totalElements,
            });
        }).catch((error) => {

        })
    }

    getpaginationProps = () => {
        let paginationProps = {
            showSizeChanger: true,
            showQuickJumper: false,
            showTotal: () => `共${this.state.total}条`,
            pageSize: this.state.pageSize,
            pageSizeOptions: ["5","10"],
            current: this.state.current,
            total: this.state.total,
            size: "small",
            onChange: (current) => this.changePage(current),
            onShowSizeChange: (curr,pageSize) => this.changePageSize(pageSize,curr),
        }
        return paginationProps;
    }

    //显示modal框
    showModal = () => {
        jrequest({
            url:"/feature/log",
            method:"get",
            data:{
                infoId:this.state.infoId,
                page:this.state.current,
                size:this.state.pageSize
            }
        }).then((result) => {
            this.setState({
                tableData: result.content,
                total: result.totalElements,
            });
        }).catch((error) => {

        })

        this.setState({
            visible: true,
        });
    }

    handleOk = () => {
        let objectObject = {
            infoId: this.state.infoId,
            version: this.staticData.version,
            updateDesc: this.staticData.desc
        }
        this.setState({confirmLoading:true});
        jrequest({
            url:"/feature/log",
            method:"post",
            data:objectObject
        }).then((result) => {
            if(result.success) {
                 //跟新后的回调
                 this.props.afterOk();
                 this.setState({confirmLoading:false,visible:false});
            }else {
                message.error(result.msg);
                this.setState({confirmLoading:false});
            }
        }).catch((error) => {
            this.setState({confirmLoading:false});
        });
    }

    handleCancel = () => {
        this.setState({
            visible: false,
          });
    }

    render() {
        return (
            <div>
                <Button style={{float:"left"}} type="primary" onClick={this.showModal}>更新说明</Button>
                <Modal
                    title="更新说明"
                    visible={this.state.visible}
                    onOk={this.handleOk}
                    okText="发布"
                    width="800px"
                    confirmLoading={this.state.confirmLoading}
                    onCancel={this.handleCancel}>
                        <Row gutter={8} style={{marginBottom: "30px"}}>
                            <Input placeholder="版本号：您只需要填写大的版本号，小版本号系统会根据日期自动补充。如您填写1.2，发布后，版本号为V1.2.190110" 
                                onChange={(e) => this.staticData.version = e.target.value} />
                        </Row>
                        <Row gutter={8} style={{marginBottom: "30px"}}>
                            <TextArea placeholder="更新内容" onChange={(e) => this.staticData.desc = e.target.value} />
                        </Row>
                        <Row gutter={8} style={{marginBottom: "30px"}}>
                            <Table 
                                bordered
                                rowKey="id"
                                pagination={ this.getpaginationProps() }
                                columns={ this.getColumnDef() }
                                dataSource={this.state.tableData}
                                 />
                        </Row>
                </Modal>
            </div>
        );
    }
}
export default EditUpdateLog