import React from 'react'
import moment from 'moment';
import PropTypes from 'prop-types';
import { util,arrayToHash,request } from 'utils';
import BirdSelector from './BirdSelector';
import BirdCascader from './BirdCascader';
import BirdMulti from './BirdMulti';
import BraftEditor from './BraftEditor';
import BirdUpload from '../File/BirdUpload';

import { DropdownRender, SwitchRender, DateTimeRender, MultiRender, ImageRender, FileRender, MoneyRender, InputRender,Link } from '../Grid/render';

import { Form, Input, DatePicker, Switch, Icon, InputNumber, Tooltip } from 'antd';
import axios from 'axios'

const FormItem = Form.Item;
const TextArea = Input.TextArea;

class AutoField extends React.Component {
  constructor(props) {
    super(props);
    let inline = !this.props.inline?false:true;
    let showText = !this.props.showText?false:true;
    let sourceKeyValue = this.props.sourceKeyValue?this.props.sourceKeyValue:{};//显示文本用
    this.state = {
      inline:inline,
      showText:showText,
      sourceKeyValue:sourceKeyValue
    }
  }
  
  componentWillReceiveProps(nextProps) {
    if (nextProps.showText != this.props.showText ) {
      let showText = nextProps.showText
      this.setState({
        showText: showText
      })
    }
    if (!util.object.equal(nextProps.sourceKeyValue,this.props.sourceKeyValue) ) {
      let sourceKeyValue = nextProps.sourceKeyValue
      this.setState({
        sourceKeyValue: sourceKeyValue
      })
    }
  }

  onChange(value) {
    let fieldOption = this.props.fieldOption;
    let dataKey = fieldOption.key;
    this.props.onChange && this.props.onChange(dataKey, value);    
  }
  
  onClick(value) {
    let fieldOption = this.props.fieldOption;
    let dataKey = fieldOption.key;
    this.props.onClick && this.props.onClick(dataKey, value);    
  }

  getValueTag(field,fieldData) {    
    let self = this;
    if (typeof (field.value) === 'undefined' || field.value === 'null') {
      field.value = '';
    }
    if (field.render && typeof (field.render) === 'function') return field.render(field.value);

    let innerProps = field.innerProps || {};

    if(self.state.showText){
      
      let rowDataMap = {};
      let col = field;
      {
        if (col.fieldType === 'command') ;
        let formatValue;
        let title = "";//self.state.keyTitleMap[col.key];
        let value = col.value;


        if (col.render) {
          formatValue = col.render(value, data)
        } else {
          if (col.fieldType === 'switch') formatValue = SwitchRender(value);
          else if (col.fieldType === 'dropdown' || col.type === 'cascader') formatValue = DropdownRender(value, this.state.sourceKeyValue);
          else if (col.fieldType === 'multi') formatValue = MultiRender(value, this.state.sourceKeyValue);
          else if (col.fieldType === 'date') formatValue = DateTimeRender(value, 'yyyy-MM-dd');
          else if (col.fieldType === 'datetime') formatValue = DateTimeRender(value, 'yyyy-MM-dd HH:mm:ss');
          else if (col.fieldType === 'img' || col.type === 'imgs') formatValue = ImageRender(value);
          else if (col.fieldType === 'file' || col.type === 'files') formatValue = FileRender(value);
          else if (col.fieldType === 'money') formatValue = MoneyRender(value);
          else if (col.fieldType === 'input' ) {formatValue = InputRender(value,data,col.key);}
          else if (col.fieldType === 'link' ) {formatValue = Link(value);}
          else formatValue = typeof (value) === 'undefined' ? '' : value;
        }
        rowDataMap[col.key] = { title, value, formatValue };
      }
      return <span>{rowDataMap[col.key].formatValue}</span>
    }else{
      switch (field.fieldType) {
        case "text":
          return <Input {...{
            value: field.value,
            onChange: e => self.onChange(e.target.value),
            onClick: e => self.onClick(e.target.value),
            disabled: field.disabled,
            placeholder: field.placeholder,
            ...innerProps
          }} />;
        case "textarea":
          return <TextArea {...{
            value: field.value,
            onChange: e => self.onChange(e.target.value),
            onClick: e => self.onClick(e.target.value),
            disabled: field.disabled,
            placeholder: field.placeholder,
            autosize: { minRows: 4, maxRows: 8 },
            ...innerProps
          }} />
        case "number":
          return <InputNumber {...{
            value: field.value || 0,
            onChange: value => self.onChange(util.string.isEmpty(value) ? 0 : value),
            onClick: value => self.onClick(util.string.isEmpty(value) ? 0 : value),
            disabled: field.disabled,
            placeholder: field.placeholder,
            min: 0,
            step: field.step || 1,
            precision: field.precision || 0,
            style: { width: '100%' },
            ...innerProps
          }} />;
        case "money":
          return <InputNumber {...{
            value: field.value || 0,
            onChange: value => self.onChange(util.string.isEmpty(value) ? 0 : value),
            onClick: value => self.onClick(util.string.isEmpty(value) ? 0 : value),
            disabled: field.disabled,
            placeholder: field.placeholder,
            min: 0,
            step: field.step || 1,
            precision: field.precision || 2,
            style: { width: '100%' },
            formatter: value => `${value}`.replace(/\B(?=(\d{3})+(?!\d))/g, ','),
            parser: value => value.replace(/\$\s?|(,*)/g, ''),
            ...innerProps
          }} />;
        case "switch":
          return <Switch {...{
            checked: field.value + '' === "1" || field.value + '' === 'true',
            onChange: value => self.onChange(value ? "1" : "0"),
            onClick: value => self.onClick(value ? "1" : "0"),
            disabled: field.disabled,
            checkedChildren: <Icon type="check" />,
            unCheckedChildren: <Icon type="cross" />,
            ...innerProps
          }} />;
        case "dropdown":
          let sourcedata = field.source.data;
          if(field.source.params&&field.source.params.length>0){
            //if(!sourcedata||sourcedata.length>0)
            sourcedata = [];
            //else{
            /*if (field.source.url) {
              let urlstr = field.source.url+"";
              if(field.source.params&&field.source.params.length>0){
                  urlstr += "?";
                  for(var param in field.source.params){
                    urlstr += field.source.params[param].key+"=";
                    for(var paramData in fieldData){
                      if(fieldData[paramData].key == field.source.params[param].key){
                        urlstr += fieldData[paramData].value+
                        "&";
                      }
                    }
                  }
              }
              sourcedata = request({
                url: urlstr,
                method: "get"
              }).then(function (result) {
                return result;
              })
              //.then(axios.spread(function (acct, perms) {
              //  console.log('请求都完成了')
              //}));
            };*/
          //}
          }

          return <BirdSelector {...{
            selectedValue: field.value,
            onChange: value => self.onChange(value),
            onClick: value => self.onClick(value ? "1" : "0"),
            disabled: field.disabled,
            data: sourcedata || [],
            url: field.source.url,
            params: field.source.params,
            dicKey: field.source.key,
            innerProps: innerProps,
            paramsData:fieldData,
            inline:this.state.inline,
            width:field.width
          }} />;
        
        case "multi":
          return <BirdMulti {...{
            selectedValue: field.value,
            onChange: value => self.onChange(value),
            onClick: value => self.onClick(value),
            disabled: field.disabled,
            options: field.source.data || [],
            url: field.source.url,
            dicKey: field.source.key,
            innerProps: innerProps,
            paramsData:fieldData,
            inline:this.state.inline,
            width:field.width
          }} />

        case "cascader":
          return <BirdCascader {...{
            value: field.value,
            onChange: value => self.onChange(value),
            onClick: value => self.onClick(value),
            disabled: field.disabled,
            data: field.source.data || [],
            url: field.source.url,
            placeholder: field.placeholder,
            innerProps: innerProps,
            paramsData:fieldData,
            inline:this.state.inline,
            width:field.width
          }} />
        case "img":
        case "imgs":
        case "file":
        case "files":
          let fileProps = {
            multiple: field.fieldType === 'imgs' || field.fieldType === 'files',
            disabled: field.disabled,
            listType: field.fieldType === 'img' || field.fieldType === 'imgs' ? 'picture' : 'text',
            value: field.value,
            onChange: value => self.onChange(value),
            onClick: value => self.onClick(value),
          };       
          if (field.fieldType === 'img' || field.fieldType === 'imgs') {
            fileProps.accept = "image/png,image/jpeg,image/jpg,image/gif,image/bmp";
          }

          return <BirdUpload {...fileProps} />;
        case "date":
          return <DatePicker {...{
            value: field.value ? moment(field.value) : null,
            onChange: (date, dateString) => self.onChange(dateString),
            onClick: (date, dateString) => self.onClick(dateString),
            disabled: field.disabled,
            format: "YYYY-MM-DD",
            style: { width: '100%' },
            ...innerProps
          }} />;
        case "datetime":
          return <DatePicker {...{
            value: field.value ? moment(field.value) : null,
            onChange: (date, dateString) => self.onChange(dateString),
            onClick: (date, dateString) => self.onClick(dateString),
            disabled: field.disabled,
            showTime: true,
            format: "YYYY-MM-DD HH:mm:ss",
            style: { width: '100%' },
            ...innerProps
          }} />;
        case "richtext":
          return <BraftEditor {...{
            initValue: field.value,
            contentId: field.key,
            onChange: value => self.onChange(value),
            onClick: value => self.onClick(value),
            innerProps: innerProps
          }} />;
        case "input":
          return <InputNumber {...{
            defaultValue:field.value,
            contentId: field.key,
            onChange: value=>self.onChange(value),
            onClick: value=>self.onClick(value)
          }} />
        default:
          return <span />;
      }
    }
  };


  render() {
    let fieldOption = this.props.fieldOption;
    let fieldData = this.props.fieldData;
    let formItemLayout = {
      labelCol: { span: this.props.labelColSpan },
      wrapperCol: { span: 20 - this.props.labelColSpan }
    };

    return <FormItem {...formItemLayout} label={
      <span>
        {(fieldOption.isRequired || fieldOption.required) && <span style={{ color: '#f46e65', marginRight: 4 }}>*</span>}
        {fieldOption.name}
        {fieldOption.tips && <span style={{ marginLeft: 5, fontStyle: 'normal', color: 'rgba(0, 0, 0, 0.45)' }}>
          <Tooltip title={fieldOption.tips}>
            <Icon type="question-circle-o" style={{ marginRight: 4 }} />
          </Tooltip>
        </span>}
      </span>
    }
    >
      {this.getValueTag(fieldOption,fieldData)}
    </FormItem>;
  }
}

AutoField.propTypes = {
  fieldOption: PropTypes.object.isRequired,
  fieldData: PropTypes.array,
  labelColSpan: PropTypes.number,
  onChange: PropTypes.func,
  onClick: PropTypes.func,
  initOption:PropTypes.func,
  inline:PropTypes.bool,
  showText: PropTypes.bool,
}

AutoField.defaultProps = {
  labelColSpan: 6
}

export default AutoField
