import React,{ PureComponent} from 'react';
import PropTypes from 'prop-types';
import AutoForm from './BirdGridForm';
import BirdGridFilter from './BirdGridFilter';
import { request, config, util, permission,deepClone,  arrayToHash,hashToArray } from 'utils';
import { arequest } from 'utils/request';
import styles from './BirdGrid.less';
import BirdButton from '../Form/BirdButton';
import { DropdownRender, SwitchRender, DateTimeRender, MultiRender, ImageRender, FileRender, MoneyRender, InputRender,Link } from './render';
import { Pagination, Modal, Card, Popconfirm, message, Row, Col, Checkbox, Button, Divider, Spin, Popover } from 'antd';
import AutoField from '../Form/AutoField';
import { cpus } from 'os';

const sourceTypes = ['dropdown', 'multi', 'cascader'];

class BirdGrid extends React.Component {
  constructor(props) {
    super(props);
    
    let gridOption = this.props.gridOption;
    let primaryKey = gridOption.primaryKey || gridOption.columns[0]['data'];
    let deleteKey = gridOption.deleteKey || gridOption.columns[0]['data'];
    let dataSource = gridOption.dataSource || [];
    let autoQuery = gridOption.autoQuery;
    if (typeof (autoQuery) === 'undefined') {
      autoQuery = true;
    }

    this.state = {
      columns: [],
      queryColumns: [],
      sumFields: [],
      actions: [],
      pageIndex: 1,
      pageSize: gridOption.pageSize || 15,
      queryLoading: false,
      sortField: gridOption.sortField || primaryKey,
      sortDirection: gridOption.sortDirection || "desc",
      filterRules: gridOption.filterRules || [],
      gridDatas: {
        totalCount: dataSource.length,
        items: dataSource,
        sum: {}
      },
      formVisiable: false,
      formConfirmLoading: false,
      formWidth: gridOption.formWidth || 520,
      formOption: {
        saveUrl: "",
        fields: [],
        value: {}
      },
      customData: gridOption.customRules || [],
      keyTitleMap: {},
      sourceKeyMap: {},//dropdown与cascader类型key与data的hash映射

      primaryKey: primaryKey,//标识列名称
      deleteKey: deleteKey,//标识列名称
      tablePermission: {},

      checkedValues: [],
      autoQuery: autoQuery, //页面渲染完成之后是否自动查询，默认为true

      filterOption: gridOption.filterOption || {}
    };

    /* <MyInput onChange={this.update.bind(this)} />;
    和<MyInput onChange={e => this.props.update(e.target.value)} />;
    都会创建新的函数props 所以因提前绑定this到相应函数  */

    this.filterChange.bind(this);
  }
  /* 性能优化 props改变才渲染 */
  // shouldComponentUpdate(nextProps,nextState){
  //   if(util.object.equal(nextProps, this.props)&&
  //   util.object.equal(nextState, this.state)){ //传入的count与组件当前props的count比较,count没改变，return false，不渲染
  //     return false    //不渲染
  //   }
  //   return true;  //渲染
  // }

  /* 初始化渲染执行之前执行 */
  componentDidMount() {
    if(this.props.onRef)
    this.props.onRef(this)//传递refs

    let gridOption = this.props.gridOption;
    //初始化权限
    let p = gridOption.permission;
    let tp = {};
    if (typeof (p) === 'string') {
      tp = {
        add: p + ':add',
        edit: p + ':edit',
        delete: p + ':delete'
      }
    } else {
      tp = p || {};
    } 

    let url = gridOption.url;
    let columns = [], queryColumns = [], sumFields = [];
    let keyTitleMap = {}, sourceKeyMap = {};
    for (let col of gridOption.columns) {
      //根据权限初始化行内按钮，没有按钮时移除command列
      if (col.type === 'command') {
        let tdActions = [];
        if (url && url.edit && permission.check(tp.edit)) {
          tdActions.unshift({ name: (url.edit_title?url.edit_title:'编辑'), onClick: (data,that,indextr) => this.editClick(data,that,indextr) });
        }
        if (url && url.edit1 && permission.check(tp.edit1)) {
          tdActions.unshift({ name: (url.edit1_title?url.edit1_title:'编辑'), onClick: (data,that,indextr) => this.edit1Click(data,that,indextr) });
        }
        let actions = col.actions || [];
        for (let action of actions) {
          if (permission.check(action.permissionName)) {
            tdActions.push(action);
          }
        }
        if (url && url.delete && permission.check(tp.delete)) {
          tdActions.push({ name: '删除', color: '#f78989', onClick: data => this.deleteClick(data[this.state.deleteKey?this.state.deleteKey:this.state.primaryKey]), confirm: true })
        }
        if (tdActions.length === 0) continue;
        col.actions = tdActions;
      } else {
        keyTitleMap[col.data] = col.title;
        if (col.type === 'richtext' && this.state.formWidth === 520) {
          this.setState({ formWidth: 800 });
        }
        if (col.query) {
          queryColumns.push(col);
        }
        if ((col.type === 'number' || col.type === 'money') && col.sum) {
          sumFields.push(col.data);
        }
        //初始化下拉选择框的数据源,优先级：data>url>key
        /*if (sourceTypes.includes(col.type) && col.source) {
          if (col.source.data && col.source.data.length > 0) {
            sourceKeyMap[col.data] = arrayToHash(col.source.data);
          } else if (col.source.url) {
            let urlstr = col.source.url+"";
            if(col.source.params&&col.source.params.length>0){
              urlstr += "?";
              for(var param in col.source.params){
                urlstr += col.source.params[param].key+"="+"&";
              }
            }
            request({ url: urlstr, method: "get" })
              .then(result => {
                col.source.data = result;
                sourceKeyMap[col.data] = arrayToHash(col.source.data);
                this.setState({ sourceKeyMap: sourceKeyMap });
              });
          } else if (col.source.key && (col.type === 'dropdown' || col.type === 'multi')) {
            request({ url: config.api.getDic + col.source.key, method: "get" })
              .then(result => {
                col.source.data = result.options;
                sourceKeyMap[col.data] = arrayToHash(col.source.data);
                this.setState({ sourceKeyMap: sourceKeyMap });
              });
          } else {
            sourceKeyMap[col.data] = {};
          }
        }*/
        //money类型的列默认向右对齐
        if (col.type === 'money') {
          col.align = col.align || 'right';
        }
      }

      //初始化列的隐藏层级：no、user、dev
      if (col.hide === 'user') { }
      else if (col.hide && col.hide !== 'no') { col.hide = 'dev'; }
      else { col.hide = 'no' }
      columns.push(col);

      this.setState({ filterOption:gridOption.filterOption})
    }

    //初始化查询条件
    let filterRules = this.state.filterRules;
    if (filterRules.length === 0) {
      filterRules.push({
        field: queryColumns.length > 0 ? queryColumns[0]['data'] : '',
        operate: 'equal',
        value: ''
      });
    }

    //初始化顶部按钮
    let tableActions = [];
    let optionActions = gridOption.actions || [];
    for (let action of optionActions) {
      if (permission.check(action.permissionName)) {
        tableActions.push(action);
      }
    }
    if (url && url.add && permission.check(tp.add)) {
      tableActions.push({ name: "新增", icon: "plus", onClick: () => this.addClick() });
    }

    this.setState({
      columns: columns,
      queryColumns: queryColumns,
      sumFields: sumFields,
      actions: tableActions,
      keyTitleMap: keyTitleMap,
      sourceKeyMap: sourceKeyMap,
      tablePermission: tp,
      filterRules: filterRules
    }, () => {
      this.state.autoQuery && this.query()
    });
  }

  componentWillReceiveProps(nextProps) {
    if (!(this.props.url && this.props.url.read) 
    && !util.object.equal(nextProps.gridOption.dataSource, this.props.gridOption.dataSource)
    ) {
      this.setState({ pageIndex: 1 }, () => this.localQuery(nextProps))
    }
    if ( !util.object.equal(nextProps.gridOption.filterRules, this.props.gridOption.filterRules)) {
      this.setState({ pageIndex: 1 ,filterRules:nextProps.gridOption.filterRules}, () => this.query())
    }
    if ( !util.object.equal(nextProps.gridOption.filterOption, this.props.gridOption.filterOption)) {
      this.setState({ filterOption:nextProps.gridOption.filterOption})
    }
  }

  /* 分页点击事件 */
  pageClick(pageIndex) {
    this.setState({
      pageIndex: pageIndex
    }, this.query);
  }

  /* 每页显示数量改变事件 */
  pageSizeChange(pageSize) {
    this.setState({
      pageIndex: 1,
      pageSize: pageSize
    }, this.query);
  }

  /* 新增点击事件 */
  addClick() {
    let formOption = {
      model: "add",
      saveUrl: this.props.gridOption.url.add,
      fields: this.state.columns,
      value: {},
      extraParams: this.state.customData,
      sourceData:this.state.sourceKeyMap
    };
    this.setState({
      formVisiable: true,
      formOption: formOption
    });
  }

  /* 编辑点击事件 */
  editClick(data,that,indextr) {
    let formOption = {
      model: "update",
      saveUrl: this.props.gridOption.url.edit,
      fields: this.state.columns,
      value: data,
      extraParams: this.state.customData,
      sourceData:this.state.sourceKeyMap,
      indextr:indextr
    };
    this.setState({
      formVisiable: true,
      formOption: formOption
    });
  }

  /* 编辑点击事件1 */
  edit1Click(data,that,indextr) {
    let formOption = {
      model: "update",
      saveUrl: this.props.gridOption.url.edit1,
      fields: this.state.columns,
      value: data,
      extraParams: this.state.customData,
      sourceData:this.state.sourceKeyMap,
      indextr:indextr
    };
    this.setState({
      formVisiable: true,
      formOption: formOption
    });
  }

  /* 删除点击事件 */
  deleteClick(id) {
    //这里直接加一个?id就太死了，但是我也好像没啥优雅的办法，就在前面判断一下再加
    let url = this.props.gridOption.url.delete;
    if(url.indexOf("/japi/birdgird/delete") !== -1) {
      url = url + "&id=" + id;
    } else {
      url = url + "?id=" + id;
    }
    request({
      url: url,
      method: "post"
    }).then((data) => {
      if(data.success)
      message.success('删除成功');
      else
        message.success(data.msg);
      this.query();
    });
  }

  /* 表单保存 */
  saveClick() {
    if (!this.refs.autoForm.validate()) return;

    this.setState({ formConfirmLoading: true });

    this.refs.autoForm.saveClick(() => {
      this.setState({
        formVisiable: false,
        formConfirmLoading: false
      });
      this.query();
      this.props.gridOption.afterSave && this.props.gridOption.afterSave()
    });
  }

  /* 弹出框取消事件 */
  cancelClick() {
    this.setState({
      formVisiable: false,
      formConfirmLoading: false
    });
  }

  /* 排序点击事件 */
  sortClick(col) {
    if (col['sortDisable'] || col.type === "command") return;
    let sortField = col['data'];
    let sortDirection = sortField !== this.state.sortField ? "asc" : this.state.sortDirection === "asc" ? "desc" : "asc";

    this.setState({
      sortField: sortField,
      sortDirection: sortDirection
    }, this.query);
  }

  /* 全选点击事件 */
  checkAllClick() {
    var self = this;
    var checkedValues = [];
    
    if(self.props.gridOption && self.props.gridOption.disableFun 
      &&typeof(self.props.gridOption.disableFun) == 'function'){
      let checkValues = this.state.gridDatas.items.map(p => p[this.state.primaryKey]);
      let checkValues1 = [];
      for(var i in this.state.gridDatas.items){
        if(!self.props.gridOption.disableFun(this.state.gridDatas.items[i])){
          checkValues1.push(checkValues[i]);
        }
      }
      if(this.state.checkedValues.length  ==  checkValues1.length)
        checkedValues = [];
        else
        checkedValues = checkValues1;
    }else if (
      this.state.checkedValues.length === this.state.gridDatas.items.length) {
      checkedValues = [];
    } else {
      let checkValues = this.state.gridDatas.items.map(p => p[this.state.primaryKey]);
      checkedValues = checkValues;
    }
    //剔除disabled的选项
    this.setState({ checkedValues: checkedValues,availableItems:checkedValues })
  }

  
  /* 全选（不按分页）点击事件 */
  checkedAllIClick() {
    if (this.state.checkedAllI == true) {
      this.setState({ checkedAllI: false })
    } else {
      this.setState({ checkedAllI: true })
    }
  }

  /* 选择框点击事件 */
  checkClick(value) {
    let checkValues = this.state.checkedValues;
    let newCheckValues= checkValues;

    let index = newCheckValues.findIndex(p => p === value);
    if (index < 0) {
      newCheckValues.push(value);
    } else {
      newCheckValues.splice(index, 1);
    }
    this.setState({ checkedValues: newCheckValues })
    this.props.gridOption.afterCheck && this.props.gridOption.afterCheck(this.state.gridDatas,
      newCheckValues)
  }

  /* 获取选择的值*/
  getCheckValues() {
    return this.state.checkedValues;
  }

  /* 数据查询 */
  query() {
    let url = this.props.gridOption.url;
    let self = this;
    var beforeResult = self.props.gridOption.beforeQuery 
    && self.props.gridOption.beforeQuery(self.state.gridDatas, 
      self.getFilters(),self.state.pageIndex,self.state.pageSize);
    if(self.props.gridOption.beforeQuery&&!beforeResult) return false;

    if (!url || !url.read) { this.localQuery(this.props) }
    else {
      this.setState({ queryLoading: true });
      let dto = {
        pageIndex: this.state.pageIndex,
        pageSize: this.state.pageSize,
        sortField: this.state.sortField,
        sortDirection: this.state.sortDirection === "asc" ? 0 : 1,
        filters: this.getFilters(),
        sumFields: this.state.sumFields
      };
      
      //将查询的东西放入localstorge，给后续的导出使用
      let dtoStr = JSON.stringify(dto);
      sessionStorage.setItem("queryDto", dtoStr);

      request({
        url: url.read,
        method: "post",
        data: dto
      }).then(result => {
        if (result === null || typeof (result) === "undefined") {
          result = { totalCount: 0, items: [] };
        }
        if(typeof (result.timeout) != "undefined" && result.timeout===1){
          util.auth.removeToken();
          permission.clear();
          window.location.href = "/";
        }
        let resultout = result;
        //self.setState({gridDatas: resultout});
                
        let cv=[];
        if(result&&result.items&&result.items[0]&&typeof(result.items[0].checkedValue)!='undefined'){ 
        
          result.items.forEach((item)=>{
            //console.log(item.checkedValue)
            if(item.checkedValue!='0'
            &&item.checkedValue!='null'&&item.checkedValue!=null){
              cv.push(item.checkedValue)
            }
          });
          //self.setState({checkedValues:cv})
        }       


        (async ()=>{
        let sourceKeyMap = {};
        let hasSource = false;
        for (let col of self.props.gridOption.columns) {
          //初始化下拉选择框的数据源,优先级：data>url>key
          if (sourceTypes.includes(col.type) && col.source) {
            hasSource = true;
              
            let changedSource = false;
            for (let stateCol of self.state.columns) {
              if(stateCol.data == col.data && stateCol.source.url != col.source.url){
                changedSource  = true;
              }
            }
            if (col.source.data && col.source.data.length > 0
              &&!changedSource) {
              sourceKeyMap[col.data] = arrayToHash(col.source.data);
              // self.setState(
              //   {sourceKeyMap: sourceKeyMap}
              // );
            } else if (col.source.url) {
              let urlstr = col.source.url+"";
              let startStr = (urlstr.indexOf("?")>0)?"&":"?";

              let qurystra = [];
              if(col.source.params&&col.source.params.length>0){
                for (let griditem of resultout.items) {
                  let pramstring = "";
                  let qurystr = "";
                  for(var param in col.source.params){
                    if(griditem[col.source.params[param].key]){
                      pramstring = griditem[col.source.params[param].key];
                    }
                    qurystr += col.source.params[param].key+"="+pramstring+"&";
                  }
                  qurystra[qurystr.replace("&","_").replace("=","-")] = qurystr;
                }
                  
                for(let j in qurystra){
                  let urlquery = urlstr + startStr + qurystra[j];
                  let result = await arequest({ url: urlquery, method: "get" });
                  col.source.data = result;
                      if(result){
                        if(sourceKeyMap[col.data] == undefined)
                          sourceKeyMap[col.data] = [];
                        sourceKeyMap[col.data][j] = arrayToHash(col.source.data);
                      }
                      // self.setState({  sourceKeyMap: sourceKeyMap  });
                      // self.props.gridOption.afterQuery && self.props.gridOption.afterQuery(resultout, self.getFilters());
                }
              }else{
                let urlquery = urlstr;
                let result = await arequest({ url: urlquery, method: "get" });
                for (let stateCol of self.state.queryColumns) {
                  if(stateCol.data == col.data){
                    stateCol.source.data = result;
                  }
                }
                col.source.data = result;
                if(result){
                  sourceKeyMap[col.data] = [];
                  sourceKeyMap[col.data] = arrayToHash(col.source.data);
                }
                // self.setState({  sourceKeyMap: sourceKeyMap  });
                // self.props.gridOption.afterQuery && self.props.gridOption.afterQuery(resultout, self.getFilters());
              }

            } else if (col.source.key && (col.type === 'dropdown' || col.type === 'multi')) {
              let result = await arequest({ url: config.api.getDic + col.source.key, method: "get" });
              for (let stateCol of self.state.queryColumns) {
                if(stateCol.data == col.data){
                  stateCol.source.data = result;
                }
              }
              col.source.data = result.options;
              sourceKeyMap[col.data] = arrayToHash(col.source.data);
              // self.setState({  sourceKeyMap: sourceKeyMap });
              // self.props.gridOption.afterQuery && self.props.gridOption.afterQuery(resultout, self.getFilters());
            } else {
              sourceKeyMap[col.data] = {};
            }
          }
        }
          
        //let sourceKeyArray = hashToArray(sourceKeyMap);
        //if(!hasSource || sourceKeyArray.length != 0){
          self.setState({  sourceKeyMap: sourceKeyMap ,
            gridDatas: resultout,
            checkedValues:cv,
            queryLoading: false
          },()=>{
            self.props.gridOption.afterQuery && self.props.gridOption.afterQuery(result, self.getFilters());
          });
          
        //}
        })();
        //this.setState({ queryLoading: false });
      //end
      /*self.setState(
        {sourceKeyMap: sourceKeyMap}
      );*/
      });

    }
  }

  /* 数据查询(本地) */
  localQuery(props) {
    let { pageIndex, pageSize } = this.state;
    let dataSource = props.gridOption.dataSource;

    //筛选
    let filters = this.getFilters();
    for (let filter of filters) {
      if (dataSource.length === 0) break;
      if (util.string.isEmpty(filter.value)) continue;
      dataSource = dataSource.filter(p => {
        let data = p[filter.field];
        if (typeof (data) === 'undefined' || data === null) return false;
        data += '';
        let value = filter.value + '';

        switch (filter.operate) {
          case 'equal':
            return data === value;
          case 'notequal':
            return data !== value;
          case 'less':
            return data < value;
          case 'lessorequal':
            return data <= value;
          case 'greater':
            return data > value;
          case 'greaterorequal':
            return data >= value;
          case 'contains':
            return data.indexOf(value) >= 0;
          case 'startswith':
            return data.indexOf(value) === 0;
          case 'endswith':
            let subLen = data.length - value.length;
            return subLen > 0 && data.lastIndexOf(value) === subLen;
          default:
            return data === value;
        }
      })
    }

    //合计
    let sum = {};
    for (let field of this.state.sumFields) {
      let fsum = 0;
      for (let data of dataSource) {
        let value = isNaN(data[field]) ? 0 : data[field];
        fsum = util.number.add(fsum, value);
      }
      sum[field] = fsum;
    }

    //排序
    let compare = (x, y) => {
      let sortField = this.state.sortField;
      let sortDirection = this.state.sortDirection;
      if (x[sortField] === y[sortField]) return 0;
      if (sortDirection === 'desc') {
        return x[sortField] > y[sortField] ? -1 : 1;
      } else {
        return x[sortField] > y[sortField] ? 1 : -1;
      }
    }
    dataSource.sort(compare);

    //分页
    let start = (pageIndex - 1) * pageSize;
    let end = start + pageSize;
    if (end >= dataSource.length) end = dataSource.length;

    let items = dataSource.slice(start, end);
    this.setState({
      gridDatas: {
        totalCount: dataSource.length,
        items: items,
        sum: sum
      }
    })
  }

  reload() {
    let gridOption = this.props.gridOption;
    gridOption.pageSize = this.state.pageSize;
    this.setState({
      pageIndex: 1,
      pageSize: gridOption.pageSize || 15,
      sortField: gridOption.sortField || this.state.columns[0].data,
      sortDirection: gridOption.sortDirection || "desc",
      filterRules: [{
        field: this.state.queryColumns.length > 0 ? this.state.queryColumns[0]['data'] : '',
        operate: 'equal',
        value: ''
      }]
    }, this.query);
  }

  setFieldSource(key, data) {
    let columns = this.state.columns;
    let sourceKeyMap = this.state.sourceKeyMap;

    for (let col of columns) {
      if (col.data === key && sourceTypes.includes(col.type)) {
        col.source = { data: data };
        sourceKeyMap[key] = arrayToHash(data);
      }
    }
    this.setState({
      columns: columns,
      sourceKeyMap: sourceKeyMap
    })
  }

  setCustomData(data,extraData) {
    let customData = this.state.customData;
    for (let filter of data) {
      let exsitIndex = customData.findIndex(f => f.field === filter.field);
      if (exsitIndex < 0) {
        customData.push(filter)
      } else {
        customData[exsitIndex] = filter;
      }
    }
    let newState = { customData: data };
    for(var k in extraData){
      newState[k] = extraData[k];
    };
    this.setState(newState, this.reload);
  }

  filterChange(index) {
    return (rule)=>{
      let rules = this.state.filterRules;
      rules[index] = rule;
      this.setState({ filterRules: rules });
      if(rule.onchange){
        var column;
        let columns = deepClone(this.state.columns); 
        let queryColumns = deepClone(this.state.queryColumns); 
        for(var col of columns){
          if(col.data === rule.target){
            column = col;
          }
        }
        rule.onchange(rule.value,column);
        for(var col of queryColumns){
          if(col.data === rule.target){
            col.source.data = column.source.data;
          }
        }
        this.setState({columns:columns,queryColumns:queryColumns});
      }
    }
  }

  addFilter() {
    let queryColumns = this.state.queryColumns;
    if (queryColumns.length === 0) return;

    let rules = this.state.filterRules;
    rules.push({
      field: queryColumns[0]['data'],
      operate: 'equal',
      value: ''
    });
    this.setState({ rules: rules });
  }

  removeFilter(index) {
    let rules = this.state.filterRules;
    if (index >= rules.length) return;

    rules.splice(index, 1);
    this.setState({ filterRules: rules });
  }

  getFilters() {
    let customDataDistinct = [];
    let filterDistinct = [];
    let self  = this;
    
    let customRules = this.state.customData.map(function (data) {
      var oper = data.operate ? data.operate : "equal";
      if(!customDataDistinct[''+data.field+oper]){
        var tdata = {
          field: data.field,
          operate: data.operate ? data.operate : "equal",
          value: data.value
        };        
        customDataDistinct[''+data.field+oper] = tdata;
        return tdata
      }
    });
    let filterRules = this.state.filterRules.filter(r => (r.value + '').length > 0 || r.rules !== null);

    filterRules.map(function (data) {
      var oper = data&& data.operate ? data.operate : "equal";
      if(data&&!filterDistinct[''+data.field+oper]){
        var tdata = {
          field: data.field,
          operate: data.operate ? data.operate : "equal",
          value: data.value
        };        
        filterDistinct[''+data.field+oper] = tdata;
        return tdata
      }
    });

    let customRules1 = deepClone(customRules); 
    var i = 0;
    customRules.map(function(data){
      var oper = data&& data.operate ? data.operate : "equal";
      if(filterDistinct[''+data.field+oper]){
        customRules1.splice(i,1);
      }
      i++;
    });


    return filterRules.concat(customRules1).filter(r => (r.value + '').length > 0 || r.rules !== null)
  }

  switchColumn(index) {
    let columns = this.state.columns;
    if (index < 0 || index >= columns.length) return;

    columns[index].hide = columns[index].hide === 'user' ? 'no' : 'user';
    this.setState({ columns: columns });
  }

  render() {
    let self = this;
    let gridOption = self.props.gridOption;
    let primaryKey = self.state.primaryKey;
    let menuPathId = self.props.menuPathId?self.props.menuPathId:'';

    let ths = self.state.columns.filter(c => c.hide === 'no' && c.colSpan !== 0).map(function (col, index) {
      let sortClass = "";
      if (!col.sortDisable && col.type !== "command") {
        sortClass = self.state.sortField !== col.data
          ? styles.sorting
          : self.state.sortDirection === 'asc' ? styles.sorting_asc : styles.sorting_desc;
      }
      let colKey = col.type === 'command' ? 'col_command' : col.data;
      colKey += '_' + index;
      return (<th key={colKey} colSpan={col.colSpan || 1} style={col.colSpan > 1 ? { textAlign: 'center' } : {}} className={sortClass} onClick={() => self.sortClick(col)}>{col.title}</th>);
    });
    let trs = self.state.gridDatas.items.map(function (data,indextr) {
      let backColor = "";
      if (typeof (gridOption.colorRender) === 'function') {
        backColor = gridOption.colorRender(data);
        if (backColor === 'success') backColor = "rgba(130,206,75,.4)";
        else if (backColor === 'warn') backColor = "rgba(255,253,186,.5)";
        else if (backColor === 'error') backColor = "rgba(255,128,102,.2)";
      }

      let rowDataMap = {};
      self.state.columns.filter(c => c.hide === 'no').forEach(col => {
        if (col.type === 'command') return;
        let formatValue;
        let title = self.state.keyTitleMap[col.data];
        let value = data[col.data];
        if (col.render) {
          formatValue = col.render(value, data)
        } else {
          if (col.type === 'switch') formatValue = SwitchRender(value);
          else if (col.type === 'dropdown' || col.type === 'cascader') formatValue = DropdownRender(value, self.state.sourceKeyMap[col.data]);
          else if (col.type === 'multi') formatValue = MultiRender(value, self.state.sourceKeyMap[col.data]);
          else if (col.type === 'date') formatValue = DateTimeRender(value, 'yyyy-MM-dd');
          else if (col.type === 'datetime') formatValue = DateTimeRender(value, 'yyyy-MM-dd HH:mm:ss');
          else if (col.type === 'img' || col.type === 'imgs') formatValue = ImageRender(value);
          else if (col.type === 'file' || col.type === 'files') formatValue = FileRender(value);
          else if (col.type === 'money') formatValue = MoneyRender(value);
          else if (col.type === 'input' ) {formatValue = InputRender(value,data,col.data);}
          else if (col.type === 'link' ) {formatValue = Link(value);}
          else formatValue = typeof (value) === 'undefined' ? '' : value;
        }
        rowDataMap[col.data] = { title, value, formatValue };
      })

      return <tr
        style={util.string.isEmpty(backColor) ? {} : { backgroundColor: backColor }}
        className="ant-table-row  ant-table-row-level-0" key={`tr_${menuPathId}_${data[primaryKey]}`}>
        {gridOption.checkable && <td><Checkbox disabled={gridOption.disableFun?gridOption.disableFun(data):false} checked={self.state.checkedValues.indexOf(data[primaryKey]) >= 0} onChange={() => self.checkClick(data[primaryKey])} /></td>}
        {
          self.state.columns.filter(c => c.hide === 'no').map(function (col, index) {
            if (col.type === "command") {
              let tdActions = col.actions || [];
              let colKey = `tr_${menuPathId}_${data[primaryKey]}_td_command_${index}`;

              return <td key={colKey}>
                {
                  tdActions.map(function (action, aIndex) {
                    if (action.hideFunc && action.hideFunc(data)) return '';

                    var actionName = action.nameFormat ? action.nameFormat(data) : action.name;
                    let color = config.color[action.color] ? config.color[action.color] : action.color;
                    return <span key={`tr_${menuPathId}_${data[primaryKey]}_action_${aIndex}`}>
                      {aIndex > 0 && <Divider type="vertical" />}
                      {action.confirm
                        ? <Popconfirm title={'确定要' + actionName + '吗？'} onConfirm={() => action.onClick(data,self,indextr)}><a style={util.string.isEmpty(color) ? {} : { color: color }} href="#">{actionName}</a></Popconfirm>
                        : <a style={util.string.isEmpty(color) ? {} : { color: color }} href="javascript:void(0);" onClick={() => action.onClick(data,self,indextr)}>{actionName}</a>}
                    </span>
                  })
                }
              </td>;
            }
            else {
              let { value, formatValue } = rowDataMap[col.data];
              let colKey = `tr_${menuPathId}_${data[primaryKey]}_td_${col.data}_${index}`;
              if (col.type === 'img' || col.type === 'imgs') {
                formatValue = ImageRender(value, rowDataMap)
              } else if (col.type === 'file' || col.type === 'files') {
                formatValue = FileRender(value, rowDataMap)
              }
              let align = col.align || 'left';
              let widthStyle = col.width ? { width: col.width } : {};
              let customStyle = col.customStyle ?  col.customStyle  : {};

              let field = col;
              let editor ;
              if(!field.editor){
                editor = {};
              }else{
                editor = field.editor;
              }
              let valuetd = typeof (editor.value) != 'undefined' ? (typeof (editor.value) === 'function'?editor.value(): editor.value):(typeof (data[field.data]) != 'undefined'?data[field.data]:"");
              
			      if(data)
              data[col.data] = editor.defaultValue?editor.defaultValue:valuetd;
              let fieldOption = {
                name: "",//field.title
                key: field.data,
                tips: editor.tips,
                isRequired: editor.isRequired,
                validateRegular: editor.validateRegular,
                step: editor.step,
                precision: editor.precision,
                fieldType: field.type,
                value: editor.defaultValue?editor.defaultValue:valuetd,
                disabled: editor.isDisabled&&editor.isDisabled(data,col.data)?true:editor.ep === 'disabled',
                source: field.source || {},
                innerProps:editor.innerProps,
                width:field.width
              }
              
              if (col.type === 'text' || col.type === 'textarea' || col.type === 'richtext') {
                let maxLength = col.maxLength || 30;
                let tgvalue = util.string.truncate(formatValue, maxLength);

                if (field.editor&&field.editor.inline
                  &&(!field.editor.inline.isShow||field.editor.inline.isShow(data,self.state.gridDatas.items))){
                return <td style={{ textAlign: align, ...widthStyle,...customStyle }} title={formatValue} key={colKey}>
                  <AutoField key={'auto_field_' + index} fieldOption={fieldOption} labelColSpan={6 / 6} 
                    inline ={true} 
                    onChange={function(key, value){
                      //console.log(key);
                      //console.log(value);
                        let stateData = self.state;
                        stateData.gridDatas.items[indextr][key] = value;
                        self.setState(stateData);
                        
                        if(col.editor.inline.onChange){
                          col.editor.inline.onChange(key,value,indextr,self);
                        }
                    }}
                    onSearch = {(key, value,that) => {
                      if(col.editor.inline.onSearch){
                        col.editor.inline.onSearch(key,value,indextr,self,that);
                      }
                    }}
                    />
                  </td>
                }else{
                  if(col.onclick){
                    return  <td style={{ textAlign: align, ...widthStyle,...customStyle }} title={formatValue} key={colKey}
                    >
                      <Popconfirm style={self.state.hidePop?"display:'none'":""} okText={'确定'} cancelText={'取消'} 
                      title={'您确认要摘取这个值吗？'} 
                      onConfirm={() => col.onclick(data[col.data],col,data)}>
                    <Button icon='sync' ghost /></Popconfirm>
                    {tgvalue}
                    </td>
                  }else{
                    return <td style={{ textAlign: align, ...widthStyle,...customStyle }} title={formatValue} key={colKey}
                    >{tgvalue}
                  </td>
                  }
                }
              } else {
                if (field.editor&&col.editor.inline
                  &&(!field.editor.inline.isShow||field.editor.inline.isShow(data,self.state.gridDatas.items))){
                  //获取选项源数据  
                  let sourceData = self.state.sourceKeyMap[col.data];
                  if(col.source && col.source.params && col.source.params.length>0 && sourceData){
                    let pramstring = "";
                    let qurystr = "";
                    for(var param in col.source.params){
                      if(data[col.source.params[param].key]){
                        pramstring = data[col.source.params[param].key];
                      }
                      qurystr += col.source.params[param].key+"="+pramstring+"&";
                    }
                    sourceData = sourceData[qurystr.replace("&","_").replace("=","-")];
                  }
                  if(sourceData)
                  sourceData = hashToArray(sourceData);

                  return <td style={{ textAlign: align, ...widthStyle,...customStyle }} key={colKey}>
                  <AutoField key={'auto_field_' + index} fieldOption={fieldOption} labelColSpan={6 / 6} 
                    inline ={true} 
                    onChange={
                      function(key, value){
                        //console.log(key);
                        //console.log(value);
                        
                        let stateData = self.state;
                        stateData.gridDatas.items[indextr][key] = value;
                        self.setState(stateData);

                        if(col.editor.inline.onChange){
                          col.editor.inline.onChange(key,value,indextr,self);
                        }
                      }
                    }
                    onSearch = {(key, value,that) => {
                      if(col.editor.inline.onSearch){
                        col.editor.inline.onSearch(key,value,indextr,self,that);
                      }
                    }}
                    fieldData = {sourceData}
                    />
                  </td>
                }else{
                  if(col.onclick){
                    return <td style={{ textAlign: align, ...widthStyle,...customStyle }} key={colKey}
                    >
                    <Popconfirm style={self.state.hidePop?"display:'none'":""} okText={'确定'} cancelText={'取消'} 
                      title={'您确认要摘取这个值吗？'} 
                      onConfirm={() => col.onclick(data[col.data],col,data)}>
                    <Button icon='sync' ghost /></Popconfirm>
                    {formatValue}
                    </td>
                  }else{
                    return <td style={{ textAlign: align, ...widthStyle,...customStyle }} key={colKey}
                  >{formatValue}
                  </td>
                  }
                  
                }
              }
            }
          })
        }
      </tr>
    });

    let sumTr = <tr style={{ backgroundColor: '#f7f7f7' }}>{self.state.columns.filter(c => c.hide === 'no').map(function (col, index) {
      let align = col.align ? col.align : col.type === 'money' ? 'right' : 'left';
      let key = `sum_col_${index}`

      if (index === 0) return <td key={key} colSpan={gridOption.checkable ? 2 : 1}>合计：</td>;
      else if (self.state.sumFields.includes(col.data)) return <td key={key} style={{ textAlign: align }}>{self.state.gridDatas.sum[col.data]}</td>;
      else return <td key={key} style={{ textAlign: 'center' }}>------</td>;
    })}</tr>;

    let actions = self.state.actions.map(function (action, index) {
      let checkedValues = self.state.checkedValues;
      let checkedDatas = self.state.gridDatas.items.filter(item => checkedValues.indexOf(item[primaryKey]) >= 0);
      return action.confirm
        ? (action.hideFunc&&action.hideFunc()?"":<Popconfirm key={"action_" + index} okText={'确定'} cancelText={'取消'} title={'确定要' + action.name + '吗？'} onConfirm={() => action.onClick(checkedValues, checkedDatas)}><BirdButton color={action.color} icon={action.icon} type="primary">{action.name}</BirdButton></Popconfirm>
        ): (action.hideFunc&&action.hideFunc()?"":<BirdButton key={"action_" + index} color={action.color} icon={action.icon} type="primary" idempotency = {true}  onClick={() => action.onClick(checkedValues, checkedDatas)}>{action.name}</BirdButton>
        )
    });

    let filterGroups = [[]];
    let filterRules = self.state.filterRules;

    for (let i = 1, len = filterRules.length; i < len; i++) {
      let offset = i % 3;
      let groupIndex = (i - offset) / 3;
      if (offset === 0) {
        filterGroups.push([]);
        groupIndex--;
      }
      filterGroups[groupIndex].push(filterRules[i])
    }


    let filters = filterGroups.map((group, gindex) => {
      return <Row key={'filter_group_' + gindex}>
        {group.map((rule, rindex) => {
          let index = gindex * 3 + rindex + 1;//在filterRules中的index
          return <Col span={8} key={'filter_group_' + gindex + '_rule_' + rindex}>
            <Row>
              <Col span={20}>
                <BirdGridFilter fields={self.state.queryColumns} rule={rule}
                  option={self.state.filterOption}
                  onChange={self.filterChange(index)} />
              </Col>
              <Col span={4}>
                <Button icon='close' type='danger' onClick={() => self.removeFilter(index)} />
              </Col>
            </Row>
          </Col>
        })}
      </Row>
    })

    return (
      <Spin spinning={this.state.queryLoading} tip="数据加载中...">
        <Card>
          <Row>
            <Col span={10}>
              {this.state.queryColumns.length > 0 && <Row>
                <Col span={16}>
                  <BirdGridFilter fields={this.state.queryColumns} rule={this.state.filterRules[0]}
                    option={self.state.filterOption}
                    onChange={this.filterChange(0)} />
                </Col>
                <Col span={8}>
                  <Button.Group style={{display:self.state.filterOption&&self.state.filterOption.display?self.state.filterOption.display:''}}>
                    <Button icon='plus' onClick={() => self.addFilter()} 
                    />
                    <Button icon='search' type="primary" onClick={() => { this.setState({ pageIndex: 1 }, this.query) }}>{gridOption.queryText || '查询'}</Button>
                  </Button.Group>
                </Col>
              </Row>}
            </Col>
            <Col span={14}>
              <div className={styles.action}>
                <Button.Group>
                  {actions}
                  <Button icon='sync' type="primary" onClick={() => this.reload()} />
                  <Popover placement="bottomRight" trigger="click" content={this.state.columns.map((col, index) =>
                    <div key={'col_opt_' + index}>
                      {(col.hide !== 'dev') && <Checkbox checked={col.hide === 'no'} onChange={() => this.switchColumn(index)}>{col.title}</Checkbox>}
                    </div>)}>
                    <Button icon='bars' type="primary" />
                  </Popover>
                </Button.Group>
              </div>
            </Col>
          </Row>
          {filters}
          <div className={styles.bird_table}>
            {/*<div className={styles.changebox1}>占位用</div>*/}
            {/* <div className={styles.bird_table_body}> */}
            <div>
              <table>
                <thead className={styles.bird_table_thead}>
                  <tr>
                    {gridOption.checkable && <th style={{ width: 15 }}>
                    {(gridOption.checkedAllIShow == true)?
                      <span>全<Checkbox
                      checked={this.state.checkedAllI}
                      disabled={gridOption.checkedAllIDisableFun?gridOption.checkedAllIDisableFun( this.state.gridDatas.items):false} 
                      onChange={() => this.checkedAllIClick()} />
                      </span>:<span></span>}

                    <Checkbox
                      checked={
                        (self.props.gridOption && self.props.gridOption.disableFun)?
                         (this.state.checkedValues.length > 0 
                          && this.state.checkedValues.length == this.state.availableItems.length):(
                        this.state.checkedValues.length > 0 
                        && this.state.checkedValues.length === this.state.gridDatas.items.length
                        )
                      }
                      disabled={gridOption.checkAllDisableFun?gridOption.checkAllDisableFun( this.state.gridDatas.items):false} 
                      onChange={() => this.checkAllClick()} />
                      </th>}
                    {ths}
                  </tr>
                </thead>
                <tbody className={styles.bird_table_body}>
                  {trs}
                  {self.state.gridDatas.items.length > 0 && self.state.gridDatas.sum && self.state.sumFields.length > 0 && sumTr}
                </tbody>
              </table>
              <Pagination className={styles.gridPagination}
                showQuickJumper
                current={this.state.pageIndex}
                total={parseInt(this.state.gridDatas.totalCount, 10)}
                pageSizeOptions={gridOption.pageSizeOptions || ["10", "15", "20", "30", "50", "100", "200"]}
                pageSize={this.state.pageSize}
                showSizeChanger={true}
                onChange={(page, pageSize) => this.pageClick(page, pageSize)}
                onShowSizeChange={(page, pageSize) => this.pageSizeChange(pageSize)}
                showTotal={total => `共 ${total} 条`} />
            </div>
            {/*<div className={styles.changebox2}>占位用</div>*/}
            <Modal title={this.state.formOption.model === 'add' ? '新增' : '编辑'}
              width={this.state.formWidth}
              visible={this.state.formVisiable}
              onOk={() => this.saveClick()}
              onCancel={() => this.cancelClick()}
              confirmLoading={this.state.formConfirmLoading}
            >
              <AutoForm formOption={this.state.formOption} ref="autoForm" />
            </Modal>
          </div>
        </Card>
      </Spin>
    );
  }
}

BirdGrid.propTypes = {
  gridOption: PropTypes.object.isRequired
}

// export default connect(mapStateToProps)(BirdGrid);
export default BirdGrid;