import React,{ PureComponent} from 'react';
import PropTypes from 'prop-types';
import BirdSelector from '../Form/BirdSelector';
import BirdCascader from '../Form/BirdCascader';
import moment from 'moment';
import { Select, DatePicker, Input, InputNumber } from 'antd';
import util from "./../../utils/util";

const Option = Select.Option;
const InputGroup = Input.Group;

const operators = {
  common: [{ value: "equal", text: "等于" }, { value: "notequal", text: "不等于" }],
  struct: [{ value: "less", text: "小于" }, { value: "lessorequal", text: "小于等于" }, { value: "greater", text: "大于" }, { value: "greaterorequal", text: "大于等于" }],
  text: [ { value: "contains", text: "包含" },{ value: "startswith", text: "开始于" }, { value: "endswith", text: "结束于" }]
}
class BirdGridFilter extends PureComponent {
  constructor(props) {
    super(props);

    this.state = {
      option:props.option?props.option:{},
      showSelectColumn: true, //是否显示选择字段
      showSelectOpera : true, //是否显示选择操作符
      selectedProvince: "110000", //选择了的省份
      selectedCity:"", //选择了的市
    }
    
  }

  componentDidMount() {
    let field = this.props.fields.find(f => f.data === this.props.rule.field);
    
    switch (field.type) {
      case "text":
      case "textarea":
        this.props.rule.operate = 'contains';
        break;
      case "number":
      case "money":
      case "date":
      case "datetime":
        this.props.rule.operate = 'equal';
        break;
      default:
        this.props.rule.operate = 'equal';
        break;
    }
    
    if(field.type === "dropdown" && field.source) {
      //不显示选择操作符
      if(field.source.multi){
        this.setState({
          showSelectOpera : false,
        });
      } else {
        this.setState({
          showSelectOpera : true,
        });
      }
    } else if(field.type === "region") {
      //不显示选择字段和操作符
      this.props.rule.operate = "equal";
      this.setState({showSelectOpera:false, showSelectColumn:false});
    } else {
      this.setState({showSelectOpera:true, showSelectColumn:true});
    }
  }

  componentWillReceiveProps(nextProps) {
    if ( !util.object.equal(nextProps.option, this.props.option)) {
      this.setState({ option:nextProps.option})
    }
    if ( !util.object.equal(nextProps.fields, this.props.fields)) {
      this.setState({ fields:nextProps.fields})
    }
  }
  
  onRuleChange = (key, value) => {
    let rule = this.props.rule;
    if (key === 'field' && rule[key] !== value) {
      rule.value = "";
    }
    
    if(key === "field"){
      let field = this.props.fields.find(f => f.data === value);
      
      switch (field.type) {
        
        case "text":
        case "textarea":
          this.props.rule.operate = 'contains';
          break;
        case "number":
        case "money":
        case "date":
        case "datetime":
          this.props.rule.operate = 'equal';
          break;
        default:
          this.props.rule.operate = 'equal';
          break;
      }
      if(field.type === "dropdown" && field.source) {
        //不显示选择操作符
        if(field.source.multi){
          this.setState({
            showSelectOpera : false,
          },() => console.log(this.state));
        } else {
          this.setState({
            showSelectOpera : true,
          });
        }
      } else if(field.type === "region") {
        //不显示选择字段和操作符
        this.props.rule.operate = "equal";
        this.setState({showSelectOpera:false, showSelectColumn:false});
      } else {
        this.setState({showSelectOpera:true, showSelectColumn:true});
      }
    }
    rule[key] = value;
    this.props.onChange && this.props.onChange(rule);
  }

  //非级联的两重多选框事件
  onMultiSelect = (value) => {
    let rule = this.props.rule;
    rule.operate = "or";
    rule.rules = value.map((item) => {
      return {
        field:rule.field,
        operate:"equal",
        value:item
      };
    });
  }

  render() {
    let field = this.props.fields.find(f => f.data === this.props.rule.field);
    let searchOperators = operators.common;
    switch (field.type) {
      case "text":
      case "textarea":
        searchOperators = operators.text.concat(searchOperators);
        break;
      case "number":
      case "money":
      case "date":
      case "datetime":
        searchOperators = searchOperators.concat(operators.struct);
        break;
      default:
        break;
    }

    let valueField;
    switch (field.type) {
      case "text":
      case "textarea":
        valueField =
          <Input id={field.data} value={this.props.rule.value} style={{ width: '37.5%' }} onChange={(e) => this.onRuleChange('value', e.target.value)} />;
        break;
      case "number":
        valueField = <InputNumber id={field.data} min={0} value={this.props.rule.value}
          style={{ width: '37.5%' }}
          onChange={(value) => this.onRuleChange('value', value)} />;
        break;
      case "money":
        valueField = <InputNumber id={field.data} min={0} value={this.props.rule.value}
          formatter={value => `${value}`.replace(/\B(?=(\d{3})+(?!\d))/g, ',')}
          parser={value => value.replace(/\$\s?|(,*)/g, '')}
          style={{ width: '37.5%' }}
          onChange={(value) => this.onRuleChange('value', value)} />;
        break;
      case "switch":
        valueField = (
          <Select id={field.data} value={this.props.rule.value} onChange={value => this.onRuleChange("value", value)} style={{ width: '37.5%' }}>
            <Option value="1">是</Option>
            <Option value="0">否</Option>
          </Select>
        );
        break;
      case "dropdown":
        if(field.source.multi === true) {
          let children = field.source.data.map((item) => {
            return <Option key={"opt_"+item.value} value={item.value}>{item.label}</Option>;
          });
          valueField = <Select 
            mode="multiple"
            style={{ width: '62.5%' }}
            onChange={value => this.onMultiSelect(value)}>
              {children}
            </Select>
        } else {
          valueField = <BirdSelector dicKey={field.source.key} data={field.source.data || []} width='37.5%'
            onChange={value => this.onRuleChange('value', value)} 
            selectedValue={this.props.rule.value} />;
        }
        break;
      case "cascader":
        valueField = <BirdCascader data={field.source.data || []} expandTrigger='hover' width='37.5%'
          onChange={value => this.onRuleChange('value', value)} value={this.props.rule.value} />;
        break;
      case "date":
        valueField =
          <DatePicker id={field.data} value={this.props.rule.value ? moment(this.props.rule.value) : null} format={"YYYY-MM-DD"}
            style={{ width: '37.5%' }}
            onChange={(date, dateString) => this.onRuleChange('value', dateString)} />;
        break;
      case "datetime":
        valueField =
          <DatePicker id={field.data} value={this.props.rule.value ? moment(this.props.rule.value) : null}
            format={"YYYY-MM-DD HH:mm"}
            style={{ width: '37.5%' }}
            onChange={(date, dateString) => this.onRuleChange('value', dateString)} />;
        break;
      case "region": //地区的，三级级联
        valueField = 
          <div>
            <Select 
              style={{width:"33.3%", float:"left"}}
              defaultValue={util.Provinces[0].value}
              onChange={value => {
                this.setState({
                  selectedProvince: value
                });
              }}
            >
              {util.Provinces.map((item) => {
                return <Option key={item.value} value={item.value}>{item.label}</Option>;
              })}
            </Select>
            <Select 
              style={{width:"33.3%", float:"left"}}
              onChange={value => {
                this.setState({
                  selectedCity: value,
                })
              }}
            >
            {util.Cities.filter((item) => {
              let province = this.state.selectedProvince.substring(0, 2);
              return item.value.startsWith(province);
            }).map((item) => {
              return <Option key={item.value} value={item.value}>{item.label}</Option>;
            })}
          </Select>
          <Select
            style={{width:"33.3%", float:"left"}}
            onChange={value => {
              let rule = this.props.rule;
              rule.operate = "equal";
              rule.value = value;
            }}
          >
            {util.Areas.filter((item) => {
              let city = this.state.selectedCity.substring(0, 4);
              return item.value.startsWith(city);
            }).map((item) => {
              return <Option key={item.value} value={item.value}>{item.label}</Option>;
              })}
            </Select>
          </div>;
        break;
      default:
        valueField = <span />;
        break;
    }
    let option = this.state.option;

    //选择操作符
    let selectOpera = (
      <Select placeholder="请选择操作符"
        style={{
          width: (option.width2 ? option.width2 : '25%'),
          display: (option.display2 ? option.display2 : '')
        }}
        value={this.props.rule.operate}
        onChange={operator => this.onRuleChange('operate', operator)}>
        {
          searchOperators.map(function (operator) {
            return <Option key={'operator_option_' + operator.value}
              value={operator.value}>{operator.text}</Option>;
          })
        }
      </Select>
    );
    if(!this.state.showSelectOpera) {
      selectOpera = <div></div>;
    }
    //选择字段
    let selectColumn = (
      <Select style={{
        width: (option.width1 ? option.width1 : '37.5%'),
        display: (option.display1 ? option.display1 : '')
      }}
        value={this.props.rule.field}
        onChange={value => this.onRuleChange('field', value)}>
        {
          this.props.fields.map(function (item) {
            return <Option key={'field_option_' + item.data}
              value={item.data}>{item.title}</Option>;
          })
        }
      </Select>
    );
    if(!this.state.showSelectColumn) {
      selectColumn = <div></div>;
    }

    return (
      <div  style={{display:(option.display?option.display:'')}}>
      {field.type === "region" ? 
        (<div>
          {selectColumn}
          {selectOpera}
          {valueField}
        </div>) : 
        (<InputGroup compact>
          {selectColumn}
          {selectOpera}
          {valueField}
        </InputGroup>)
      }  
      </div>
    );
  }
}

BirdGridFilter.propTypes = {
  fields: PropTypes.array.isRequired,
  onChange: PropTypes.func,
  rule: PropTypes.object
}

BirdGridFilter.defaultProps = {
  fields: []
}

export default BirdGridFilter;
