import React,{ PureComponent} from 'react';
import PropTypes from 'prop-types';
import { util } from 'utils';
import { Row, Col, Card } from 'antd';
import BirdGrid from './BirdGrid';
import BirdTree from './BirdTree';

class BirdTreeGrid extends PureComponent {
  constructor(props) {
    super(props);

    this.state = {
      treeOption: this.props.treeOption,
      itemHash: {},
      selectedKeys: [],
      expandedKeys: []
    }
  }

  /* 性能优化 props改变才渲染 */
  shouldComponentUpdate(nextProps,nextState){
    if(util.object.equal(nextProps, this.props)&&
    util.object.equal(nextState, this.state)){ //传入的count与组件当前props的count比较,count没改变，return false，不渲染
      return false    //不渲染
    }
    return true;  //渲染
  }
  
  //树节点点击事件，不支持点选多个节点
  itemClick(clickKey,selectKeys,type) {
    let customRules = this.props.gridOption.customRules||[];
    customRules.push({
      field: this.props.treeOption.paramName,
      value: clickKey
    })
    if(this.refs&&this.refs.grid&&this.refs.grid.setCustomData)
    this.refs.grid.setCustomData(customRules);

    this.props.treeOption.onSelect && this.props.treeOption.onSelect(clickKey,selectKeys,type);
  }

  getGrid() {
    return this.refs.grid;
  }
  getTree() {
    return this.refs.tree;
  }

  render() {
    let self = this;
    let option = {
      ...self.state.treeOption,
      initFirstLeaf: true,
      onSelect: (key) => self.itemClick(key)
    };

    let gridOption = this.props.gridOption;
    gridOption.autoQuery = false;

    let treeSpan = this.props.treeOption.span || 4;

    return (<Row type="flex" justify="center">
      <Col span={treeSpan}>
        <Card title={self.props.treeOption.title}>
          <BirdTree treeOption={option} ref="tree" onRef={(ref) => this.refs.tree = ref }/>
        </Card>
      </Col>
      <Col span={24 - treeSpan}>
        <BirdGrid gridOption={gridOption} ref="grid" onRef={(ref) => this.refs.grid = ref }/>
      </Col>
    </Row>)
  }
}

BirdTreeGrid.propTypes = {
  treeOption: PropTypes.shape({
    url: PropTypes.string.isRequired,
    title: PropTypes.string.isRequired,
    paramName: PropTypes.string.isRequired,//点击时关联的表格字段名
    textField: PropTypes.string,
    valueField: PropTypes.string,
    parentField: PropTypes.string,
    initialValue: PropTypes.string,
    span: PropTypes.number
  }),
  gridOption: PropTypes.object.isRequired
};

export default BirdTreeGrid;
