import React from 'react';
import DetailHeader from './DetailHeader';
import { Input, Select, Collapse, Button, Modal, message, Icon, Row, Col } from 'antd';
import { jrequest } from '../../utils/request';

class DataSourceDetail extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            realNode: props.nodeDetail.realNode,
            dbInfo: props.nodeDetail.realNode.entry,
            linking: false,
        }
        this.staticData = {
            realNode: props.nodeDetail.realNode,
            dbInfo: props.nodeDetail.realNode.entry
        }
    }

    componentWillReceiveProps(props) {
        let node = props.nodeDetail.realNode;
        let info = props.nodeDetail.realNode.entry;
        this.setState({
            realNode: node,
            dbInfo: info
        });
        this.staticData = {
            realNode: props.nodeDetail.realNode,
            dbInfo: props.nodeDetail.realNode.entry
        }
    }

    //更改数据库选择
    changeDbType = (value) => {
        // let node = this.state.realNode;
        let node = this.staticData.realNode;
        node.type = value;
        // this.setState({
        //     realNode: node
        // });
        this.staticData.realNode = node;
    }

    //更改链接名称
    changeNodeName = (event) => {
        // let node = this.state.realNode;
        let node = this.staticData.realNode;
        node.name = event.target.value;
        // this.setState({
        //     realNode: node
        // });
        this.staticData.realNode = node;
    }

    //更改输入
    changeInput = (event, field) => {
        // let info = this.state.dbInfo;
        let info = this.staticData.dbInfo;
        info[field] = event.target.value;
        // this.setState({
        //     dbInfo: info
        // });
        this.staticData.dbInfo = info;
    }

    onBtnDel = (e) => {
        let onOkHandler = () => {
            let id = this.state.realNode.id;

            jrequest({
                url:"/meta/node",
                method:"delete",
                data: {'id':id}
            }).then((result) => {
                //成功之后的回调
                this.props.refreshTreeNode(false);
            });
        }

        Modal.confirm({
            title:"确定要本数据源？过程不可逆",
            okType:"danger",
            onOk() {
                onOkHandler();
            }
        });
    }

    //保存数据源
    onBtnSave = (e) => {
        let data = this.staticData.realNode;
        data.entry = this.staticData.dbInfo;

        jrequest({
            url:"/meta/datasource",
            method:"put",
            data:data
        }).then((result) => {
            this.setState({
                realNode: this.staticData.realNode,
                dbInfo: this.staticData.dbInfo
            });
            message.success("保存成功");
        }).catch((error) => {
            message.error("保存失败");
        });
    }

    onBtnTry = (e) => {
        this.setState({
            linking: true
        });

        let flag = true;
        setTimeout(function(){ flag = false }, 15000);

        let id = this.state.realNode.id;
        jrequest({
            url:"/meta/children",
            method:"get",
            data:{prtId:id}
        }).then((result) => {
            if(result.data !== null) {
                if(flag) {
                    message.success("连接成功");
                } else {
                    message.error("连接超时");
                }
                this.setState({linking:false});
            } else {
                message.error("连接失败");
                this.setState({linking:false});
            }
        });
    }

    render() {
        /*
            注意，react的渲染机制是每次都会拿render的结构做比较，然后再挂载
            如果前一次和后一次的节点一样（数据不一样）那么react就不会去渲染
            所以，要给这里render的节点加key来每次做刷新
        */
        let loadingIcon = <Icon type="api" />
        if(this.state.linking) {
            loadingIcon = <Icon type="loading"/>
        }
        let buttons = (
            <Row>
                <Col span={3}><Button key="db-del" icon="delete" type="danger" onClick={this.onBtnDel}>删除数据源</Button></Col>
                <Col span={3}><Button key="db-save" icon="save" type="primary" onClick={this.onBtnSave}>保存数据源</Button></Col>
                <Col span={3}><Button key="db-try" type="primary" onClick={this.onBtnTry}>{loadingIcon}测试链接</Button></Col>
            </Row>
        );
        let realNode = this.state.realNode;
        let dbInfo = realNode.entry;

        let dbSelects = (
            <Select key={realNode.type} defaultValue={realNode.type} style={{ width: "250px" }} onChange={this.changeDbType}>
                <Select.Option value="MsSQLNative">SQLServer数据库</Select.Option>
                <Select.Option value="MySQL">MySQL数据库</Select.Option>
                <Select.Option value="HIVE">Hive数据库</Select.Option>
                <Select.Option value="HIVE2KBS">hive2kbs数据库</Select.Option>
                <Select.Option value="ClickHouse">ClickHouse数据库</Select.Option>
                <Select.Option value="IMPALA">Impala数据库</Select.Option>
            </Select>
        );
        return (
            <div>
                {buttons}
                <Collapse defaultActiveKey={['1']} style={{maxWidth:"550px"}}>
                    <Collapse.Panel header="数据源基本信息" key="1">
                        <div style={{ marginBottom: 10 }}><Input key={realNode.id} addonBefore="Id" defaultValue={realNode.id} disabled={true} /></div>
                        <div style={{ marginBottom: 10 }}><Input key={realNode.name} addonBefore="连接名称" defaultValue={realNode.name} onChange={this.changeNodeName} /></div>
                        <div style={{ marginBottom: 10 }}><Input addonAfter={dbSelects} defaultValue="数据库类型" disabled={true}/></div>
                        <div style={{ marginBottom: 10 }}><Input key={dbInfo.host} addonBefore="服务器地址" defaultValue={dbInfo.host} onChange={(e) => this.changeInput(e,"host")}/></div>
                        <div style={{ marginBottom: 10 }}><Input key={dbInfo.port} addonBefore="端口" defaultValue={dbInfo.port} onChange={(e) => this.changeInput(e,"port")}/></div>
                        <div style={{ marginBottom: 10 }}><Input key={dbInfo.serverName} addonBefore="服务名(Schema)" defaultValue={dbInfo.serverName} onChange={(e) => this.changeInput(e,"serverName")}/></div>
                        <div style={{ marginBottom: 10 }}><Input key={dbInfo.user} addonBefore="用户名" defaultValue={dbInfo.user} onChange={(e) => this.changeInput(e,"user")}/></div>
                        <div style={{ marginBottom: 10 }}><Input.Password addonBefore="密码" defaultValue={dbInfo.pwd} onChange={(e) => this.changeInput(e,"pwd")}/></div>
                    </Collapse.Panel>
                    <Collapse.Panel header="高级信息" key="2">
                    <div style={{ marginBottom: 10 }}><Input key={dbInfo.keytabPath} addonBefore="keytabPath" defaultValue={dbInfo.keytabPath} onChange={(e) => this.changeInput("keytabPath")}/></div>
                    <div style={{ marginBottom: 10 }}><Input key={dbInfo.krb5ConfigPath} addonBefore="krb5ConfigPath" defaultValue={dbInfo.krb5ConfigPath} onChange={(e) => this.changeInput("krb5ConfigPath")} /></div>
                    <div style={{ marginBottom: 10 }}><Input key={dbInfo.hadoopHomeDir} addonBefore="hadoopHomeDir" defaultValue={dbInfo.hadoopHomeDir} onChange={(e) => this.changeInput("hadoopHomeDir")} /></div>
                    <div style={{ marginBottom: 10 }}><Input key={dbInfo.principal} addonBefore="principal" defaultValue={dbInfo.principal} onChange={(e) => this.changeInput("principal")}/></div>
                    </Collapse.Panel>
                </Collapse>
            </div>
        );
    }
}

export default DataSourceDetail