import React from 'react';
import { jrequest } from 'utils/request';
import { Input, Modal, Button, Select, Collapse } from 'antd';

class AddDataSource extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            visible: false,
            confirmLoading:false,
        }
        this.staticData = {
            name:"",
            type:"",
            host:"",
            port:"",
            serverName:"",
            user:"",
            pwd:"",
            keytabPath:"",
            krb5ConfigPath:"",
            hadoopHomeDir:"",
            principal:""
        }
    }

    componentWillReceiveProps(props) {
        //重新刷新缓存
        this.staticData = {
            name:"",
            type:"",
            host:"",
            port:"",
            serverName:"",
            user:"",
            pwd:"",
            keytabPath:"",
            krb5ConfigPath:"",
            hadoopHomeDir:"",
            principal:""
        }
    }

    onCancel = () => {
        this.setState({
            visible: false
        });
    }

    onOk = () => {
        this.setState({
            confirmLoading: true
        });

        let node = this.props.realNode;
        let tmpData = this.staticData;
        let data = {
            prtId: node.id,
            name: tmpData.name,
            bizDomain: node.bizDomain,
            nodeType: "phy.DataSource",
            parent:true,
            comment: "",
            remark: "",
            namespace: node.namespace,
            cnName: "",
            type:tmpData.type,
            entry:{
                host:tmpData.host,
                port:tmpData.port,
                serverName:tmpData.serverName,
                user:tmpData.user,
                pwd:tmpData.pwd,
                keytabPath:tmpData.keytabPath,
                krb5ConfigPath:tmpData.krb5ConfigPath,
                hadoopHomeDir:tmpData.hadoopHomeDir,
                principal:tmpData.principal
            }
        }

        jrequest({
            url:"/meta/datasource",
            method:"post",
            data:data
        }).then((result) => {
            this.setState({
                confirmLoading:false,
                visible: false
            });
            //成功后的回调
            this.props.afterOk(true);
        }).catch((error) => {
            this.setState({
                confirmLoading:false
            });
        });
    }

    changeNodeName = (e) => {
        this.staticData.name = e.target.value;
    } 
    changeDbType = (value) => {
        this.staticData.type = value;
    }
    changeInput = (event, field) => {
        this.staticData[field] = event.target.value;
    }


    render() {
        let tmpData = this.staticData;
        let dbSelects = (
            <Select key={tmpData.type} defaultValue="请选择" style={{ width: "250px" }} onChange={this.changeDbType}>
                <Select.Option value="MsSQLNative">SQLServer数据库</Select.Option>
                <Select.Option value="MySQL">MySQL数据库</Select.Option>
                <Select.Option value="HIVE">Hive数据库</Select.Option>
                <Select.Option value="HIVE2KBS">hive2kbs数据库</Select.Option>
                <Select.Option value="ClickHouse">ClickHouse数据库</Select.Option>
                <Select.Option value="IMPALA">Impala数据库</Select.Option>
            </Select>
        );
        return (
            <div>
                <Button icon="plus-circle" type="primary" onClick={(e) => {
                    this.setState({ visible: true });
                }}>添加数据源</Button>
                <Modal
                    visible={this.state.visible}
                    onOk={this.onOk}
                    onCancel={this.onCancel}
                    confirmLoading={this.confirmLoading}>
                    <Collapse defaultActiveKey={['1']} style={{ maxWidth: "550px" }}>
                        <Collapse.Panel header="数据源基本信息" key="1">
                            <div style={{ marginBottom: 10 }}><Input key={tmpData.name} addonBefore="连接名称" onChange={this.changeNodeName} /></div>
                            <div style={{ marginBottom: 10 }}><Input addonAfter={dbSelects} defaultValue="数据库类型" disabled={true} /></div>
                            <div style={{ marginBottom: 10 }}><Input key={tmpData.host} addonBefore="服务器地址" onChange={(e) => this.changeInput(e, "host")} /></div>
                            <div style={{ marginBottom: 10 }}><Input key={tmpData.port} addonBefore="端口" onChange={(e) => this.changeInput(e, "port")} /></div>
                            <div style={{ marginBottom: 10 }}><Input key={tmpData.serverName} addonBefore="服务名(Schema)" onChange={(e) => this.changeInput(e, "serverName")} /></div>
                            <div style={{ marginBottom: 10 }}><Input key={tmpData.user} autoComplete="" addonBefore="用户名" onChange={(e) => this.changeInput(e, "user")} /></div>
                            <div style={{ marginBottom: 10 }}><Input.Password autoComplete="" addonBefore="密码" onChange={(e) => this.changeInput(e, "pwd")} /></div>
                        </Collapse.Panel>
                        <Collapse.Panel header="高级信息" key="2">
                            <div style={{ marginBottom: 10 }}><Input key={tmpData.keytabPath} addonBefore="keytabPath" onChange={(e) => this.changeInput("keytabPath")} /></div>
                            <div style={{ marginBottom: 10 }}><Input key={tmpData.krb5ConfigPath} addonBefore="krb5ConfigPath" onChange={(e) => this.changeInput("krb5ConfigPath")} /></div>
                            <div style={{ marginBottom: 10 }}><Input key={tmpData.hadoopHomeDir} addonBefore="hadoopHomeDir" onChange={(e) => this.changeInput("hadoopHomeDir")} /></div>
                            <div style={{ marginBottom: 10 }}><Input key={tmpData.principal} addonBefore="principal" onChange={(e) => this.changeInput("principal")} /></div>
                        </Collapse.Panel>
                    </Collapse>
                </Modal>
            </div>
        )
    }
}

export default AddDataSource