import React from 'react';
import { Checkbox, Col, Row, Button, Modal, message, Select, Input } from 'antd';
import { jrequest } from '../../../utils/request';

const {Search} = Input;

class ReverseProject extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            visible:false,
            confirmLoading:false,

            dbs:[],               //数据源待选列表
            selectedDb:null,      //选择了的数据源id
            selectedSchema:null,  //选择了的Schema id
            schemas:[],           //schema待选列表
            tables:{},            //table待选列表，选择了shema后后去，结构是{id:name}
            allTables:{},         //所有的table，用于搜索时与已搜索的区分
            selectedTable:[],     //选择了的表id
            clickedAll: false,    //是否点击了全选
        }

        //初始化获取dbs
        jrequest({
            url:"/meta/children",
            method:"get",
            data:{prtId:"DB"}
        }).then((result) => {
            this.state.dbs = result.data;
        });
    }

    componentWillReceiveProps(props) {
        jrequest({
            url:"/meta/children",
            method:"get",
            data:{prtId:"DB"}
        }).then((result) => {
            let dbs = result.data;
            this.setState({
                dbs: dbs,
                selectedDb: null,
                selectedSchema:null,  
                schemas:[],           
                tables:{},           
                allTables:{},         
                selectedTable:[],     
                clickedAll: false,
            })
        });
    }

    //数组分组
    groupArray(data, cols) {
        const list = [];
        let current = [];

        data.forEach(t => {
            current.push(t);
            if (current.length === cols) {
                list.push(current);
                current = [];
            }
        });
    
        if (current.length) {
            list.push(current);
        }
        return list;
    }

    onOk = () => {
        this.setState({
            confirmLoading:true
        });
        jrequest({
            url:"/meta/importPdms",
            method:"post",
            data:{
                ids:this.state.selectedTable,
                prtId: this.props.realNode.id
            }
        }).then((result) => {
            if(result.code !== 1) {
                message.success("导入成功");
                this.setState({
                    confirmLoading:false,
                    visible:false
                });
                //成功后的回调
                this.props.afterOk(true);
            } else {
                message.error("导入失败");
                this.setState({
                    confirmLoading:false,
                });
            }
        });
    }

    onCancel = () => {
        this.setState({
            visible:false
        });
    }

    //获取渲染的数据源列表
    getDbs = () => {
        let datasource = this.state.dbs.map(db => {
            return (
                <Select.Option key={db.id} value={db.id}>{db.name}</Select.Option>
            );
        });
        return datasource;
    }

    //获取渲染的Schema列表
    getSchemas = () => {
        let schemas = this.state.schemas;
        let list = schemas.map(schema => {
            return (
                <Select.Option key={schema.id} value={schema.id}>{schema.name}</Select.Option>
            );
        });
        return list;
    }

    //搜索表
    searchTable = (value) => {
        if(value === '') {
            let allTables = this.state.allTables;
            this.setState({
                tables: allTables
            });
        } else {
            value = value.toLowerCase();
            let kv = {};
            let tables = Object.keys(this.state.allTables).filter(id => {
                let name = this.state.allTables[id];
                if(name) {
                    name = name.toLowerCase();
                    return name.includes(value);
                } else {
                    return false;
                }
            }).forEach((id) => {
                kv[id] = this.state.allTables[id];
            });
            this.setState({
                tables:kv
            });
        }
    }

    onClickTable = (id) => {
        let selectedTable = this.state.selectedTable;
        let index = selectedTable.indexOf(id);
        if(index > -1){
            selectedTable.splice(index,1);
        } else {
            selectedTable.push(id);
        }
        this.setState({
            selectedTable: selectedTable
        });
    }

    isSelected = (id) => {
        let selectedTable = this.state.selectedTable;
        return selectedTable.indexOf(id) > -1;
    }

    //获取渲染的表
    getTables = () => {
        let tables = this.state.tables;
        let grouped = this.groupArray(Object.keys(tables), 5);
        let checkBoxs = grouped.map((group,idx) => {
            let cols = group.map(id => {
                let name = tables[id];
                return (
                    <Col span={4} key={id} style={{overflow:"hidden",textOverflow: "ellipsis",whiteSpace: "nowrap"}} title={name}>
                        <Checkbox key={"cbx_"+id} value={id} onChange={(e) => this.onClickTable(id)} checked={this.isSelected(id)}>{name}</Checkbox>
                    </Col>
                );
            });
            return (
                <Row key={"row_"+idx}>
                    {cols}
                </Row>
            );
        });
        return (
            <div>
                {checkBoxs}
            </div>
        );
    }

    onClickAll = () => {
        let allIds = Object.keys(this.state.allTables);
        if(this.state.clickedAll) {
            this.setState({
                selectedTable:[],
                clickedAll : false
            });
        } else {
            this.setState({
                selectedTable:allIds,
                clickedAll : true
            });
        }
    }

    //从后台获取Schema
    onSelectDb = (value) => {
        jrequest({
            url:"/meta/children",
            method:"get",
            data:{prtId:value}
        }).then((result) => {
            let children = result.data;
            this.setState({
                selectedDb: value,
                selectedSchema: null,
                schemas: children,
                tables: {}
            });
        });
    }

    //从后台获取Table
    onSelectSchema = (value) => {
        jrequest({
            url:"/meta/detail",
            method:"get",
            data:{id:value}
        }).then((result) => {
            let nodeDetail = result.data;
            let children = nodeDetail.children;
            this.setState({
                selectedSchema: value,
                tables: children,
                allTables: children
            });
        });
    }

    render() {
        return (
            <div>
                <Button icon="project" type="primary" onClick={() => {
                    this.setState({
                        visible: true
                    });
                }}>反向工程</Button>

                <Modal
                    visible={this.state.visible}
                    onOk={this.onOk}
                    okText="导入"
                    width="80%"
                    onCancel={this.onCancel}
                    confirmLoading={this.state.confirmLoading}>
                    
                    <Row>
                        <Col span={5}>
                            选择数据源:<Select defaultValue="请选择" onChange={this.onSelectDb} style={{width:"200px"}}>
                                {this.getDbs()}
                            </Select>
                        </Col>
                        <Col span={5}>
                            选择Schema:<Select defaultValue={this.state.selectedSchema ? this.state.selectedSchema : "请选择"} 
                                onChange={this.onSelectSchema} style={{width:"200px"}}>
                                {this.getSchemas()}
                            </Select>
                        </Col>
                        <Col span={5}>
                            <div style={{width:"200px"}}>
                            <Search placeholder="查找" onSearch={this.searchTable}/>
                            </div>
                        </Col>
                        <Col span={5}>
                            <Button type="primary" onClick={this.onClickAll}>全选</Button>
                        </Col>
                    </Row>
                    <Row style={{maxHeight:"500px", overflowX:"scroll"}}>
                        {this.getTables()}
                    </Row>
                </Modal>
            </div>
        );
    }
}

export default ReverseProject