import React from 'react';
import DetailHeader from './DetailHeader';
import { Checkbox, Col, Row, Input, Button, Table, Tabs, Select, Divider, Icon, message, Modal } from 'antd';
import { jrequest } from 'utils/request';

class PDMDetail extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            realNode: props.nodeDetail.realNode,
            columns: props.nodeDetail.columns,
            relations: props.nodeDetail.relations,
            isTypeSelect: {},
            selectKeys:[], //选择了的行key
            selectRelKeys:[], //选择了的关系行key
            tableIdNameMap:{},

            saveLoading: false,
            newSaveLoading: false
        }

        this.staticData = {
            realNode: props.nodeDetail.realNode,
            columns: props.nodeDetail.columns,
            relations: props.nodeDetail.relations
        }

        this.newCount = 0; //用于生成新行的key

        //获取关联关系的id和name键值
        jrequest({
            url:"/meta/children",
            method:"get",
            data:{prtId: props.prtId}
        }).then((result) => {
            let list = result.data;
            let kv = {};
            list.forEach(element => {
                kv[element.id] = element.name;
            });
            this.state.tableIdNameMap = kv;
        });
    }

    componentWillReceiveProps(props) {
        this.setState({
            realNode: props.nodeDetail.realNode,
            columns: props.nodeDetail.columns,
            relations: props.nodeDetail.relations,
            isTypeSelect: {},
            selectKeys: []
        });
        this.staticData.realNode = props.nodeDetail.realNode;
        this.staticData.columns = props.nodeDetail.columns;
        this.staticData.relations = props.nodeDetail.relations;

        //重新获取关联关系的id和name键值
        jrequest({
            url:"/meta/children",
            method:"get",
            data:{prtId: props.prtId}
        }).then((result) => {
            let list = result.data;
            let kv = {};
            list.forEach(element => {
                kv[element.id] = element.name;
            });
            this.setState({
                tableIdNameMap: kv
            });
        });
    }

    //字段类型的渲染框，select或者input
    createTypeSelectOrInput = (text, row, index) => {
        if (this.state.isTypeSelect[index] == null || this.state.isTypeSelect[index] === true) {
            return (
                //这里需要加一层div然后将鼠标默认事件给停掉
                <div onMouseDown={(e) => { e.preventDefault(); return false;}}>
                    <Select key={row.id + "-type-elect"} defaultValue={row.type} showSearch
                        style={{ minWidth: "100%" }}
                        //onChange事件，修改静态值
                        onChange={(value) => {
                            let row = this.staticData.columns[index];
                            row.type = value;
                        }}
                        //填写选择时的事件
                        filterOption={(input, option) =>
                            option.props.children.toLowerCase().indexOf(input.toLowerCase()) >= 0
                        }
                        //扩展下来列表，切换渲染
                        dropdownRender={menu => (
                            <div>
                                {menu}
                                <Divider style={{ margin: '4px 0' }} />
                                <Button type="link" style={{ padding: '8px', cursor: 'pointer' }} onClick={() => {
                                    let typeSelect = this.state.isTypeSelect;
                                    typeSelect[index] = false;
                                    //修改的话要将静态的也给设置到状态上
                                    this.setState({
                                        //其他好像不需要更新
                                        //realNode: this.staticData.realNode,
                                        columns: this.staticData.columns,
                                        //relations: this.staticData.relations,
                                        isTypeSelect: typeSelect
                                    });
                                }}>
                                    <Icon type="edit" theme="filled" /> 手动输入
                            </Button>
                            </div>
                        )}
                    >
                        <Select.Option value="VARCHAR">VARCHAR</Select.Option>
                        <Select.Option value="CHAR">CHAR</Select.Option>
                        <Select.Option value="INT">INT</Select.Option>
                        <Select.Option value="LONG">LONG</Select.Option>
                        <Select.Option value="DECIMAL">DECIMAL</Select.Option>
                        <Select.Option value="FLOAT">FLOAT</Select.Option>
                        <Select.Option value="NUMBER">NUMBER</Select.Option>
                        <Select.Option value="DATE">DATE</Select.Option>
                        <Select.Option value="DATETIME">DATETIME</Select.Option>
                        <Select.Option value="TEXT">TEXT</Select.Option>
                    </Select>
                </div>
            )
        } else {
            let row = this.staticData.columns[index];
            return (
                <Input defaultValue={row.type} onChange={(e) => {
                    row.type = e.target.value;
                }} />
            );
        }
    }

    //获取关系列信息
    getRelationDef = () => {
        let columnDef = [{
            title:"关联关系说明",
            dataIndex:"cmt",
            key:"cmt",
            render: (text, row, index) =>
                <Input defaultValue={text} onChange={(e) => {
                    let row = this.staticData.relations[index];
                    row.cmt = e.target.value;
                }} />
        },{
            title:"引用目标表",
            dataIndex:"dest",
            key:"dest",
            render: (text, row, index) => {
                let children = Object.keys(this.state.tableIdNameMap).map(child => {
                    return (
                        <Select.Option key={child} value={child}>{this.state.tableIdNameMap[child]}</Select.Option>
                    );
                });
                return (
                    <Select defaultValue={text} onChange={(value) => {
                        let row = this.staticData.relations[index];
                        row.dest = value;
                    }}>
                        {children}
                    </Select>
                );
            }
        },{
            title:"引用目标字段",
            dataIndex:"destName",
            key:"destName",
            render: (text, row, index) => 
                <Input defaultValue={text} onChange={(e) => {
                    let row = this.staticData.relations[index];
                    row.destName = e.target.value;
                }} />
        },{
            title:"引用源表",
            dataIndex:"src",
            key:"src",
            render: (text, row, index) => {
                let children = Object.keys(this.state.tableIdNameMap).map(child => {
                    return (
                        <Select.Option key={child} value={child}>{this.state.tableIdNameMap[child]}</Select.Option>
                    );
                });
                return (
                    <Select defaultValue={this.state.realNode.id} onChange={(value) => {
                        let row = this.staticData.relations[index];
                        row.src = value;
                    }}>
                        {children}
                    </Select>
                );
            }
        },{
            title:"引用源字段",
            dataIndex:"srcName",
            key:"srcName",
            render: (text, row, index) => 
                <Input defaultValue={text} onChange={(e) => {
                    let row = this.staticData.relations[index];
                    row.srcName = e.target.value;
                }} />
        },{
            title:"引用备注",
            dataIndex:"relDtl",
            key:"relDtl",
            render: (text, row, index) => 
                <Input defaultValue={text} onChange={(e) => {
                    let row = this.staticData.relations[index];
                    row.relDtl = e.target.value;
                }} />
        },{
            title:"引用类型",
            dataIndex:"relType",
            key:"relType",
            render: (text, row, index) => 
                <Input defaultValue={text} onChange={(e) => {
                    let row = this.staticData.relations[index];
                    row.relType = e.target.value;
                }} />
        }]

        return columnDef;
    }

    getRelSelection = () => {
        let rowSelection = {
            onChange: (selectedRowKeys, selectedRows) => {
                this.setState({
                    selectRelKeys: selectedRowKeys
                });
            },
        };
        return rowSelection;
    }

    //获取字段的列信息
    getColumnDef = () => {
        let columnDef = [{
            title: "字段",
            dataIndex: 'name',
            key: "name",
            width: "15%",
            render: (text, row, index) => <Input defaultValue={text} onChange={(e) => {
                let row = this.staticData.columns[index];
                row.name = e.target.value;
            }} />
        }, {
            title: "类型",
            dataIndex: "type",
            key: "type",
            width: "10%",
            render: this.createTypeSelectOrInput
        }, {
            title: "属性名称",
            dataIndex: "cnName",
            key: "cnName",
            width: "15%",
            render: (text,row,index) =>
                <Input defaultValue={text} onChange={(e) => {
                    let row = this.staticData.columns[index];
                    row.cnName = e.target.value;
                }} />
        }, {
            title: "属性描述",
            dataIndex: "comment",
            key: "comment",
            width: "10%",
            render: (text,row,index) => 
                <Input defaultValue={text} onChange={(e) => {
                    let row = this.staticData.columns[index];
                    row.comment = e.target.value;
                }} />
        }, {
            title: "长度",
            dataIndex: "length",
            key: "length",
            width: "8%",
            render: (text,row,index) => 
                <Input type="number" defaultValue={text} onChange={(e) => {
                    let row = this.staticData.columns[index];
                    row.length = parseInt(e.target.value);
                }} />
        }, {
            title: "精度",
            dataIndex: "prec",
            key: "prec",
            width: "8%",
            render: (text,row,index) => 
                <Input type="number" defaultValue={text} onChange={(e) => {
                    let row = this.staticData.columns[index];
                    row.prec = parseInt(e.target.value);
                }} />
        }, {
            title: "可为空",
            dataIndex: "nullable",
            key: "nullable",
            width: "10%",
            render: (text, row, index) => 
                <Checkbox checked={row.entry.nullable} onChange={(e) => {
                    let row = this.staticData.columns[index];
                    row.entry.nullable = e.target.checked;
                    //修改的话要将静态的也给设置到状态上
                    this.setState({
                        columns: this.staticData.columns,
                    });
                }} />
        }, {
            title: "主键",
            dataIndex: "pk",
            key: "pk",
            width: "5%",
            render: (text, row, index) => 
                <Checkbox checked={row.entry.pk} onChange={(e) => {
                    let row = this.staticData.columns[index];
                    row.entry.pk = e.target.checked;
                    //修改的话要将静态的也给设置到状态上
                    this.setState({
                        columns: this.staticData.columns,
                    });
                }} />
        }, {
            title: "备注",
            dataIndex: "remark",
            key: "remark",
            render: (text, row, index) => 
                <Input.TextArea defaultValue={text} onChange={(e) => {
                    let row = this.staticData.columns[index];
                    row.remark = e.target.value;
                }} />
        }];
        return columnDef;
    }

    //获取选择行的事件及操作
    getRowSelection = () => {
        let rowSelection = {
            onChange: (selectedRowKeys, selectedRows) => {
                this.setState({
                    selectKeys: selectedRowKeys
                });
            },
        };
        return rowSelection;
    }

    //保存
    onBtnSave = () => {
        this.setState({
            saveLoading:true
        });
        let data = {
            realNode: this.staticData.realNode,
            columns: this.staticData.columns,
            relations: this.staticData.relations
        }

        let id = this.state.realNode.id;
        jrequest({
            url:"/meta/pdmAll",
            method:"put",
            data:data
        }).then((result) => {
            this.setState({
                realNode: this.staticData.realNode,
                columns: this.staticData.columns,
                relations: this.staticData.relations,
                saveLoading: false
            });
            message.success("保存成功");
            this.props.refreshTreeNode(true);

            //获取数据刷新
            jrequest({
                url:"/meta/detail",
                method:"get",
                data:{id:id}
            }).then((result) => {
                let newRealNode = result.data.realNode;
                let newColumns = result.data.columns;
                let newRelations = result.data.relations;
                this.setState({
                    realNode: newRealNode,
                    columns: newColumns,
                    relations: newRelations,
                    isTypeSelect: {},
                    selectKeys:[],
                    selectRelKeys:[],
                });
                this.staticData.realNode = newRealNode;
                this.staticData.columns = newColumns;
                this.staticData.relations = newRelations;
            });
        }).catch((error) => {
            this.setState({
                saveLoading: false
            });
            message.error("保存失败");
        });
    }

    //删除表
    onBtnDel = () => {
        let onOkHandler = () => {
            let id = this.state.realNode.id;

            jrequest({
                url:"/meta/node",
                method:"delete",
                data: {'id':id}
            }).then((result) => {
                //成功之后的回调
                this.props.refreshTreeNode(false);
            });
        }

        Modal.confirm({
            title:"确定要删除本表？过程不可逆",
            okType:"danger",
            onOk() {
                onOkHandler();
            }
        });
    }

    //添加字段
    onBtnAddCol = () => {
        let row = {
            prtId: this.state.realNode.id,
            nodeType:"model.PhysicalModelClm",
            namespace: this.state.realNode.namespace,
            entry: {
                pk:false,
                nullable:false
            },
            pk:false,
            nullable:false,
            dataKind:"model.PhysicalModelClm",
            parent: false,
            key: this.newCount+""
        };
        this.newCount += 1;

        this.staticData.columns.push(row);
        this.setState({
            columns: this.staticData.columns
        });
    }

    //删除字段
    onBtnDelCol = () => {
        let selects = this.state.selectKeys;
        let newColumns = this.staticData.columns.filter(col => selects.indexOf(col.key) === -1)
        
        this.setState({
            columns: newColumns
        });
        this.staticData.columns = newColumns;
    }

    //添加关系
    onBtnAddRel = () => {
        let row = {
            src: this.state.realNode.id,
            key: this.newCount+""
        }
        this.newCount += 1;

        this.staticData.relations.push(row);
        this.setState({
            relations: this.staticData.relations
        });
    }

    //删除关系
    onBtnDelRel = () => {
        let selects = this.state.selectRelKeys;
        let newRelations = this.staticData.relations.filter(col => selects.indexOf(col.key) === -1)
        
        this.setState({
            relations: newRelations
        });
        this.staticData.relations = newRelations;
    }


    render() {
        let columnDefs = this.getColumnDef();
        let rowSelection = this.getRowSelection();
        let relColumnDefs = this.getRelationDef();
        let relSelection = this.getRelSelection();

        let allColCount = <span style={{marginLeft:"10px"}}>总行数:{this.state.columns.length}</span>;
        let allRelCount = <span style={{marginLeft:"10px"}}>总行数:{this.state.relations.length}</span>;
        let topButtons = (
            <Row>
                <Col span={2}>
                    <Button icon={this.state.saveLoading ? "loading":"save"} type="primary" onClick={this.onBtnSave}>保存</Button>
                </Col>
                <Col span={2}>
                    <Button icon="delete" icon="delete" type="danger" onClick={this.onBtnDel}>删除表</Button>
                </Col>
            </Row>
        );
        let tableButtons = (
            <Row>
                <Col span={2}>
                    <Button icon="plus-circle" type="primary" onClick={this.onBtnAddCol}>添加字段</Button>
                </Col>
                <Col span={2}>
                    <Button icon="minus-circle" type="danger" onClick={this.onBtnDelCol}>删除字段</Button>
                </Col>
                {allColCount}
            </Row>
        );
        let relButtons = (
            <Row>
                <Col span={2}>
                    <Button icon="plus-circle" type="primary" onClick={this.onBtnAddRel}>添加关系</Button>
                </Col>
                <Col span={2}>
                    <Button icon="minus-circle" type="danger" onClick={this.onBtnDelRel}>删除关系</Button>
                </Col>
                {allRelCount}
            </Row>
        );

        return (
            <div style={{ backgroundColor: "white" }}>
                {topButtons}
                <Tabs type="card" style={{ marginTop: "20px" }} >
                    <Tabs.TabPane tab="属性信息" key="columns">
                        {tableButtons}
                        <Table
                            columns={columnDefs}
                            dataSource={this.state.columns}
                            bordered
                            pagination={false}
                            scroll={{ y: "600px", x:"1300px" }}
                            rowSelection={rowSelection}
                        />
                    </Tabs.TabPane>
                    <Tabs.TabPane tab="关系信息" key="relations">
                        {relButtons}
                        <Table
                            columns={relColumnDefs}
                            dataSource={this.state.relations}
                            bordered
                            pagination={false}
                            scroll={{ y: "600px" }}
                            rowSelection={relSelection}
                        />
                    </Tabs.TabPane>
                    <Tabs.TabPane tab="基本信息" key="info">
                        <div style={{ maxWidth: "550px", marginLeft:"25px" }}>
                            <div style={{ marginBottom: 10 }}>
                                <Input key={this.state.realNode.name} addonBefore="名称" defaultValue={this.state.realNode.name} onChange={(e) => {
                                    this.staticData.realNode.name = e.target.value;
                                }} />
                            </div>
                            <div style={{ marginBottom: 10 }}>
                                <Input key={this.state.realNode.name} addonBefore="中文名称" defaultValue={this.state.realNode.cnName} onChange={(e) => {
                                    this.staticData.realNode.cnName = e.target.value;
                                }} />
                            </div>
                            <div style={{ marginBottom: 10 }}>
                                <Input key={this.state.realNode.name} addonBefore="描述" defaultValue={this.state.realNode.comment} onChange={(e) => {
                                    this.staticData.realNode.comment = e.target.value;
                                }} />
                            </div>
                            <div style={{ marginBottom: 10 }}>
                                {/* <Input key={this.state.realNode.name} addonBefore="备注" defaultValue={this.state.realNode.remark} onChange={(e) => {
                                    this.staticData.realNode.remark = e.target.value;
                                }} /> */}
                                <Select size="large" placeholder="选择表类型" key={this.state.realNode.name} defaultValue={this.state.realNode.remark} onChange={(value) => {
                                    this.staticData.realNode.remark = value
                                }}>
                                     <Option value=""> 表类型 </Option>
                                     <Option value="basicBehavior">行为记录表</Option>
                                     <Option value="basicData">基础信息表</Option>
                                </Select>
                            </div>
                        </div>
                    </Tabs.TabPane>
                </Tabs>

            </div>
        )
    }
}

export default PDMDetail