import React from 'react';
import DetailHeader from './DetailHeader';
import { Checkbox, Col, Row, Button, Modal, message } from 'antd';
import AddFolder from "./Modals/AddFolder.js";
import DeleteFolder from './Modals/DeleteFolder.js';
import { jrequest } from '../../utils/request';
import AddPDM from './Modals/AddPDM.js';
import ReverseProject from './Modals/ReverseProject';

class PackageDetail extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            nodeDetail: props.nodeDetail,
            selectedChild : [],    //选择了的子选择
            clickSelectAll: false, //点击了全选
            children:[]            //基本是固定的，所有子选项
        }

        this.state.children = Object.keys(props.nodeDetail.children);
    }

    componentWillReceiveProps(props) {
        this.setState({
            nodeDetail: props.nodeDetail,
            selectedChild : [],
            clickSelectAll: false,
            children: Object.keys(props.nodeDetail.children)
        });
    }

    //数组分组
    groupArray(data, cols) {
        const list = [];
        let current = [];

        data.forEach(t => {
            current.push(t);
            if (current.length === cols) {
                list.push(current);
                current = [];
            }
        });
    
        if (current.length) {
            list.push(current);
        }
        return list;
    }


    //点击选择child
    onClickChild = (id) => {
        let selected = this.state.selectedChild;
        let index = selected.indexOf(id);
        if(index > -1) {
            selected.splice(index,1);
        } else {
            selected.push(id);
        }
        this.setState({
            selectedChild: selected
        });
    }

    //是否已经选择了
    isSelected = (id) => {
        let index = this.state.selectedChild.indexOf(id);
        return index > -1;
    }

    //全选按钮
    onAllSelect = () => {
        let children = this.state.children;
        if(this.state.clickSelectAll){
            this.setState({
                selectedChild: [],
                clickSelectAll: false
            });
        } else {
            this.setState({
                selectedChild: children,
                clickSelectAll: true
            });
        }
    }

    //选择表后删除的
    onBtnDelTbl = (e) => {
        if(this.state.selectedChild.length === 0){
            message.error("请选择表");
            return;
        }

        let onOkHandler = () => {
            let selected = this.state.selectedChild;
            jrequest({
                url:"/meta/nodes",
                method:"delete",
                data: selected
            }).then((result) => {
                let newChildren = this.state.children.filter(id => selected.indexOf(id) !== -1)
                let node = this.state.nodeDetail;
                let nodeChildren = node.children;
                selected.forEach((id) => {
                    delete nodeChildren[id];
                });
                this.props.refreshTreeNode(true);
                this.setState({
                    children: newChildren,
                    nodeDetail: node
                });
            });
        }

        Modal.confirm({
            title:"确定要删除选择的表？过程不可逆",
            okType:"danger",
            onOk() {
                onOkHandler();
            }
        });
    }

    //添加后刷新一下
    afterAddPDM = (isActionAdd) => {
        let id = this.state.nodeDetail.realNode.id;

        jrequest({
            url:"/meta/detail",
            method:"get",
            data:{id:id}
        }).then((result) => {
            let node = result.data;
            this.props.refreshTreeNode(isActionAdd,node);
        });
    }

    render () {
        let nodeDetail = this.state.nodeDetail;
        let buttons = [
            <div key={this.props.nodeDetail.nodeType+"-caddF"} style={{float:"left", marginRight:"10px"}}>
                <AddFolder key={this.props.nodeDetail.nodeType+"-addF"} realNode={this.props.nodeDetail.realNode} afterOk={this.props.refreshTreeNode}/>
            </div>, 
            <div key={this.props.nodeDetail.nodeDetail+"-cdelF"} style={{float:"left", marginRight:"10px"}}>
                <DeleteFolder key={this.props.nodeDetail.nodeType+"-delF"} realNode={this.props.nodeDetail.realNode} afterOk={this.props.refreshTreeNode}/>
            </div>
        ];
        if(nodeDetail.nodeType === 'PDM') {
            buttons.push(
                <div key="c-click-resverse" style={{float:"left", marginRight:"10px"}}>
                    <ReverseProject key={this.props.nodeDetail.nodeType+"-reverseP"} realNode={this.props.nodeDetail.realNode} afterOk={this.afterAddPDM} />
                </div>,
                // {label:"添加表", icon:"file-add", type:"primary", onClick:(e) => {}},
                <div key="c-click-add" style={{float:"left", marginRight:"10px"}}>
                    <AddPDM key={this.props.nodeDetail.realNode.id+"-addP"} realNode={this.props.nodeDetail.realNode} afterOk={this.afterAddPDM}/>
                </div>,
                <div key="c-click-del" style={{float:"left", marginRight:"10px"}}>
                    <Button style={{marginRight:"8px"}} key="clike-del" icon="delete" type="danger" onClick={this.onBtnDelTbl}>删除表</Button>
                </div>,
                // {label:"模型部署", icon:"thunderbolt", type:"primary", onClick:(e) => {}},
                <div key="c-click-all" style={{float:"left", marginRight:"10px"}}>
                    <Button style={{marginRight:"8px"}} key="click-all" icon="check-square" type="primary" onClick={(e) => {this.onAllSelect()}}>全选</Button>
                </div>
            );
        }

        let children = Object.keys(nodeDetail.children).map(key => {
            let entry = {id:key,name:nodeDetail.children[key]};
            return entry;
        });
        let grouped = this.groupArray(children, 5);
        let checkBoxs = grouped.map((group,idx) => {
            let cols = group.map(i => {
                return (
                    <Col span={4} key={i["id"]} style={{overflow:"hidden",textOverflow: "ellipsis",whiteSpace: "nowrap"}} title={i["name"]}>
                        <Checkbox key={"cbx_"+i["id"]} value={i["id"]} onChange={(e) => this.onClickChild(i["id"])} checked={this.isSelected(i["id"])}>{i["name"]}</Checkbox>
                    </Col>
                );
            });
            return (
                <Row key={"row_"+idx}>
                    {cols}
                </Row>
            );
        });
        return (
            <div>
                <Row>
                    {buttons}
                </Row>
                {checkBoxs}
            </div>
        );
    }
}

export default PackageDetail