import React from 'react';
import DetailHeader from './DetailHeader';
import { Checkbox, Col, Row, Input, Button } from 'antd';
import MetaTableDetail from "./MetaTableDetail.js";
import { jrequest } from '../../utils/request';

const {Search} = Input;

class SchemaDetail extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            selectedChild : [],    //选择了的子选择
            clickSelectAll: false, //点击了全选
            children:[],            //基本是固定的，所有子选项
            searchChildren:[],

            selectedNode:null, //点击后获取的节点
        }

        this.state.children = Object.keys(props.nodeDetail.children);
        this.state.searchChildren = Object.keys(props.nodeDetail.children).map(key => {
            let entry = {id:key,name:props.nodeDetail.children[key]};
            return entry;
        });
    }

    componentWillReceiveProps(props) {
        let newChildren =  Object.keys(props.nodeDetail.children);
        let newSearchChildren = Object.keys(props.nodeDetail.children).map(key => {
            let entry = {id:key,name:props.nodeDetail.children[key]};
            return entry;
        });

        this.setState({
            selectedChild : [],    
            clickSelectAll: false, 
            children: newChildren,            
            searchChildren: newSearchChildren,

            selectedNode:null,
        });
    }

    //数组分组
    groupArray(data, cols) {
        const list = [];
        let current = [];

        data.forEach(t => {
            current.push(t);
            if (current.length === cols) {
                list.push(current);
                current = [];
            }
        });
    
        if (current.length) {
            list.push(current);
        }
        return list;
    }

    //搜索按钮
    searchTable = (value) => {
        let children = Object.keys(this.props.nodeDetail.children).map(key => {
            let entry = {id:key,name:this.props.nodeDetail.children[key]};
            return entry;
        });
        if(value === ''){
            children = Object.keys(this.props.nodeDetail.children).map(key => {
                let entry = {id:key,name:this.props.nodeDetail.children[key]};
                return entry;
            });
        } else {
            value = value.toLowerCase();
            children = children.filter(i => {
                let name = i.name.toLowerCase();
                return name.includes(value);
            });
        }
        this.setState({
            searchChildren: children
        });
    }

    onClickTbl = (id) => {
        jrequest({
            url:"/meta/detail",
            method:"get",
            data:{id:id}
        }).then((result) => {
            let node = result.data;
            this.setState({
                selectedNode: node
            });
        });
    }

    render() {
        let searchBar = (
            <Row>
                <Col span={6} offset={18}>
                    <Search placeholder="查找" onSearch={this.searchTable} style={{ width: 200 }}/>
                </Col>
            </Row>
        );

        let children = this.state.searchChildren;
        let grouped = this.groupArray(children, 5);
        let checkBoxs = grouped.map((group,idx) => {
            let cols = group.map(i => {
                return (
                    <Col span={4} key={i["id"]} style={{overflow:"hidden",textOverflow: "ellipsis",whiteSpace: "nowrap"}} title={i["name"]}>
                        <Button key={"btn_"+i["id"]} icon="profile" type="link" onClick={() => this.onClickTbl(i["id"])}>{i["name"]}</Button>
                    </Col>
                );
            });
            return (
                <Row key={"row_"+idx}>
                    {cols}
                </Row>
            );
        });
        if(this.state.selectedNode) {
            return (
                <MetaTableDetail nodeDetail={this.state.selectedNode} />
            );
        } else {
            return (
                <div>
                    {searchBar}
                    {checkBoxs}
                </div>
            );
        }
    }
}

export default SchemaDetail