import React from 'react';
import PropTypes from 'prop-types';
import { Icon, Layout ,notification,Tabs,Menu,message} from 'antd';
import { Link } from 'react-router-dom';
import styles from './layoutTwo.less';
import { connect } from 'dva';
import { config,util, arrayToTree,request } from 'utils';
import { withRouter } from 'dva/router';
import { Helmet } from 'react-helmet';
import { Loader } from 'components';
import UserAvatar from '../components/user/UserAvatar';
import Sider from '../components/sider/Sider';
import Breadcrumb from '../components/Bread/Breadcrumb';
// import SkinToolbox from 'components/SkinToolbox';
import MessageBar from '../components/message/messagebar';
import TopTabs from './toptaps/toptaps';
// const { TabPane } = Tabs;
import * as menuService from 'services/menus';

const { Header, Content } = Layout;
const { openPages, iconFontJS, iconFontCSS, logo } = config;

//var globalPanes = [];
let activeKey = "252";
const LayoutTwo = ({ children, dispatch, app, loading, location }) => {
  let { pathname } = location;
  const { menus, menuHash, menuPath, openKeys, selectedKeys,topMenus,topOpenKeys } = app;
  const { globalPanes } = app;

  const siderProps = {
    menus,
    openKeys,
    selectedKeys,
    onOpenChange(openKeys) {
      dispatch({ type: 'app/updateState', payload: { openKeys: openKeys } })
    },
    onSelect(selectedKey) {
      let menuPath = [];
      let menuId = parseInt(selectedKey.replace('tMenu_', ''));
      let menu = menuHash[menuId];
      menuPath.unshift(menu);
      if (menu != null && menu.parentId) {
        menuPath.unshift(menuHash[menu.parentId]);
      }
      dispatch({
        type: 'app/updateState', payload: {
          selectedKeys: [selectedKey],
          menuPath: menuPath
        }
      })
    }
  }

  const onChangeTheme = theme => {
    //$$.setStore('theme', theme);
    //this.setState({
    //  theme
    //});
  };

  const onClick = function(clickedKeys) {
    let parentId = clickedKeys[0];
    if(clickedKeys[0] === "264"){
      parentId = "";
    }
    request({
      url: config.api.menus,  
      method: 'get',
      data: {type:"U",ParentId:parentId}
    }).then(menus => {
      let menuHash = {}, openKeys = [];
      menus = util.json.generateNode(menus);

      menus.forEach(item => { menuHash[item.id] = item });
      if (menus.length > 0) {
        openKeys = [`sMenu_${menus[0].id}`];
      }
      let menuTree = arrayToTree(menus, 'id', 'parentId');
      dispatch({
        type: 'app/updateState', payload: {
          menus: menuTree,
          cfMenu: menuTree.length > 0 ? menuTree[0] : {},
          menuHash,
          openKeys,
          topOpenKeys: clickedKeys
        }
      });
    });
  }

  const remove = function(menuPath,lastIndex){
    globalPanes.splice(lastIndex+1,1);
    // 恢复面包屑
    dispatch({
      type: 'app/updateState', payload: {
        menuPath: menuPath,
        globalPanes:globalPanes
      }
    });
  }

  if (openPages && openPages.includes(pathname)) {
    return (<div>
      <Loader fullScreen spinning={loading.effects['app/query']} />
      {children}
    </div>)
  }
  
  const curUser = util.auth.getUser()
  let rolename = curUser == null?"":curUser.rolename;
  
                //<Link to="/purchase/bird-purchaseplan" title="首页" className={styles.button}>
                //  <Icon type="home" style={{ fontSize: 20, marginLeft: 20 }} />
                //</Link>
  // /synthesis/dashboard
        //<SkinToolbox   onchange={onChangeTheme}/>

  //处理传递给顶部可关闭导航tab的数据
  let menuData = menuPath[menuPath.length-1];
  // let panes = globalPanes?globalPanes:[];
  
  const targetPanes = globalPanes.filter(pane => pane.key == (menuData == undefined?"252":menuData.id)); //当前页面panel
  if(targetPanes&&targetPanes.length>0){
    activeKey = targetPanes[0].key;
    // globalPanes.map((v,index)=>{
    //   if(v.key === activeKey)
    //   v.content = children;
    // });
    // globalPanes = panes;
  }else{
    if(globalPanes.length >= 8){
      message.error('最多8个');
    }else{

      globalPanes.push({ title: typeof(menuData) === 'undefined'?"仪表盘":menuData.name, content:children, 
      key: typeof(menuData) === 'undefined'?"252":menuData.id,
      menuPath:menuPath,closable:true });
      activeKey = menuData == undefined?"252":menuData.id;
      // globalPanes = panes;
    }
  }

  // const End = ()=>{
  //   return (<TopTabs panes={globalPanes} activeKey={activeKey} refs="toptabs"
  //           remove = {remove}
  //           ></TopTabs>)
  // }
  
  return (
    <Layout className={styles.layoutTwo}>
      <Loader fullScreen spinning={loading.effects['app/query']} />
      <Helmet>
        <title>{config.name}</title>
        <meta name="viewport" content="width=device-width, initial-scale=1.0" />
        <link rel="icon" href={logo} type="image/x-icon" />
        {iconFontJS && <script src={iconFontJS} />}
        {iconFontCSS && <link rel="stylesheet" href={iconFontCSS} />}
      </Helmet>
        <Sider {...siderProps} />
        <Layout>
          <Header style={{ background: '#fff', padding: 0 }}>        
              <div className={styles.rightWarpper}>    
                <MessageBar current={topOpenKeys} menus={topMenus} onClick={onClick}/>           
              {/* {rolename}  */}         
                <UserAvatar/>
              </div>
          </Header>

          <Content>
            <Breadcrumb menuPath={menuPath} />
            {/* {children} */}
            {/* <End /> */}
            <TopTabs panes={globalPanes} activeKey={activeKey} refs="toptabs"
            remove = {remove}
            ></TopTabs>
            
          </Content>
          { <div className={styles.footer}>
            360lj.com © 2018 亮健好药
            <a href="https://m.kuaidi100.com/" target="_blank">快递查询</a>
            </div> }
        </Layout>
      </Layout>

  )
}

LayoutTwo.propTypes = {
  children: PropTypes.element.isRequired,
  location: PropTypes.object,
  dispatch: PropTypes.func,
  app: PropTypes.object,
  loading: PropTypes.object
}
export default withRouter(connect(({ app, loading }) => ({ app, loading }))(LayoutTwo))
// connect( ( { bindActionCreators } ) => ( { bindActionCreators } ) )( TopTabs );
