import React, { Component } from 'react';
import { jrequest } from 'utils/request';
import { Modal, Row, Col, Table, Button, Popconfirm, message } from 'antd';
import { BirdGrid } from 'components/Grid';


import 'antd/dist/antd.css';

class AreaTags extends Component {
    constructor(props) {
        super(props);
        this.state = {
            ObjectId: props.location.query.ObjectId,
            formVisiable: false,
            formConfirmLoading: false,
            gridDatas: [],//根据子组件返回标签全部
            tagList: [
            ],
            current: 1,
            total: 0,
            pageSize:10
        };

        //初始化对象标签列表
        jrequest({
            url: "/areaData/areaDataTag?kw=" + this.state.ObjectId + "&page=" + this.state.current + "&pageSize=" + this.state.pageSize,
            method: "get",
        }).then(result => {
            let tagListTemp = result.data.items;
            this.setState({
                tagList: tagListTemp,
                total: result.data.totalCount
            });
        });

    }

    // // 生命周期函数
    // componentWillMount() {
    //     console.log(this.props.location.query.ObjectId)
    //     this.setState({
    //         ObjectId: this.props.location.query.ObjectId
    //     })
    // }

    // 刷新数据
    refreshData = () => {
        jrequest({
            url: "/areaData/areaDataTag?kw=" + this.state.ObjectId + "&page=" + this.state.current + "&pageSize="+ this.state.pageSize,
            method: "get",
        }).then(result => {
            let tagListTemp = result.data.items;
            this.setState({
                tagList: tagListTemp,
                total: result.data.totalCount
            });
        });
    }

    // 格式化时间
    timestampToTime = (timestamp) => {
        var date = new Date(timestamp);//时间戳为10位需*1000，时间戳为13位的话不需乘1000
        var Y = date.getFullYear() + '-';
        var M = (date.getMonth() + 1 < 10 ? '0' + (date.getMonth() + 1) : date.getMonth() + 1) + '-';
        var D = (date.getDate() < 10 ? '0' + date.getDate() : date.getDate()) + ' ';
        var h = (date.getHours() < 10 ? '0' + date.getHours() : date.getHours()) + ':';
        var m = (date.getMinutes() < 10 ? '0' + date.getMinutes() : date.getMinutes()) + ':';
        var s = (date.getSeconds() < 10 ? '0' + date.getSeconds() : date.getSeconds());
        return Y + M + D + h + m + s;
    }

    // 设置表格标签
    getTableColumnDef = () => {
        return [{
            title: "标签ID",
            dataIndex: "TagId",
            key: "TagId",
            render: (text, record, index) => {
                return <span>{text}</span>
            }
        }, {
            title: "标签名称",
            dataIndex: "TagName",
            key: "TagName",
            render: (text, record, index) => {
                return <span>{text}</span>
            }
        }, {
            title: "类型",
            dataIndex: "TagTypeName",
            key: "TagTypeName",
            render: (text, record, index) => {
                return <span>{text}</span>
            }
        }, {
            title: "时间",
            dataIndex: "TagCrtTime",
            key: "TagCrtTime",
            render: (text, record, index) => {
                return <span>{this.timestampToTime(text)}</span>
            }
        }, {
            title: "状态",
            dataIndex: "IsValid",
            key: "IsValid",
            render: (text, record, index) => {
                return text == "是" ? "有效" : "无效"
            }
        }, {
            title: "操作",
            key: "action",
            render: (text, record, index) =>
                this.state.tagList.length >= 1 ? (
                    <Popconfirm title="是否删除?" onConfirm={() => this.onDeleteTag(record, index)}>
                        <a>删除</a>
                    </Popconfirm>
                ) : null,
        }];
    }

    // 删除标签数据
    onDeleteTag = (record, index) => {
        let relf = this
        console.log(record)
        jrequest({
            url: "/areaData/areaDataTag?id=" + record.id,
            method: "delete",
        }).then(result => {
            if (result.success) {
                message.success("删除成功");
                if ((relf.state.total - 1) % 10 == 0) {
                    relf.setState({
                        current: relf.state.current - 1
                    })
                }

                this.refreshData();
            } else {
                message.error("删除失败");
            }
        });

    }

    /* 弹出框取消事件 */
    cancelClick = () => {
        this.setState({
            formVisiable: false,
            formConfirmLoading: false
        });
    }

    /* 弹出模态框框取消事件 */
    onAddTags = () => {
        this.setState({
            formVisiable: true
        });
    }

    // 表格点击换页
    changePage = (page) => {
        jrequest({
            url: "/areaData/areaDataTag?kw=" + this.state.ObjectId + "&page=" + page + "&pageSize="+ this.state.pageSize,
            method: "get",
        }).then(result => {
            let tagListTemp = result.data.items;
            this.setState({
                current: page,
                tagList: tagListTemp,
                total: result.data.totalCount
            });
        });
    }

    // 显示总条数
    showTotal = () => {
        return '总共' + this.state.total + '条'
    }

    onShowSizeChange = (current, pageSize) => {
        this.setState({
            pageSize: pageSize
        }, () => { this.refreshData()});

    }

    // 保存添加的标签
    saveTagList = (record) => {
        console.log(record)
        record.map((x) => {
            x.id = this.state.ObjectId + "_" + x.TagId;
            delete x._id;
            delete x.checkedValue;
            x.ObjectId = this.state.ObjectId;
            x.TagCrtTime = Date.parse(new Date());
            x.IsValid = "是"
        })
        jrequest({
            url: "/areaData/areaDataTag",
            method: "post",
            data: record
        }).then(result => {
            if (result.success) {
                message.success("保存成功");
                this.refreshData();
            } else {
                message.error("保存失败");
            }
        });
    }


    render() {
        let relf = this;

        //对象列表
        let gridOptionsTag = {
            title: "标签列表",
            url: {
                read: "/web/mem_tag/AllTagList"
            },
            primaryKey: '_id',
            beforeQuery: function (gridDatas, filters, pageIndex, pageSize) {
                let filterDistinct = [];
                if (pageIndex * pageSize > 1000000) {
                    message.error("不能翻页超过" + pageIndex + "页");
                    return false;
                } else
                    return true;
            },
            columns: [
                { title: "标签ID", data: "_id", type: "text", editor: { ap: 'hide', ep: 'hide' }, isRequired: true },
                { title: "标签名称", data: "TagName", type: "text", editor: {}, query: false, query: true },
                { title: "标签类型名称", data: "TagTypeName", type: "text", editor: { ap: 'hide', ep: 'hide' }, hide: true },
                {
                    title: "标签类型", data: "TagTypeId", type: "dropdown", editor: {},
                    source: { url: '/web/Mem_TagType/type?t=' + self.state.randStr }, query: true
                },
                { title: "标签名称", data: "TagName", type: "text", editor: {}, query: false, query: true },
            ],
            checkable: true,
            actions: [{
                name: '确认选择',
                onClick: (e, record, index) => {
                    relf.saveTagList(record)
                    relf.setState({
                        formVisiable: false,
                        formConfirmLoading: false
                    });

                }
            }],
        };

        return (
            <div style={{ padding: '25px', background: 'white' }}>
                <div style={{ padding: '10px 0px' }}>
                    <Button type="primary" onClick={this.onAddTags}>添加标签</Button>
                </div>

                <div>
                    <Table
                        size="small"
                        dataSource={this.state.tagList}
                        bordered
                        pagination={{ pageSize: 10 }}
                        columns={this.getTableColumnDef()}
                        rowKey={record => record.TagId}
                        pagination={{  // 分页
                            current: this.state.current,
                            total: this.state.total,
                            onChange: this.changePage,
                            showTotal: this.showTotal,
                            showSizeChanger: true,
                            showQuickJumper: true,
                            onShowSizeChange: (current, pageSize) => this.onShowSizeChange(current, pageSize)
                        }}
                    />
                </div>

                <div>
                    <Modal title="选择标签"
                        width="70%"
                        visible={relf.state.formVisiable}
                        okButtonProps={{ disabled: true }}
                        onCancel={() => relf.cancelClick()}
                        confirmLoading={relf.state.formConfirmLoading}>

                        <Row>
                            <Col>
                                <BirdGrid gridOption={gridOptionsTag} ref={relf.state.gridDatas}></BirdGrid>
                            </Col>
                        </Row>

                    </Modal>
                </div>
            </div>
        );
    }
}

export default AreaTags;