
import React from 'react';
import { DragDropContext } from 'react-dnd';
import HTML5Backend from 'react-dnd-html5-backend';
import cloneDeep from 'lodash/cloneDeep';
import findIndex from 'lodash/findIndex';
import * as Table from 'reactabular-table';
import * as dnd from 'reactabular-dnd';
import * as resolve from 'table-resolver';
import { Pagination } from 'antd';
import styles from '../../components/Grid/BirdGrid.less';

const rows = [
    {
      id: 1,
      name: {
        first: 'John',
        last: 'Johnson'
      },
      company: 'John Inc.',
      sentence: 'consequatur nihil minima corporis omnis nihil rem'
    },
    {
      id: 2,
      name: {
        first: 'Mike',
        last: 'Mikeson'
      },
      company: 'Mike Inc.',
      sentence: 'a sequi doloremque sed id quo voluptatem voluptatem ut voluptatibus'
    },
    {
      id: 3,
      name: {
        first: 'Jake',
        last: 'Jackson'
      },
      company: 'Jake Inc.',
      sentence: 'sed id quo voluptatem voluptatem ut voluptatibus'
    },
    {
      id: 4,
      name: {
        first: 'Don',
        last: 'Donson'
      },
      company: 'Don Inc.',
      sentence: 'voluptatem voluptatem ut voluptatibus'
    }
  ];
  
  class DragAndDropTable extends React.Component {
    constructor(props) {
      super(props);
  
      let gridOption = this.props.gridOption;
      this.state = {
        columns: [
          {
            props: {
              style: {
                width: 100
              }
            },
            header: {
              label: 'Name',
              props: {
                label: 'Name',
                onMove: o => this.onMoveColumn(o)
              }
            },
            children: [
              {
                property: 'name.first',
                props: {
                  style: {
                    width: 50
                  }
                },
                header: {
                  label: 'First Name',
                  props: {
                    label: 'First Name',
                    onMove: o => this.onMoveChildColumn(o)
                  }
                }
              },
              {
                property: 'name.last',
                props: {
                  style: {
                    width: 50
                  }
                },
                header: {
                  label: 'Last Name',
                  props: {
                    label: 'Last Name',
                    onMove: o => this.onMoveChildColumn(o)
                  }
                }
              }
            ]
          },
          {
            property: 'company',
            props: {
              label: 'Company',
              style: {
                width: 100
              }
            },
            header: {
              label: 'Company',
              props: {
                onMove: o => this.onMoveColumn(o)
              }
            }
          },
          {
            property: 'sentence',
            props: {
              style: {
                width: 300
              }
            },
            header: {
              label: 'Sentence',
              props: {
                label: 'Sentence',
                onMove: o => this.onMoveColumn(o)
              }
            }
          }
        ],
        rows,
        pageIndex: 1,
        pageSize: 15,
        // queryLoading: false,
        // sortField: gridOption.sortField || primaryKey,
        // sortDirection: gridOption.sortDirection || "desc",
        // filterRules: gridOption.filterRules || [],
        gridDatas: {
          totalCount:10,
          items: [],
          sum: {}
        },
      };
  
      this.onRow = this.onRow.bind(this);
      this.onMoveRow = this.onMoveRow.bind(this);
      this.onMoveColumn = this.onMoveColumn.bind(this);
      this.onMoveChildColumn = this.onMoveChildColumn.bind(this);
    }
    
  /* 分页点击事件 */
  pageClick(pageIndex) {
    this.setState({
      pageIndex: pageIndex
    }, this.query);
  }

  /* 每页显示数量改变事件 */
  pageSizeChange(pageSize) {
    this.setState({
      pageIndex: 1,
      pageSize: pageSize
    }, this.query);
  }

  query(){
    
  }
    render() {
      const renderers = {
        header: {
          cell: dnd.Header
        },
        body: {
          row: dnd.Row
        }
      };
      const { columns, rows } = this.state;
      const resolvedColumns = resolve.columnChildren({ columns });
      const resolvedRows = resolve.resolve({
        columns: resolvedColumns,
        method: resolve.nested
      })(rows);
  
      return (
        <div className={styles.bird_table}><Table.Provider
          renderers={renderers}
          columns={resolvedColumns}
        >
          <Table.Header
            headerRows={resolve.headerRows({ columns })}
            className={styles.bird_table_thead}
          />
  
          <Table.Body
            rows={resolvedRows}
            rowKey="id"
            onRow={this.onRow}
            className={styles.bird_table_body}
          />
          </Table.Provider>
                  
          <Pagination className={styles.gridPagination}
            showQuickJumper
            current={this.state.pageIndex}
            total={parseInt(this.state.gridDatas.totalCount, 10)}
            pageSizeOptions={ ["10", "15", "20", "30", "50", "100", "200"]}
            pageSize={this.state.pageSize}
            showSizeChanger={true}
            onChange={(page, pageSize) => this.pageClick(page, pageSize)}
            onShowSizeChange={(page, pageSize) => this.pageSizeChange(pageSize)}
            showTotal={total => `共 ${total} 条`} />
        </div>
      );
    }
    onRow(row) {
      return {
        rowId: row.id,
        onMove: this.onMoveRow
      };
    }
    onMoveRow({ sourceRowId, targetRowId }) {
      const rows = dnd.moveRows({
        sourceRowId,
        targetRowId
      })(this.state.rows);
  
      if (rows) {
        this.setState({ rows });
      }
    }
    onMoveColumn(labels) {
      const movedColumns = dnd.moveLabels(this.state.columns, labels);
  
      if (movedColumns) {
        // Retain widths to avoid flashing while drag and dropping.
        const source = movedColumns.source;
        const target = movedColumns.target;
        const sourceWidth = source.props.style && source.props.style.width;
        const targetWidth = target.props.style && target.props.style.width;
  
        source.props.style = {
          ...source.props.style,
          width: targetWidth
        };
        target.props.style = {
          ...target.props.style,
          width: sourceWidth
        };
  
        this.setState({
          columns: movedColumns.columns
        });
      }
    }
    onMoveChildColumn(labels) {
      const movedChildren = dnd.moveChildrenLabels(this.state.columns, labels);
  
      if (movedChildren) {
        const columns = cloneDeep(this.state.columns);
  
        columns[movedChildren.target].children = movedChildren.columns;
  
        // Here we assume children have the same width.
        this.setState({ columns });
      }
    }
  }
  
  // Set up drag and drop context
  const DragAndDrop = DragDropContext(HTML5Backend)(DragAndDropTable);
  
  //<DragAndDropTable />
  
// class BirdGridPermissionPage extends React.Component {
//     constructor(props) {
//       super(props)
//       this.state = {
//       }
//     }
  
  
//     render() {
//         return <DragAndDropTable />
//     }
// }
export default DragAndDrop 
