import React from 'react';
import { jrequest } from 'utils/request';
import { Row, Col, Spin, Icon, Input, Button, Table, Popconfirm, message } from 'antd';
import EditFeatureModal from 'components/Feature/EditFeatureModal.js';
import EditUpdateLog from 'components/Feature/EditUpdateLog.js';

class FeaturePage extends React.Component {
    constructor(props) {
        super(props);

        this.state = {
            searchName: "",
            searchLoading:false,

            tableData: [],
            total:0,
            current:1,
            pageSize:10,
        }
        this.staticData = {
            searchName : "",
        }

        //初次获取数据
        jrequest({
            url:"/feature/features",
            method:"get",
            data:{
                name:this.state.searchName,
                page:this.state.current,
                size:this.state.pageSize
            }
        }).then((result) => {
            this.setState({
                tableData: result.content,
                total: result.totalElements,
            });
        });
    }

    getColumnDef = () => {
        let columnDef = [{
            title:"功能编码",
            dataIndex:"id",
            key:"id",
            width:350
        },{
            title:"功能名称",
            dataIndex:"name",
            key:"name",
            width:200
        },{
            title:"功能介绍",
            dataIndex:"descrition",
            key:"descrition",
            width: 400,
            render: (text,row,index) => {
                return (
                <div style={{overflow:"hidden",textOverflow: "ellipsis",whiteSpace: "nowrap",maxWidth: "360px"}} title={text}>
                    {text}
                </div>
                );
            }
        },{
            title:"版本号",
            dataIndex:"version",
            key:"version",
            width:200
        },{
            title:"操作",
            key: 'operation',
            //fixed: 'right',
            width: 250,
            render: (text,row,index)=> {
                let modalData = {
                    name:row.name,
                    descrition: row.descrition,
                    version: row.version,
                    url: row.url,
                    objectData: row,
                    isUpdate:true,
                    type:"编辑",
                }
                return (
                    <div>
                        <EditFeatureModal key={row.id} modalData={modalData} afterOk={this.refresh} />
                        <EditUpdateLog key={row.id + "_update"} infoId={row.id} afterOk={this.refresh} />
                        <Popconfirm
                            title="确定删除？过程不可逆" 
                            onConfirm={() => this.deleteFeature(row.id)}
                            icon={<Icon type="question-circle-o" style={{ color: 'red' }} />}>
                            <Button style={{float:"left"}} type="danger">删除</Button>
                        </Popconfirm>
                    </div>
                );
            }
        }];
        return columnDef;
    }

    deleteFeature = (id) => {
        jrequest({
            url:"/feature/feature",
            method:"delete",
            data:id
        }).then((result) => {
            if(result.success) {
                this.refresh();
            } else {
                message.error(result.msg);
            }
        }).catch((error) => {

        })
    }

    //modal成功后的刷新
    refresh = () => {
        jrequest({
            url:"/feature/features",
            method:"get",
            data:{
                name:this.state.searchName,
                page:this.state.current,
                size:this.state.pageSize
            }
        }).then((result) => {
            this.setState({
                tableData: result.content,
                total: result.totalElements,
            });
        });
    }

    searchTable = () => {
        this.setState({searchLoading:true});
        jrequest({
            url:"/feature/features",
            method:"get",
            data:{
                name:this.staticData.searchName,
                page:this.state.current,
                size:this.state.pageSize
            }
        }).then((result) => {
            this.setState({
                searchLoading:false,
                tableData: result.content,
                total: result.totalElements,
                searchName: this.staticData.searchName
            });
        });
    }

    createButtonBar = () => {
        let modalData = {
            name: "",
            url: "",
            description: "",
            version: "",
            type:"发布功能",
            isUpdate:false
        };
        return (
            <Row gutter={8}>
                <Col span={2}>
                    <EditFeatureModal modalData={modalData} afterOk={this.refresh} />
                </Col>
                <Col span={3}>
                    <Input defaultValue={this.state.searchName} onChange={(e) => this.staticData.searchName = e.target.value}/>
                </Col>
                <Col span={3}>
                    <Button type="primary" icon={this.state.searchLoading ? "loading" : "search"} onClick={this.searchTable}>搜索</Button>
                </Col>
            </Row>
        );
    }

    changePage = (current) => {
        jrequest({
            url:"/feature/features",
            method:"get",
            data:{
                name:this.state.searchName,
                page:current,
                size:this.state.pageSize
            }
        }).then((result) => {
            this.setState({
                tableData: result.content,
                total: result.totalElements,
                current: current
            });
        }).catch((error) => {

        })
    }

    changePageSize = (pageSize,current) => {
        this.setState({pageSize: pageSize});
        this.refresh();
    }

    //分页配置信息
    getpaginationProps = () => {
        let paginationProps = {
            showSizeChanger: true,
            showQuickJumper: false,
            showTotal: () => `共${this.state.total}条`,
            pageSize: this.state.pageSize,
            current: this.state.current,
            total: this.state.total,
            onChange: (current) => this.changePage(current),
            onShowSizeChange: (current,pageSize) => this.changePageSize(pageSize,current),
        }
        return paginationProps;
    }

    render() {
        let buttonBar = this.createButtonBar();
        let paginationProps = this.getpaginationProps();
        return (
            <div style={{backgroundColor:"white"}}>
                {buttonBar}
                <Table
                    bordered
                    rowKey="id"
                    pagination={ paginationProps }
                    columns={ this.getColumnDef() }
                    dataSource={this.state.tableData}
                    scroll={{ x: 1500 }}
                />
            </div>
        )
    }
}
export default FeaturePage