import React from 'react';
import { jrequest } from 'utils/request';
import { Tabs, Input, InputNumber, Table, Button, Popconfirm, message } from 'antd';
import BraftEditor from 'braft-editor'
import 'braft-editor/dist/index.css'

const { TabPane } = Tabs;

export default class MemberLevelSettingPage extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            lvSettings : [],
            lvDesc : null,
            editorState: null,
        };

        //初始化列表
        jrequest({
            url:"/userLevelSetting/levels",
            method:"get",
        }).then(result => {
            let lvs = result.data;
            this.setState({lvSettings:lvs});
        });
        //初始化说明
        jrequest({
            url:"/userLevelSetting/desc",
            method:"get"
        }).then(result => {
            let obj = result.data;
            let content = obj.description;
            if(!content) {
                content = "";
            }
            let editorState = BraftEditor.createEditorState(content);
            this.setState({lvDesc: obj, editorState: editorState});
        });
    }

    refreshData = () => {
        jrequest({
            url:"/userLevelSetting/levels",
            method:"get",
        }).then(result => {
            let lvs = result.data;
            this.setState({lvSettings:lvs});
        });
        jrequest({
            url:"/userLevelSetting/desc",
            method:"get"
        }).then(result => {
            let obj = result.data;
            let content = obj.description;
            if(!content) {
                content = "";
            }
            let editorState = BraftEditor.createEditorState(content);
            this.setState({lvDesc: obj, editorState: editorState});
        });
    }

    getTableColumnDef = () => {
        return [{
            title:"序号",
            key:"index",
            render: (text,record, index) => {
                return <span>{index + 1}</span>
            }
        }, {
            title:"等级名称",
            dataIndex:"name",
            key:"name",
            render: (text, record, index) => {
                return <Input onChange={e => {
                    let lvSettings = this.state.lvSettings;
                    lvSettings[index].name = e.target.value;
                    this.setState({lvSettings:lvSettings});
                }} defaultValue={text} />
            }
        }, {
            title:"等级级别",
            dataIndex:"level",
            key:"level",
            render: (text, record, index) => {
                return <InputNumber onChange={value => {
                    let lvSettings = this.state.lvSettings;
                    lvSettings[index].level = value;
                    this.setState({lvSettings:lvSettings});
                }} defaultValue={text} />
            }
        }, {
            title:"成长范围",
            key:"increaseRange",
            render: (text, record, index) => {
                return (
                    <div>
                    <InputNumber onChange={value => {
                        let lvSettings = this.state.lvSettings;
                        lvSettings[index].growthFrom = value;
                        this.setState({lvSettings:lvSettings});
                    }} defaultValue={record.growthFrom} />
                    <span>至</span>
                    <InputNumber onChange={value => {
                        let lvSettings = this.state.lvSettings;
                        lvSettings[index].growthTo = value;
                        this.setState({lvSettings:lvSettings});
                    }} defaultValue={record.growthTo} />
                    </div>
                );
            }
        },{
            title:"操作",
            key:"action",
            render: (text, record, index) => {
                return <Button type="danger" onClick={() => this.onDeleteLevel(index)}>删除</Button>
            }
        }];
    }

    onDeleteLevel = (index) => {
        let levels = this.state.lvSettings;
        levels.splice(index, 1);
        this.setState({lvSettings: levels});
    }

    onAddLevelSetting = () => {
        let obj = {
            id: new Date().getTime() + "",
            name:"",
            level:0,
            growthFrom:0,
            growthTo:0
        };
        let lvSettings = this.state.lvSettings;
        lvSettings.push(obj);
        this.setState({lvSettings:lvSettings});
    }

    onSaveAll = () => {
        let lvDesc = this.state.lvDesc;
        lvDesc.description = this.state.editorState.toHTML();
        
        let data = {
            list: this.state.lvSettings,
            desc: this.state.lvDesc
        };
        console.log(data);
        jrequest({
            url:"/userLevelSetting/saveAll",
            method:"post",
            data:data
        }).then(result => {
            if(result.success) {
                message.success("保存成功");
                this.refreshData();
            } else {
                message.error("保存失败");
            }
        });
    }

    handleEditorChange = (editorState) => {
        this.setState({ editorState })
    }

    submitContent = () => {
        let content = this.state.editorState.toHTML()
        let lvDesc = this.state.lvDesc;
        lvDesc.description = content;
        jrequest({
            url:"/userLevelSetting/desc",
            method:"post",
            data:lvDesc
        }).then(result => {
            if(result.success) {
                message.success("保存成功");
                this.refreshData();
            } else {
                message.error("保存失败");
            }
        });
    }

    render() {
        return (
            <div style={{backgroundColor:"white"}}>
                <div>
                    <Button type="primary" onClick={this.onAddLevelSetting}>添加等级</Button>
                    <Button type="primary" onClick={this.onSaveAll}>保存</Button>
                </div>
                <Tabs type="card">
                    <TabPane tab="等级设置" key="1">
                        <Table rowKey="id" dataSource={this.state.lvSettings} columns={this.getTableColumnDef()} />
                    </TabPane>

                    <TabPane tab="等级描述" key="2">
                    <BraftEditor
                        value={this.state.editorState}
                        onChange={this.handleEditorChange}
                        // onSave={this.submitContent}
                    />
                    </TabPane>
                </Tabs>
            </div>
        );
    }

}