import React from 'react';
import { config, request, permission, util } from 'utils'
import { jrequest } from 'utils/request';
import { Tree, Row, Col, Spin, Icon } from 'antd';
import MetaCoreCss from './metacore.css';

import RootDetail from 'components/MetaCore/RootDetail.js';
import PackageDetail from 'components/MetaCore/PackageDetail.js';
import DataSourceDetail from 'components/MetaCore/DataSourceDetail.js';
import SchemaDetail from 'components/MetaCore/SchemaDetail.js';
import PDMDetail from 'components/MetaCore/PDMDetail.js';
import MetaTableDetail from 'components/MetaCore/MetaTableDetail.js';

const { TreeNode } = Tree;
const { JAPIV } = config

class MetaCoreManagePage extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      treeData: [],
      nodeDetail:{},
      selectedId:null,
      selectedNode:null, //选择了的节点
      loading: false
    };

    //这里的获取初始数据的异步方法需要放在constructor里执行
    // jrequest({
    //   url: "/meta/roots",
    //   method: "get",
    // }).then((result) => {
    //   this.state.treeData = result.data;
    // });
    this.onPageInit();
  }

  onPageInit = () => {
    jrequest({
      url: "/meta/roots",
      method: "get",
    }).then((result) => {
      // this.state.treeData = result.data;
      this.setState({
        treeData: result.data
      });
    });
  }

  //异步刷树数据
  onLoadData = treeNode => {
    return new Promise(resolve => {
      //下面这一块的意思是，当这个treeNode已经有子节点时，直接返回就行，不用获取数据
      if (treeNode.props.children) {
        resolve();
        return;
      }
      
      jrequest({
        url:"/meta/children",
        method:"get",
        data:{prtId: treeNode.props.dataRef.id}
      }).then((result) => {
        treeNode.props.dataRef.children = result.data;
        this.setState({
          treeData: [...this.state.treeData],
        });
        resolve();
      });
    });
  }

  //收缩时清掉节点，为了修改了父级信息后而产生子节点，如数据源的修改
  onExpand = (expandedKeys,info) => {
    let node = info.node;
    let expanded = info.expanded;
    if(expanded) {
      jrequest({
        url:"/meta/children",
        method:"get",
        data:{prtId: node.props.dataRef.id}
      }).then((result) => {
        node.props.dataRef.children = result.data;
        this.setState({
          treeData: [...this.state.treeData],
        });
      });
    }
  }

  //刷新树
  refreshTreeNode = (isAddAction, newDetailNode) => {
    if(isAddAction) {
      //如果是添加的操作，那么选择的节点就是父节点，直接刷新即可
      let node = this.state.selectedNode;
      jrequest({
        url:"/meta/children",
        method:"get",
        data:{prtId: node.props.dataRef.id}
      }).then((result) => {
        node.props.dataRef.children = result.data;
        if(newDetailNode) {
          this.setState({
            treeData: [...this.state.treeData],
            nodeDetail: newDetailNode
          })
        } else {
          this.setState({
            treeData: [...this.state.treeData],
          });
        }
      });
    } else {
      //如果是删除操作，就刷新父级节点
      let node = this.state.selectedNode;
      let id = node.props.dataRef.id
      let treeData = this.state.treeData;
      treeData.forEach((node) => {
        this.recDel(node, id)
      });
      this.setState({
        treeData: [...this.state.treeData],
        selectedId:null,   //选择的置空
        selectedNode:null,  //选择的置空
        nodeDetail: null   //已获取的节点也置空
      });
    }
  }

  //递归查找删除树节点
  recDel = (treeNode, key) => {
    if(treeNode.children) {
     let index;
     let found = false;
     treeNode.children.forEach((child,i) => {
      if(child.id === key){
       index = i;
       found = true;
      } else {
       this.recDel(child,key);
      }
     });
     if(found){
      treeNode.children.splice(index,1);
     }
    }
   }

  //递归的渲染树节点
  renderTreeNodes = data =>
    data.map(item => {
      let title = item.name;
      if (item.cnName != null && item.cnName !== "") {
        title = item.cnName;
      }
      if (item.children) {
        return (
          <TreeNode title={title} key={item.id} isLeaf={!item.parent} dataRef={item}>
            {this.renderTreeNodes(item.children)}
          </TreeNode>
        );
      }
      return <TreeNode title={title} key={item.id} isLeaf={!item.parent} dataRef={item} />;
    });

  //点击树节点
  onSelect = (selectedKey, info) => {
    if(selectedKey.length === 0) {
      return;
    }
    
    this.setState({
      loading:true
    });
    jrequest({
      url:"/meta/detail",
      method:"get",
      data:{id:selectedKey[0]}
    }).then((result) => {
      if(selectedKey === 'PDM' || selectedKey === 'ITF' || selectedKey === 'DB' || selectedKey === 'HOST'){
        result.data.dataNodeType = 'root';
      }
      this.setState({
        nodeDetail: result.data,
        selectedId: selectedKey[0],
        selectedNode: info.node,
        loading:false
      });
    });
  }

  render() {
    let detailjsx = (<div></div>);
    if(this.state.loading) {
      let antIcon = <Icon type="loading" style={{ fontSize: 24 }} spin />;
      detailjsx = (
        <div style={{textAlign:"center", borderRadius:"4px", marginBottom:"20px", padding:"30px 50px", margin:"20px 0"}}>
          <Spin indicator={antIcon} />
        </div>
      )
    } else if(this.state.nodeDetail && this.state.nodeDetail.realNode) {
      let nodeType = this.state.nodeDetail.realNode.nodeType;
      if(nodeType === 'PDM' || nodeType === 'ITF' || nodeType === 'DB' || nodeType === 'HOST') {
        //根
        detailjsx = (<RootDetail nodeDetail={this.state.nodeDetail} refreshTreeNode={this.refreshTreeNode} />)
      } else if(nodeType === 'core.BasePackage') {
        //目录
        detailjsx = (<PackageDetail nodeDetail={this.state.nodeDetail} refreshTreeNode={this.refreshTreeNode}/>)
      } else if(nodeType === 'phy.DataSource') {
        //数据源/落地库
        detailjsx = (<DataSourceDetail nodeDetail={this.state.nodeDetail} refreshTreeNode={this.refreshTreeNode}/>)
      } else if(nodeType === 'relation.Schema') {
        //反向工程获取的Schema
        detailjsx = (<SchemaDetail nodeDetail={this.state.nodeDetail}/>)
      } else if(nodeType === 'model.PhysicalModel') {
        //表，PDM
        let prtId = this.state.nodeDetail.realNode.prtId;
        detailjsx = (<PDMDetail nodeDetail={this.state.nodeDetail} prtId={prtId} refreshTreeNode={this.refreshTreeNode} />)
      } else if(nodeType === "relation.Table" || nodeType === "relation.View"){
        //反向工程获取的表、视图
        detailjsx = (<MetaTableDetail nodeDetail={this.state.nodeDetail}/>)
      }
    } 

    return(
      <div>
        <Row gutter={8}>
          <Col span={4} className={MetaCoreCss.metaCoreTree}>
            <Tree showLine loadData={this.onLoadData} onSelect={this.onSelect} onExpand={this.onExpand}>
              {this.renderTreeNodes(this.state.treeData)}
            </Tree>
          </Col>
          <Col span={20}>
            {detailjsx}
          </Col>
        </Row>
      </div>
    );
  }
}
export default MetaCoreManagePage