import React from 'react';
import Panel from 'components/Panel';
import { BirdGrid } from 'components/Grid';
import { Tabs, Row, Col, InputNumber, Button, Select, Descriptions, Modal, message, Divider, Card } from 'antd';
import { config } from 'utils';
import BraftEditor from 'braft-editor';
import { jrequest } from 'utils/request';
import 'antd/dist/antd.css';

const { TabPane } = Tabs;

const { JAPIV } = config;

class PointsRule extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            visiblePointsClearRule: false,
            visiblePointsToRMB: false,
            visibleEvidence: false,

            pointsClearRule: "",
            selectedPointsClearRule: "",
            points: "",
            RMB: "",
            selectedPoints: "",
            selectedRMB: "",
            pointsToBeConsumed: 0,
            pointsSendOut: 0,
            pointsExpired: 0,
            pointsConsumed: 0,
            evidenceData: {},
            visible: false,
            editorState: null,
            pointDesc: null,

            pointLogId: null,
            growthLog: null,
        }

        //获取积分清零规则
        jrequest({
            url: "/userPointClearRule/rule",
            method: "get"
        }).then(result => {
            console.log(result);
            this.setState({
                pointsClearRule: result.data.description,
                selectedPointsClearRule: result.data.description
            });
        });

        //获取积分兑换RMB规则
        jrequest({
            url: "/userPointRmbRule/rule",
            method: "get"
        }).then(result => {
            this.setState({
                points: parseInt(result.data.description),
                RMB: parseInt(result.data.description.split("=")[1]),
                selectedPoints: parseInt(result.data.description),
                selectedRMB: parseInt(result.data.description.split("=")[1])
            });
        });

        //获取待消费、已发放、已消费、已过期积分
        jrequest({
            url: "/userPointStastic/point",
            method: "get"
        }).then(result => {
            let obj = result.data;
            this.setState({
                pointsSendOut: obj.pointsSendOut,
                pointsConsumed: obj.pointsConsumed,
                pointsExpired: obj.pointsExpired,
                pointsToBeConsumed: obj.pointsToBeConsumed,
            });
        });

        //获取积分说明
        jrequest({
            url: "/userPointLog/desc",
            method: "get"
        }).then(result => {
            let obj = result.data;
            let content = obj.description;
            if (!content) {
                content = "";
            }
            let editorState = BraftEditor.createEditorState(content);
            this.setState({ pointDesc: obj, editorState: editorState });
        });

    }

    refreshData = () => {
        //获取积分说明
        jrequest({
            url: "/userPointLog/desc",
            method: "get"
        }).then(result => {
            let obj = result.data;
            let content = obj.description;
            if (!content) {
                content = "";
            }
            let editorState = BraftEditor.createEditorState(content);
            this.setState({ pointDesc: obj, editorState: editorState });
        });
    }


    //弹出积分清零股则模态框
    showModalPointsClearRule = () => {
        this.setState({
            visiblePointsClearRule: true,
        });
    }

    //关闭积分清零规则模态框
    handleCancelPointsClearRule = () => {
        this.setState({
            visiblePointsClearRule: false
        });
    }


    onChangeSelectedPointsClearRule = (event) => {
        // alert(event);
        this.setState({
            selectedPointsClearRule: event
        })
    }
    // 保存积分清零规则至数据库
    savePointsClearRule = () => {
        // alert("savePointsClearRule")
        this.setState({
            pointsClearRule: this.state.selectedPointsClearRule,
            visiblePointsClearRule: false,
        });
        jrequest({
            url: "/userPointClearRule/rule",
            method: "post",
            data: { "description": this.state.selectedPointsClearRule }
        }).then(result => {
            if (result.success) {
                message.success("保存成功");
                this.refreshData();
            } else {
                message.error("保存失败");
            }
        });

    }


    //弹出积分兑换人名币股则模态框
    showModalPointsToRMB = () => {
        this.setState({
            visiblePointsToRMB: true,
        });
    }

    //关闭积分兑换人名币规则模态框
    handleCancelPointsToRMB = () => {
        this.setState({

            visiblePointsToRMB: false
        });
    }

    // 修改积分规则兑换人民币 积分
    onchangeSelectedPoints = (event) => {
        // console.log(event);
        this.setState({
            selectedPoints: event
        });
    }

    onchangeSelectedRMB = (event) => {
        // console.log(event);
        this.setState({
            selectedRMB: event
        });
    }

    // 保存积分兑换人名币规则至数据库
    savePointsToRMB = (event) => {
        // console.log(this.state.selectedPoints + "积分" + this.state.selectedRMB + "人民币");
        this.setState({
            points: this.state.selectedPoints,
            RMB: this.state.selectedRMB,
            visiblePointsToRMB: false,
        });
        jrequest({
            url: "/userPointRmbRule/rule",
            method: "post",
            data: { "description": this.state.selectedPoints + "积分" + "=" + this.state.selectedRMB + "人民币" }
        }).then(result => {
            if (result.success) {
                message.success("保存成功");
                this.refreshData();
            } else {
                message.error("保存失败");
            }
        });
    }

    //弹出证据链查看模态框
    showModalEvidence = (data) => {
        jrequest({
            url: "/userGrowthLog/logById?id=" + data.id,
            method: "get"
        }).then((res) => {
            // console.log(res);
            if (res.success) {
                let log = res.data;
                this.setState({
                    growthLog: log,
                    evidenceData: data,
                    visibleEvidence: true,
                });
            } else {
                this.setState({
                    evidenceData: data,
                    visibleEvidence: true,
                });
            }
        }).catch(err => {
            console.log(err);
        });
    }

    //关闭证据链查看模态框
    handleCancelEvidence = () => {
        this.setState({
            visibleEvidence: false
        })
    }

    // 积分说明富文本编辑器状态改变
    handleEditorChange = (editorState) => {
        this.setState({ editorState })
    }


    // 保存积分说明至数据库
    savePointStatemen = () => {
        let pointDesc = this.state.pointDesc;
        pointDesc.description = this.state.editorState.toHTML();

        console.log(pointDesc);
        jrequest({
            url: "/userPointLog/desc",
            method: "post",
            data: pointDesc
        }).then(result => {
            if (result.success) {
                message.success("保存成功");
                this.refreshData();
            } else {
                message.error("保存失败");
            }
        });
    }


    render() {

        let self = this;

        let gridOption = {
            title: "表格示例",
            url: {
                read: JAPIV + "/userPointLog/logsCondition",
                // add: "/test/add",
                // edit: "/test/edit",
                // delete: "/test/delete"
            },
            // method: "get",
            // size: 8,
            // page: 1,
            checkable: false,
            //actions: [{ name: '外部按钮', color: 'error', onClick: function () { } }],
            columns: [
                { title: "id", data: "id", editor: {}, type: "text", hide: true },
                {
                    title: "时间", data: "crtDate", type: "date", editor: {}, query: true, render: (value, data) => {
                        return <div>{value}</div>;
                    }
                },
                { title: "积分值", data: "value", editor: {}, type: "number" },
                {
                    title: "用户", type: "text", render() {
                        return <div>***********</div>
                    }
                },
                { title: "用户行为端口", data: "terminalName", type: "text", editor: {}, query: true },
                { title: "用户行为", data: "terminalAction", type: "text", editor: {}, query: true },
                { title: "用户原积分", data: "origin", type: "number" },
                { title: "用户变化后积分", data: "balance", type: "number" },
                {
                    title: "证据链", type: "command", actions: [{
                        name: '查看',
                        color: '#67c23a',
                        onClick: (data) => {
                            // console.log(data);
                            this.showModalEvidence(data);
                        }
                    }]
                }
            ]
        };

        return (
            <div style={{ background: 'white' }}>
                <Tabs type="card">
                    <TabPane tab="积分规则" key="1">

                        <div className="site-card-wrapper" style={{ background: 'white', padding: '0px' }}>
                            <Row gutter={24} style={{ padding: '10px 20px' }}>
                                <Col span={4}>
                                    <b>积分清零规则</b>
                                </Col>
                                <Col span={4}>
                                    <b>{self.state.pointsClearRule}</b>
                                </Col>
                                <Col span={4}>
                                    <a onClick={this.showModalPointsClearRule}><b>编辑</b></a>
                                </Col>
                            </Row>
                            <Row gutter={24} style={{ padding: '10px 20px' }}>
                                <Col span={4}>
                                    <b> 积分兑换RMB</b>
                                </Col>
                                <Col span={4}>
                                    <b>{self.state.points}积分 = {this.state.RMB} 元</b>
                                </Col>
                                <Col span={4}>
                                    <a onClick={this.showModalPointsToRMB}><b>编辑</b></a>
                                </Col>
                            </Row>
                        </div>

                        <div className="site-card-wrapper" style={{ background: 'white', padding: '10px 0px' }}>
                            <Row gutter={24}>
                                <Col span={4}>
                                    <Card title="待消费" bordered={false}>
                                        <b>{self.state.pointsToBeConsumed.toLocaleString()}</b>
                                    </Card>
                                </Col>
                                <Col span={4}>
                                    <Card title="已发放" bordered={false}>
                                        <b>{self.state.pointsSendOut.toLocaleString()}</b>
                                    </Card>
                                </Col>
                                <Col span={4}>
                                    <Card title="已消费" bordered={false}>
                                        <b>{self.state.pointsConsumed.toLocaleString()}</b>
                                    </Card>
                                </Col>
                                <Col span={4}>
                                    <Card title="已过期" bordered={false}>
                                        <b>{self.state.pointsExpired.toLocaleString()}</b>
                                    </Card>
                                </Col>
                            </Row>
                        </div>

                        <br />

                        <div style={{ padding: '0px 25px' }}>
                            <h3>历史记录</h3>
                        </div>

                        <div>
                            <BirdGrid gridOption={gridOption} ref="grid" />
                        </div>
                    </TabPane>

                    <TabPane tab="积分说明" key="2">
                        <BraftEditor
                            value={this.state.editorState}
                            onChange={this.handleEditorChange}
                        // onSave={this.submitContent}
                        />
                        <Button type="primary" style={{ margin: '0 45%' }} onClick={this.savePointStatemen}>保存</Button>
                    </TabPane>

                </Tabs>




                <Modal
                    title="积分清零规则"
                    visible={this.state.visiblePointsClearRule}
                    onCancel={this.handleCancelPointsClearRule}
                    footer={[
                        <Button key="submit" type="primary" onClick={this.savePointsClearRule}>
                            保存
                                </Button>,
                    ]}>
                    <div>
                        <Select defaultValue="选择清零周期" style={{ width: "80%", margin: "0px 10%" }} value={this.state.selectedPointsClearRule} onChange={this.onChangeSelectedPointsClearRule}>
                            <Option value="不清零">不清零</Option>
                            <Option value="每季第一天">每季第一天</Option>
                            <Option value="半年第一天">半年第一天</Option>
                            <Option value="每年第一天">每年第一天</Option>
                        </Select>
                    </div>
                </Modal>

                <Modal
                    title="积分兑换RMB"
                    visible={this.state.visiblePointsToRMB}
                    onCancel={this.handleCancelPointsToRMB}
                    footer={[
                        <Button key="submit" type="primary" onClick={this.savePointsToRMB}>
                            保存
                                </Button>,
                    ]}>
                    <div>
                        <div style={{ marginBottom: 16 }}>
                            <InputNumber min={0} defaultValue="0" value={this.state.selectedPoints} onChange={this.onchangeSelectedPoints} />
                            <Button>积分</Button><span> 等于</span>
                            <InputNumber min={0} step={0.01} defaultValue="0" value={this.state.selectedRMB} onChange={this.onchangeSelectedRMB} />
                            <Button>元</Button>
                        </div>

                    </div>
                </Modal>


                <Modal
                    title="证据链"
                    visible={this.state.visibleEvidence}
                    onCancel={this.handleCancelEvidence}
                    footer={[
                        <Button key="submit" type="primary" onClick={this.handleCancelEvidence} >
                            返回
                                </Button>,
                    ]}
                    width="60%"
                >
                    <div>
                        <Descriptions title="基础信息" bordered column={2} style={{padding:'10px 0px'}}>
                            <Descriptions.Item label="数据来源平台">{this.state.evidenceData.terminalName}</Descriptions.Item>
                            <Descriptions.Item label="订单编号" span={2}>{this.state.evidenceData.dataCode}</Descriptions.Item>
                            <Descriptions.Item label="用户行为" span={2}>{this.state.evidenceData.terminalAction}</Descriptions.Item>
                        </Descriptions>
                        <Descriptions title="变量字段" bordered column={2} style={{padding:'10px 0px'}}>
                            <Descriptions.Item label="订单金额">{this.state.evidenceData.orderAmount}</Descriptions.Item>
                            <Descriptions.Item label="优惠金额">{this.state.evidenceData.discountedPrice}</Descriptions.Item>
                            <Descriptions.Item label="物流运费">{this.state.evidenceData.transportCost}</Descriptions.Item>
                            <Descriptions.Item label="实付金额">{this.state.evidenceData.payment}</Descriptions.Item>
                        </Descriptions>
                        {/* <Descriptions title="成长值计算" column={1} style={{ margin: '25px 20px 0 0' }}>
                                    <Descriptions.Item label="计算公式">{this.state.evidenceData.remark}</Descriptions.Item>
                                    <Descriptions.Item label="成长值变动">500.00 *  5% = 25</Descriptions.Item>
                                </Descriptions> */}
                        <Descriptions title="积分值计算" bordered column={2} style={{padding:'10px 0px'}}>
                            <Descriptions.Item label="计算公式">{this.state.evidenceData.remark}</Descriptions.Item>
                            <Descriptions.Item label="积分值变动">{this.state.evidenceData.payment}*{this.state.evidenceData.remark ? this.state.evidenceData.remark.split("*")[1] : ""}={this.state.evidenceData.value}</Descriptions.Item>
                        </Descriptions>
                        {this.state.growthLog ?
                            <Descriptions title="成长值计算"  bordered column={2} style={{padding:'10px 0px'}}>
                                <Descriptions.Item label="计算公式">{this.state.growthLog.remark}</Descriptions.Item>
                                <Descriptions.Item label="成长值变动">{this.state.evidenceData.payment} * {this.state.growthLog.remark ? this.state.growthLog.remark.split("*")[1] : ""} = {this.state.growthLog.value}</Descriptions.Item>
                            </Descriptions>
                            : <div></div>}
                    </div>
                </Modal>



            </div>
        );
    }
}



export default PointsRule;
