import React from 'react';
import { jrequest } from 'utils/request';
import { Row, Col, Spin, Icon, Input, Button, Table, Popconfirm, message } from 'antd';
import EditSqlManageModal from '../../components/SqlMange/EditSqlManageModal';

export default class SqlManage extends React.Component {
    constructor(props) {
        super(props);

        this.state = {
            searchName: "",
            searchLoading: false,

            tableData: [],
            total: 0,
            current: 1,
            pageSize: 10,
        }
        this.staticData = {
            searchName: "",
        }

        //初次获取数据
        jrequest({
            url: "/sql/sqls",
            method: "get",
            data: {
                name: this.state.searchName,
                page: this.state.current,
                size: this.state.pageSize
            }
        }).then((result) => {
            this.setState({
                tableData: result.content,
                total: result.totalElements,
            });
        });
    }

    getColumnDef = () => {
        let columnDef = [{
            title: "配置编号",
            dataIndex: "id",
            key: "id",
            width: 350
        }, {
            title: "配置描述",
            dataIndex: "description",
            key: "description",
            width: 300,
        }, {
            title: "sql配置",
            dataIndex: "sql",
            key: "sql",
            render: (text, row, index) => {
                return (
                    <div style={{ overflow: "hidden", textOverflow: "ellipsis", whiteSpace: "nowrap", maxWidth: "360px" }} title={text}>
                        {text}
                    </div>
                );
            }
        }, {
            title: "操作",
            key: 'operation',
            //fixed: 'right',
            width: 250,
            render: (text, row, index) => {
                let modalData = {
                    description: row.description,
                    sql: row.sql,
                    objectData: row,
                    isUpdate: true,
                    type: "编辑",
                };
                return (
                    <div>
                        <EditSqlManageModal key={row.id} modalData={modalData} afterOk={this.refresh} />
                        <Popconfirm
                            title="确定删除？过程不可逆"
                            onConfirm={() => this.deleteSql(row.id)}
                            icon={<Icon type="question-circle-o" style={{ color: 'red' }} />}>
                            <Button style={{ float: "left" }} type="danger">删除</Button>
                        </Popconfirm>
                    </div>
                );
            }
        }];
        return columnDef;
    }

    deleteSql = (id) => {
        jrequest({
            url:"/sql/sql",
            method:"delete",
            data:id
        }).then((result) => {
            if(result.success) {
                this.refresh();
            } else {
                message.error(result.msg);
            }
        }).catch((error) => {

        })
    }

    //modal成功后的刷新
    refresh = () => {
        jrequest({
            url:"/sql/sqls",
            method:"get",
            data:{
                name:this.state.searchName,
                page:this.state.current,
                size:this.state.pageSize
            }
        }).then((result) => {
            this.setState({
                tableData: result.content,
                total: result.totalElements,
            });
        });
    }

    searchTable = () => {
        this.setState({searchLoading:true});
        jrequest({
            url:"/sql/sqls",
            method:"get",
            data:{
                description:this.staticData.searchName,
                page:this.state.current,
                size:this.state.pageSize
            }
        }).then((result) => {
            this.setState({
                searchLoading:false,
                tableData: result.content,
                total: result.totalElements,
                searchName: this.staticData.searchName
            });
        });
    }

    createButtonBar = () => {
        let modalData = {
            description: "",
            sql: "",
            type:"添加sql配置",
            isUpdate:false
        };
        return (
            <Row gutter={8}>
                <Col span={2}>
                    <EditSqlManageModal modalData={modalData} afterOk={this.refresh} />
                </Col>
                <Col span={3}>
                    <Input defaultValue={this.state.searchName} onChange={(e) => this.staticData.searchName = e.target.value}/>
                </Col>
                <Col span={3}>
                    <Button type="primary" icon={this.state.searchLoading ? "loading" : "search"} onClick={this.searchTable}>搜索</Button>
                </Col>
            </Row>
        );
    }

    changePage = (current) => {
        jrequest({
            url:"/sql/sqls",
            method:"get",
            data:{
                description:this.state.searchName,
                page:current,
                size:this.state.pageSize
            }
        }).then((result) => {
            this.setState({
                tableData: result.content,
                total: result.totalElements,
                current: current
            });
        }).catch((error) => {

        })
    }

    changePageSize = (pageSize,current) => {
        this.setState({pageSize: pageSize});
        this.refresh();
    }

    //分页配置信息
    getpaginationProps = () => {
        let paginationProps = {
            showSizeChanger: true,
            showQuickJumper: false,
            showTotal: () => `共${this.state.total}条`,
            pageSize: this.state.pageSize,
            current: this.state.current,
            total: this.state.total,
            onChange: (current) => this.changePage(current),
            onShowSizeChange: (current,pageSize) => this.changePageSize(pageSize,current),
        }
        return paginationProps;
    }

    render() {
        let buttonBar = this.createButtonBar();
        let paginationProps = this.getpaginationProps();
        return (
            <div style={{backgroundColor:"white"}}>
                {buttonBar}
                <Table
                    bordered
                    rowKey="id"
                    pagination={ paginationProps }
                    columns={ this.getColumnDef() }
                    dataSource={this.state.tableData}
                    scroll={{ y: "600px", x:"1000px" }}
                />
            </div>
        )
    }
}