/* global window */
import axios from 'axios'
import qs from 'qs'
import jsonp from 'jsonp'
import lodash from 'lodash'
import pathToRegexp from 'path-to-regexp'
import { message } from 'antd'
import config from './config'
import util from './util';

const fetch = (options) => {
  let {
    method = 'post',
    data,
    fetchType,
    url,
    dataHeader
  } = options

  const cloneData = lodash.cloneDeep(data)

  try {
    let domin = ''
    if (url.match(/[a-zA-z]+:\/\/[^/]*/)) {
      domin = url.match(/[a-zA-z]+:\/\/[^/]*/)[0]
      url = url.slice(domin.length)
    }
    const match = pathToRegexp.parse(url)
    url = pathToRegexp.compile(url)(data)
    for (let item of match) {
      if (item instanceof Object && item.name in cloneData) {
        delete cloneData[item.name]
      }
    }
    url = domin + url
  } catch (e) {
    message.error(e.message)
  }

  if (fetchType === 'JSONP') {
    return new Promise((resolve, reject) => {
      jsonp(url, {
        param: `${qs.stringify(data)}&callback`,
        name: `jsonp_${new Date().getTime()}`,
        timeout: 4000
      }, (error, result) => {
        if (error) {
          reject(error)
        }
        resolve({ statusText: 'OK', status: 200, data: result })
      })
    })
  } else if (fetchType === 'YQL') {
    url = `http://query.yahooapis.com/v1/public/yql?q=select * from json where url='${options.url}?${encodeURIComponent(qs.stringify(options.data))}'&format=json`
    data = null
  }

  //使用axios拦截器添加sso.token请求头
  axios.interceptors.request.use(
    config => {
      var token = util.auth.getToken();
      if (token) {  // 判断是否存在token，如果存在的话，则每个http header都加上token
        config.headers['Sso-Token'] = token;
      }
      return config;
    }, err => {
      return Promise.reject(err);
    });

  switch (method.toLowerCase()) {
    case 'get':
      return axios.get(url, {
        params: cloneData
      })
    case 'delete':
      return axios.delete(url, {
        data: cloneData
      })
    case 'post':
      return axios.post(url, cloneData,dataHeader)
    case 'put':
      return axios.put(url, cloneData,dataHeader)
    case 'patch':
      return axios.patch(url, cloneData,dataHeader)
    default:
      return axios(options)
  }
}


const Asyncfetch = async (options) => {
  let {
    method = 'post',
    data,
    fetchType,
    url,
    dataHeader
  } = options

  const cloneData = lodash.cloneDeep(data)

  try {
    let domin = ''
    if (url.match(/[a-zA-z]+:\/\/[^/]*/)) {
      domin = url.match(/[a-zA-z]+:\/\/[^/]*/)[0]
      url = url.slice(domin.length)
    }
    const match = pathToRegexp.parse(url)
    url = pathToRegexp.compile(url)(data)
    for (let item of match) {
      if (item instanceof Object && item.name in cloneData) {
        delete cloneData[item.name]
      }
    }
    url = domin + url
  } catch (e) {
    message.error(e.message)
  }

  if (fetchType === 'JSONP') {
    return await jsonp(url, {
        param: `${qs.stringify(data)}&callback`,
        name: `jsonp_${new Date().getTime()}`,
        timeout: 4000
      }, (error, result) => {
        if (error) {
          return error
        }
        return { statusText: 'OK', status: 200, data: result }
      })
  } else if (fetchType === 'YQL') {
    url = `http://query.yahooapis.com/v1/public/yql?q=select * from json where url='${options.url}?${encodeURIComponent(qs.stringify(options.data))}'&format=json`
    data = null
  }

  //使用axios拦截器添加sso.token请求头
  axios.interceptors.request.use(
    config => {
      var token = util.auth.getToken();
      if (token) {  // 判断是否存在token，如果存在的话，则每个http header都加上token
        config.headers['Sso-Token'] = token;
      }
      return config;
    }, err => {
      return Promise.reject(err);
    });

  switch (method.toLowerCase()) {
    case 'get':
      return await axios.get(url, {
        params: cloneData
      })
    case 'delete':
      return await axios.delete(url, {
        data: cloneData
      })
    case 'post':
      return await axios.post(url, cloneData,dataHeader)
    case 'put':
      return await axios.put(url, cloneData,dataHeader)
    case 'patch':
      return await axios.patch(url, cloneData,dataHeader)
    default:
      return await axios(options)
  }
}

export default function request(options) {
  //判断url前缀是否是java的，是就直接返回java的那个request
  if (options.url && options.url.indexOf("/japi") === 0) {
    options.url = options.url.replace("/japi","");
    return jrequest(options);
  }

  //TODO:后台访问地址前缀移入配置文件
  if (options.url && options.url.indexOf('//') === -1 && options.url.indexOf('api/v') < 0 && options.url.indexOf('web/') < 0) {
    options.url = config.apiPrefix + options.url;
  }
  if (options.url && options.url.indexOf('//') === -1 && options.url.indexOf('api/v') < 0 && options.url.indexOf('web/') > 0) {
    options.url = config.webApiPrefix + options.url;
  }
  if (options.url && options.url.indexOf('//') > -1) {
    const origin = `${options.url.split('//')[0]}//${options.url.split('//')[1].split('/')[0]}`

    if (window.location.origin !== origin) {
      if (config.CORS && config.CORS.indexOf(origin) > -1) {
        options.fetchType = 'CORS'
      } else if (config.YQL && config.YQL.indexOf(origin) > -1) {
        options.fetchType = 'YQL'
      } else {
        options.fetchType = 'JSONP'
      }
    }
  }

  return fetch(options).then((response) => {
    let data = options.fetchType === 'YQL' ? response.data.query.results.json : response.data
    if (data['httpCode'] === "200") {
      return Promise.resolve(data['result'])
    } else if (data['code'] + '' === '500') {
      return Promise.reject({ response });
    } else {
      return Promise.resolve(data);
    }
  }).catch((error) => {
    const { response } = error
    let msg
    let statusCode
    if (response && response instanceof Object) {
      const { data, statusText } = response
      statusCode = response.status
      msg = data.message || statusText
    } else {
      statusCode = 600
      msg = error.message || 'Network Error'
    }
    message.error(msg);
    return Promise.reject({ success: false, statusCode, message: msg })
  })
};
  

export let arequest = async (options) => {
  //TODO:后台访问地址前缀移入配置文件
  if (options.url && options.url.indexOf('//') === -1 && options.url.indexOf('api/v') < 0 && options.url.indexOf('web/') < 0) {
    options.url = config.apiPrefix + options.url;
  }
  if (options.url && options.url.indexOf('//') === -1 && options.url.indexOf('api/v') < 0 && options.url.indexOf('web/') > 0) {
    options.url = config.webApiPrefix + options.url;
  }
  if (options.url && options.url.indexOf('//') > -1) {
    const origin = `${options.url.split('//')[0]}//${options.url.split('//')[1].split('/')[0]}`

    if (window.location.origin !== origin) {
      if (config.CORS && config.CORS.indexOf(origin) > -1) {
        options.fetchType = 'CORS'
      } else if (config.YQL && config.YQL.indexOf(origin) > -1) {
        options.fetchType = 'YQL'
      } else {
        options.fetchType = 'JSONP'
      }
    }
  }
  try {
   let response = await Asyncfetch(options);
   
   let data = options.fetchType === 'YQL' ? response.data.query.results.json : response.data
   if (data['httpCode'] === "200") {
     return data['result']
   } else if (data['code'] + '' === '500') {
     return { response };
   } else {
     return data;
   }
  }catch(error) {
      const { response } = error
      let msg
      let statusCode
      if (response && response instanceof Object) {
        const { data, statusText } = response
        statusCode = response.status
        msg = data.message || statusText
      } else {
        statusCode = 600
        msg = error.message || 'Network Error'
      }
      message.error(msg);
      return { success: false, statusCode, message: msg }
    }
};

//调用Java服务器的接口
export let jrequest = (options) => {
  if(options.url && options.url.indexOf(config.JAPIV) === -1) {
    options.url = config.JAPIV + options.url;
  }

  return fetch(options).then((response) => {
    let success = response.data.success;
    let list = response.data.content;
    return Promise.resolve(response.data);
  }).catch((error) => {
    const { response } = error
    let msg
    let statusCode
    if (response && response instanceof Object) {
      statusCode = response.data.code
      msg = response.data.msg
    } else {
      statusCode = 600
      msg = error.message || 'Network Error'
    }
    message.error(msg);
    return Promise.reject({ success: false, statusCode, message: msg })
  });
};