import store from 'store';

const tokenKey = "sso.token";
const userKey = 'bird.curUser';

let util = {
  object: {
    deepClone: function (p, c) {
      var nc = c ? c : p.constructor === Array ? [] : {};
      for (var i in p) {
        if (typeof p[i] === 'object') {
          if (p[i] == null) {
            nc[i] = null;
            continue;
          }
          nc[i] = (p[i].constructor === Array) ? [] : {};
          util.object.deepClone(p[i], nc[i]);
        } else {
          nc[i] = p[i];
        }
      }
      return nc;
    },
    equal: function (a, b) {
      return JSON.stringify(a) === JSON.stringify(b);
    }
  },
  number: {
    getDiscount: function (price, originPrice) {
      if (price <= 0 || originPrice <= 0 || price >= originPrice) return "";
      var d = (price / originPrice * 10).toFixed(2);
      while (d[d.length - 1] === "0" || d[d.length - 1] === ".") {
        d = d.substring(0, d.length - 1);
      }
      return d;
    },
    add: function (a, b) {
      var c, d, e;
      try {
        c = a.toString().split(".")[1].length;
      } catch (f) {
        c = 0;
      }
      try {
        d = b.toString().split(".")[1].length;
      } catch (f) {
        d = 0;
      }
      return e = Math.pow(10, Math.max(c, d)), (util.number.mul(a, e) + util.number.mul(b, e)) / e;
    },
    sub: function (a, b) {
      var c, d, e;
      try {
        c = a.toString().split(".")[1].length;
      } catch (f) {
        c = 0;
      }
      try {
        d = b.toString().split(".")[1].length;
      } catch (f) {
        d = 0;
      }
      return e = Math.pow(10, Math.max(c, d)), (util.number.mul(a, e) - util.number.mul(b, e)) / e;
    },
    mul: function (a, b) {
      var c = 0, d = a.toString(), e = b.toString();
      try {
        c += d.split(".")[1].length;
      } catch (f) {
      }
      try {
        c += e.split(".")[1].length;
      } catch (f) {
      }
      return Number(d.replace(".", "")) * Number(e.replace(".", "")) / Math.pow(10, c);
    },
    div: function (a, b) {
      var c, d, e = 0, f = 0;
      try {
        e = a.toString().split(".")[1].length;
      } catch (g) {
      }
      try {
        f = b.toString().split(".")[1].length;
      } catch (g) {
      }
      return c = Number(a.toString().replace(".", "")), d = Number(b.toString().replace(".", "")), util.number.mul(c / d, Math.pow(10, f - e));
    }
  },
  string: {
    // 最大显示长度
    truncate: function (str, max, postfix) {
      if (typeof (str) !== "string") {
        return str;
      }
      if (typeof (postfix) === "undefined") {
        postfix = "...";
      }
      if (str.length <= max) return str;
      return str.substring(0, max) + postfix;
    },
    // 连字符转驼峰
    hyphenToHump: function (str) {
      return str.replace(/-(\w)/g, (...args) => {
        return args[1].toUpperCase()
      })
    },
    // 驼峰转连字符
    humpToHyphen: function (str) {
      return str.replace(/([A-Z])/g, '-$1').toLowerCase()
    },
    isEmpty: function (str) {
      if (typeof (str) === "undefined") return true;
      return ((str + '').replace(/(^\s*)|(\s*$)/g, "").length === 0);
    },
    generateRandom(len) {
      len = len || 32;
      var $chars = '0123456789';    /****ABCDEFGHJKMNPQRSTWXYZabcdefhijkmnprstwxyz2345678 默认去掉了容易混淆的字符oOLl,9gq,Vv,Uu,I1****/
      var maxPos = $chars.length;
      var pwd = '';
      for (let i = 0; i < len; i++) {
        pwd += $chars.charAt(Math.floor(Math.random() * maxPos));
      }
      return pwd;
    }
  },
  date: {
    format: function (date, format) {
      if (typeof (date) === 'string') {
        date = new Date(Date.parse(date.replace(/-/g, "/")))
      }
      const o = {
        'M+': date.getMonth() + 1,
        'd+': date.getDate(),
        'h+': date.getHours(),
        'H+': date.getHours(),
        'm+': date.getMinutes(),
        's+': date.getSeconds(),
        'q+': Math.floor((date.getMonth() + 3) / 3),
        S: date.getMilliseconds()
      }
      if (/(y+)/.test(format)) {
        format = format.replace(RegExp.$1, `${date.getFullYear()}`.substr(4 - RegExp.$1.length))
      }
      for (let k in o) {
        if (new RegExp(`(${k})`).test(format)) {
          format = format.replace(RegExp.$1, RegExp.$1.length === 1 ? o[k] : (`00${o[k]}`).substr(`${o[k]}`.length))
        }
      }
      return format
    }
  },
  store: {
    get: function (key) {
      var cacheObj = store.get(key);
      if (cacheObj) {
        var now = new Date();

        if (!cacheObj.expire) return null;
        var expireTime = new Date(cacheObj.expire);
        if (expireTime < now) {//超出有效期,移除
          util.store.remove(key);
          return null;
        } else {
          //有效期不足半小时,则刷新
          if (expireTime - now < (1000 * 60 * 30)) {
            cacheObj.expire = new Date((now / 1000 + 60 * 60 * 12 ) * 1000);//重置过期时间12小时
            store.set(key, cacheObj)
          }
          return cacheObj.value;
        }
      }
      return null;
    },
    set: function (key, value, span) {
      var sp = span || (60 * 12);//默认缓存12小时

      var now = new Date();
      var expire = new Date((now / 1000 + sp * 60) * 1000);
      var cacheObj = {
        expire: expire,
        value: value
      };
      store.set(key, cacheObj);
    },
    remove: function (key) {
      store.remove(key);
    },
    clearAll: function () {
      store.clearAll();
    }
  },
  auth: {
    getToken: function () {
      return util.store.get(tokenKey);
    },
    setToken: function (token) {
      util.store.set(tokenKey, token)
    },
    removeToken: function () {
      util.store.remove(tokenKey);
      util.store.remove(userKey);
    },
    getUser: function () {
      return util.store.get(userKey);
    },
    setUser: function (user) {
      util.store.set(userKey, user);
    },
  },
  //格式化数据，建立多叉树
  json:{
    formatTreeData:function (treejson)
    {
      if(!treejson)return;
      var sortParentChild = function(a,b)
      {
        return a.parentId - b.parentId;
      }
      treejson = treejson.sort(sortParentChild)

      var formatTreeJson = {};
      for(var i = 0; i < treejson.length; i++)
      {
        if(formatTreeJson[treejson[i].parentId])
        {
          var x = formatTreeJson[treejson[i].parentId].i * 1 + 1;
          formatTreeJson[treejson[i].parentId].i = x;

          //node数以三位递增，不足补0
          treejson[i].node = x*1 < 10 ? ("00" + x):( x*1 > 100 ? x : ("0" + x));
          formatTreeJson[treejson[i].parentId].child.push(treejson[i]);
        }
        else
        {
          formatTreeJson[treejson[i].parentId] = {};
          formatTreeJson[treejson[i].parentId].i = 1;
          formatTreeJson[treejson[i].parentId].child = [];
          treejson[i].node = "001";
          formatTreeJson[treejson[i].parentId].child.push(treejson[i]);
        }
      }
      return formatTreeJson;
    },
    generateNode:function (treejson)
		{
			var formatJson = this.formatTreeData(treejson);
      var nodeArr = [];
      var targetjson = "";
      for (let j in formatJson) {
        //jsonarr.push(formatJson[i]); //属性
        /*for(var i = 0, len = formatJson[0].child.length; i < len; i++)
        {
          if(formatJson[0].child[i].id == j){
            nodeArr.push(formatJson[0].child[i]);
          }
        }
        if(j!=0)*/
			  this.node(formatJson[j], "", nodeArr,targetjson);
      }
			
			// 其中nodeArr就是生产后的数据
      //console.log(JSON.stringify(nodeArr));
      return nodeArr;
		},
		node:function (json, pnode, arr,targetjson)
		{
			if(!json) return;
 
			for(var i = 0, len = json.child.length; i < len; i++)
			{
				json.child[i].node = pnode + json.child[i].node;
				arr.push(json.child[i]);
				//2015-4-9
				this.node(targetjson[json.child[i].id], json.child[i].node, arr);
			}
		}
  },
  Provinces:[
    {"label": "北京市","value":"110000","disabled": false},
{"label": "天津市","value":"120000","disabled": false},
{"label": "河北省","value":"130000","disabled": false},
{"label": "山西省","value":"140000","disabled": false},
{"label": "内蒙古自治区","value":"150000","disabled": false},
{"label": "辽宁省","value":"210000","disabled": false},
{"label": "吉林省","value":"220000","disabled": false},
{"label": "黑龙江省","value":"230000","disabled": false},
{"label": "上海市","value":"310000","disabled": false},
{"label": "江苏省","value":"320000","disabled": false},
{"label": "浙江省","value":"330000","disabled": false},
{"label": "安徽省","value":"340000","disabled": false},
{"label": "福建省","value":"350000","disabled": false},
{"label": "江西省","value":"360000","disabled": false},
{"label": "山东省","value":"370000","disabled": false},
{"label": "河南省","value":"410000","disabled": false},
{"label": "湖北省","value":"420000","disabled": false},
{"label": "湖南省","value":"430000","disabled": false},
{"label": "广东省","value":"440000","disabled": false},
{"label": "广西壮族自治区","value":"450000","disabled": false},
{"label": "海南省","value":"460000","disabled": false},
{"label": "重庆市","value":"500000","disabled": false},
{"label": "四川省","value":"510000","disabled": false},
{"label": "贵州省","value":"520000","disabled": false},
{"label": "云南省","value":"530000","disabled": false},
{"label": "西藏自治区","value":"540000","disabled": false},
{"label": "陕西省","value":"610000","disabled": false},
{"label": "甘肃省","value":"620000","disabled": false},
{"label": "青海省","value":"630000","disabled": false},
{"label": "宁夏回族自治区","value":"640000","disabled": false},
{"label": "新疆维吾尔自治区","value":"650000","disabled": false},
{"label": "台湾省","value":"710000","disabled": false},
{"label": "香港特别行政区","value":"810000","disabled": false},
{"label": "澳门特别行政区","value":"820000","disabled": false},
{"label": "国外","value":"830000","disabled": false},
  ],
  Cities:[
    {"label": "东城区","value":"110100","disabled": false},
{"label": "西城区","value":"110200","disabled": false},
{"label": "崇文区","value":"110300","disabled": false},
{"label": "宣武区","value":"110400","disabled": false},
{"label": "朝阳区","value":"110500","disabled": false},
{"label": "丰台区","value":"110600","disabled": false},
{"label": "石景山区","value":"110700","disabled": false},
{"label": "海淀区","value":"110800","disabled": false},
{"label": "门头沟区","value":"110900","disabled": false},
{"label": "房山区","value":"111000","disabled": false},
{"label": "通州区","value":"111100","disabled": false},
{"label": "顺义区","value":"111200","disabled": false},
{"label": "昌平区","value":"111300","disabled": false},
{"label": "大兴区","value":"111400","disabled": false},
{"label": "怀柔区","value":"111500","disabled": false},
{"label": "平谷区","value":"111600","disabled": false},
{"label": "密云县","value":"111700","disabled": false},
{"label": "延庆县","value":"111800","disabled": false},
{"label": "和平区","value":"120100","disabled": false},
{"label": "河东区","value":"120200","disabled": false},
{"label": "河西区","value":"120300","disabled": false},
{"label": "南开区","value":"120400","disabled": false},
{"label": "河北区","value":"120500","disabled": false},
{"label": "红桥区","value":"120600","disabled": false},
{"label": "塘沽区","value":"120700","disabled": false},
{"label": "汉沽区","value":"120800","disabled": false},
{"label": "大港区","value":"120900","disabled": false},
{"label": "东丽区","value":"121000","disabled": false},
{"label": "西青区","value":"121100","disabled": false},
{"label": "津南区","value":"121200","disabled": false},
{"label": "北辰区","value":"121300","disabled": false},
{"label": "武清区","value":"121400","disabled": false},
{"label": "宝坻区","value":"121500","disabled": false},
{"label": "宁河县","value":"121600","disabled": false},
{"label": "静海县","value":"121700","disabled": false},
{"label": "蓟县","value":"121800","disabled": false},
{"label": "滨海新区","value":"121900","disabled": false},
{"label": "石家庄市","value":"130100","disabled": false},
{"label": "唐山市","value":"130200","disabled": false},
{"label": "秦皇岛市","value":"130300","disabled": false},
{"label": "邯郸市","value":"130400","disabled": false},
{"label": "邢台市","value":"130500","disabled": false},
{"label": "保定市","value":"130600","disabled": false},
{"label": "张家口市","value":"130700","disabled": false},
{"label": "承德市","value":"130800","disabled": false},
{"label": "沧州市","value":"130900","disabled": false},
{"label": "廊坊市","value":"131000","disabled": false},
{"label": "衡水市","value":"131100","disabled": false},
{"label": "太原市","value":"140100","disabled": false},
{"label": "大同市","value":"140200","disabled": false},
{"label": "阳泉市","value":"140300","disabled": false},
{"label": "长治市","value":"140400","disabled": false},
{"label": "晋城市","value":"140500","disabled": false},
{"label": "朔州市","value":"140600","disabled": false},
{"label": "晋中市","value":"140700","disabled": false},
{"label": "运城市","value":"140800","disabled": false},
{"label": "忻州市","value":"140900","disabled": false},
{"label": "临汾市","value":"141000","disabled": false},
{"label": "吕梁市","value":"141100","disabled": false},
{"label": "呼和浩特市","value":"150100","disabled": false},
{"label": "包头市","value":"150200","disabled": false},
{"label": "乌海市","value":"150300","disabled": false},
{"label": "赤峰市","value":"150400","disabled": false},
{"label": "通辽市","value":"150500","disabled": false},
{"label": "鄂尔多斯市","value":"150600","disabled": false},
{"label": "呼伦贝尔市","value":"150700","disabled": false},
{"label": "巴彦淖尔市","value":"150800","disabled": false},
{"label": "乌兰察布市","value":"150900","disabled": false},
{"label": "兴安盟","value":"152200","disabled": false},
{"label": "锡林郭勒盟","value":"152500","disabled": false},
{"label": "阿拉善盟","value":"152900","disabled": false},
{"label": "沈阳市","value":"210100","disabled": false},
{"label": "大连市","value":"210200","disabled": false},
{"label": "鞍山市","value":"210300","disabled": false},
{"label": "抚顺市","value":"210400","disabled": false},
{"label": "本溪市","value":"210500","disabled": false},
{"label": "丹东市","value":"210600","disabled": false},
{"label": "锦州市","value":"210700","disabled": false},
{"label": "营口市","value":"210800","disabled": false},
{"label": "阜新市","value":"210900","disabled": false},
{"label": "辽阳市","value":"211000","disabled": false},
{"label": "盘锦市","value":"211100","disabled": false},
{"label": "铁岭市","value":"211200","disabled": false},
{"label": "朝阳市","value":"211300","disabled": false},
{"label": "葫芦岛市","value":"211400","disabled": false},
{"label": "长春市","value":"220100","disabled": false},
{"label": "吉林市","value":"220200","disabled": false},
{"label": "四平市","value":"220300","disabled": false},
{"label": "辽源市","value":"220400","disabled": false},
{"label": "通化市","value":"220500","disabled": false},
{"label": "白山市","value":"220600","disabled": false},
{"label": "松原市","value":"220700","disabled": false},
{"label": "白城市","value":"220800","disabled": false},
{"label": "延边自治州","value":"222400","disabled": false},
{"label": "哈尔滨市","value":"230100","disabled": false},
{"label": "齐齐哈尔市","value":"230200","disabled": false},
{"label": "鸡西市","value":"230300","disabled": false},
{"label": "鹤岗市","value":"230400","disabled": false},
{"label": "双鸭山市","value":"230500","disabled": false},
{"label": "大庆市","value":"230600","disabled": false},
{"label": "伊春市","value":"230700","disabled": false},
{"label": "佳木斯市","value":"230800","disabled": false},
{"label": "七台河市","value":"230900","disabled": false},
{"label": "牡丹江市","value":"231000","disabled": false},
{"label": "黑河市","value":"231100","disabled": false},
{"label": "绥化市","value":"231200","disabled": false},
{"label": "大兴安岭地区","value":"232700","disabled": false},
{"label": "黄浦区","value":"310100","disabled": false},
{"label": "卢湾区","value":"310300","disabled": false},
{"label": "徐汇区","value":"310400","disabled": false},
{"label": "长宁区","value":"310500","disabled": false},
{"label": "静安区","value":"310600","disabled": false},
{"label": "普陀区","value":"310700","disabled": false},
{"label": "闸北区","value":"310800","disabled": false},
{"label": "虹口区","value":"310900","disabled": false},
{"label": "杨浦区","value":"311000","disabled": false},
{"label": "闵行区","value":"311200","disabled": false},
{"label": "宝山区","value":"311300","disabled": false},
{"label": "嘉定区","value":"311400","disabled": false},
{"label": "浦东新区","value":"311500","disabled": false},
{"label": "金山区","value":"311600","disabled": false},
{"label": "松江区","value":"311700","disabled": false},
{"label": "青浦区","value":"311800","disabled": false},
{"label": "南汇区","value":"311900","disabled": false},
{"label": "奉贤区","value":"312000","disabled": false},
{"label": "崇明县","value":"312100","disabled": false},
{"label": "南京市","value":"320100","disabled": false},
{"label": "无锡市","value":"320200","disabled": false},
{"label": "徐州市","value":"320300","disabled": false},
{"label": "常州市","value":"320400","disabled": false},
{"label": "苏州市","value":"320500","disabled": false},
{"label": "南通市","value":"320600","disabled": false},
{"label": "连云港市","value":"320700","disabled": false},
{"label": "淮安市","value":"320800","disabled": false},
{"label": "盐城市","value":"320900","disabled": false},
{"label": "扬州市","value":"321000","disabled": false},
{"label": "镇江市","value":"321100","disabled": false},
{"label": "泰州市","value":"321200","disabled": false},
{"label": "宿迁市","value":"321300","disabled": false},
{"label": "杭州市","value":"330100","disabled": false},
{"label": "宁波市","value":"330200","disabled": false},
{"label": "温州市","value":"330300","disabled": false},
{"label": "嘉兴市","value":"330400","disabled": false},
{"label": "湖州市","value":"330500","disabled": false},
{"label": "绍兴市","value":"330600","disabled": false},
{"label": "金华市","value":"330700","disabled": false},
{"label": "衢州市","value":"330800","disabled": false},
{"label": "舟山市","value":"330900","disabled": false},
{"label": "台州市","value":"331000","disabled": false},
{"label": "丽水市","value":"331100","disabled": false},
{"label": "合肥市","value":"340100","disabled": false},
{"label": "芜湖市","value":"340200","disabled": false},
{"label": "蚌埠市","value":"340300","disabled": false},
{"label": "淮南市","value":"340400","disabled": false},
{"label": "马鞍山市","value":"340500","disabled": false},
{"label": "淮北市","value":"340600","disabled": false},
{"label": "铜陵市","value":"340700","disabled": false},
{"label": "安庆市","value":"340800","disabled": false},
{"label": "黄山市","value":"341000","disabled": false},
{"label": "滁州市","value":"341100","disabled": false},
{"label": "阜阳市","value":"341200","disabled": false},
{"label": "宿州市","value":"341300","disabled": false},
{"label": "巢湖市","value":"341400","disabled": false},
{"label": "六安市","value":"341500","disabled": false},
{"label": "亳州市","value":"341600","disabled": false},
{"label": "池州市","value":"341700","disabled": false},
{"label": "宣城市","value":"341800","disabled": false},
{"label": "福州市","value":"350100","disabled": false},
{"label": "厦门市","value":"350200","disabled": false},
{"label": "莆田市","value":"350300","disabled": false},
{"label": "三明市","value":"350400","disabled": false},
{"label": "泉州市","value":"350500","disabled": false},
{"label": "漳州市","value":"350600","disabled": false},
{"label": "南平市","value":"350700","disabled": false},
{"label": "龙岩市","value":"350800","disabled": false},
{"label": "宁德市","value":"350900","disabled": false},
{"label": "南昌市","value":"360100","disabled": false},
{"label": "景德镇市","value":"360200","disabled": false},
{"label": "萍乡市","value":"360300","disabled": false},
{"label": "九江市","value":"360400","disabled": false},
{"label": "新余市","value":"360500","disabled": false},
{"label": "鹰潭市","value":"360600","disabled": false},
{"label": "赣州市","value":"360700","disabled": false},
{"label": "吉安市","value":"360800","disabled": false},
{"label": "宜春市","value":"360900","disabled": false},
{"label": "抚州市","value":"361000","disabled": false},
{"label": "上饶市","value":"361100","disabled": false},
{"label": "济南市","value":"370100","disabled": false},
{"label": "青岛市","value":"370200","disabled": false},
{"label": "淄博市","value":"370300","disabled": false},
{"label": "枣庄市","value":"370400","disabled": false},
{"label": "东营市","value":"370500","disabled": false},
{"label": "烟台市","value":"370600","disabled": false},
{"label": "潍坊市","value":"370700","disabled": false},
{"label": "济宁市","value":"370800","disabled": false},
{"label": "泰安市","value":"370900","disabled": false},
{"label": "威海市","value":"371000","disabled": false},
{"label": "日照市","value":"371100","disabled": false},
{"label": "莱芜市","value":"371200","disabled": false},
{"label": "临沂市","value":"371300","disabled": false},
{"label": "德州市","value":"371400","disabled": false},
{"label": "聊城市","value":"371500","disabled": false},
{"label": "滨州市","value":"371600","disabled": false},
{"label": "菏泽市","value":"371700","disabled": false},
{"label": "郑州市","value":"410100","disabled": false},
{"label": "开封市","value":"410200","disabled": false},
{"label": "洛阳市","value":"410300","disabled": false},
{"label": "平顶山市","value":"410400","disabled": false},
{"label": "安阳市","value":"410500","disabled": false},
{"label": "鹤壁市","value":"410600","disabled": false},
{"label": "新乡市","value":"410700","disabled": false},
{"label": "焦作市","value":"410800","disabled": false},
{"label": "濮阳市","value":"410900","disabled": false},
{"label": "许昌市","value":"411000","disabled": false},
{"label": "漯河市","value":"411100","disabled": false},
{"label": "三门峡市","value":"411200","disabled": false},
{"label": "南阳市","value":"411300","disabled": false},
{"label": "商丘市","value":"411400","disabled": false},
{"label": "信阳市","value":"411500","disabled": false},
{"label": "周口市","value":"411600","disabled": false},
{"label": "驻马店市","value":"411700","disabled": false},
{"label": "济源市","value":"419000","disabled": false},
{"label": "武汉市","value":"420100","disabled": false},
{"label": "黄石市","value":"420200","disabled": false},
{"label": "十堰市","value":"420300","disabled": false},
{"label": "宜昌市","value":"420500","disabled": false},
{"label": "襄阳市","value":"420600","disabled": false},
{"label": "鄂州市","value":"420700","disabled": false},
{"label": "荆门市","value":"420800","disabled": false},
{"label": "孝感市","value":"420900","disabled": false},
{"label": "荆州市","value":"421000","disabled": false},
{"label": "黄冈市","value":"421100","disabled": false},
{"label": "咸宁市","value":"421200","disabled": false},
{"label": "随州市","value":"421300","disabled": false},
{"label": "恩施自治州","value":"422800","disabled": false},
{"label": "仙桃市","value":"429000","disabled": false},
{"label": "潜江市","value":"429100","disabled": false},
{"label": "天门市","value":"429200","disabled": false},
{"label": "神农架林区","value":"429300","disabled": false},
{"label": "长沙市","value":"430100","disabled": false},
{"label": "株洲市","value":"430200","disabled": false},
{"label": "湘潭市","value":"430300","disabled": false},
{"label": "衡阳市","value":"430400","disabled": false},
{"label": "邵阳市","value":"430500","disabled": false},
{"label": "岳阳市","value":"430600","disabled": false},
{"label": "常德市","value":"430700","disabled": false},
{"label": "张家界市","value":"430800","disabled": false},
{"label": "益阳市","value":"430900","disabled": false},
{"label": "郴州市","value":"431000","disabled": false},
{"label": "永州市","value":"431100","disabled": false},
{"label": "怀化市","value":"431200","disabled": false},
{"label": "娄底市","value":"431300","disabled": false},
{"label": "湘西自治州","value":"433100","disabled": false},
{"label": "广州市","value":"440100","disabled": false},
{"label": "韶关市","value":"440200","disabled": false},
{"label": "深圳市","value":"440300","disabled": false},
{"label": "珠海市","value":"440400","disabled": false},
{"label": "汕头市","value":"440500","disabled": false},
{"label": "佛山市","value":"440600","disabled": false},
{"label": "江门市","value":"440700","disabled": false},
{"label": "湛江市","value":"440800","disabled": false},
{"label": "茂名市","value":"440900","disabled": false},
{"label": "肇庆市","value":"441200","disabled": false},
{"label": "惠州市","value":"441300","disabled": false},
{"label": "梅州市","value":"441400","disabled": false},
{"label": "汕尾市","value":"441500","disabled": false},
{"label": "河源市","value":"441600","disabled": false},
{"label": "阳江市","value":"441700","disabled": false},
{"label": "清远市","value":"441800","disabled": false},
{"label": "东莞市","value":"441900","disabled": false},
{"label": "中山市","value":"442000","disabled": false},
{"label": "潮州市","value":"445100","disabled": false},
{"label": "揭阳市","value":"445200","disabled": false},
{"label": "云浮市","value":"445300","disabled": false},
{"label": "南宁市","value":"450100","disabled": false},
{"label": "柳州市","value":"450200","disabled": false},
{"label": "桂林市","value":"450300","disabled": false},
{"label": "梧州市","value":"450400","disabled": false},
{"label": "北海市","value":"450500","disabled": false},
{"label": "防城港市","value":"450600","disabled": false},
{"label": "钦州市","value":"450700","disabled": false},
{"label": "贵港市","value":"450800","disabled": false},
{"label": "玉林市","value":"450900","disabled": false},
{"label": "百色市","value":"451000","disabled": false},
{"label": "贺州市","value":"451100","disabled": false},
{"label": "河池市","value":"451200","disabled": false},
{"label": "来宾市","value":"451300","disabled": false},
{"label": "崇左市","value":"451400","disabled": false},
{"label": "海口市","value":"460100","disabled": false},
{"label": "三亚市","value":"460200","disabled": false},
{"label": "五指山市","value":"460300","disabled": false},
{"label": "琼海市","value":"460400","disabled": false},
{"label": "儋州市","value":"460500","disabled": false},
{"label": "文昌市","value":"460600","disabled": false},
{"label": "万宁市","value":"460700","disabled": false},
{"label": "东方市","value":"460800","disabled": false},
{"label": "定安县","value":"460900","disabled": false},
{"label": "屯昌县","value":"461000","disabled": false},
{"label": "澄迈县","value":"461100","disabled": false},
{"label": "临高县","value":"461200","disabled": false},
{"label": "白沙黎族自治县","value":"461300","disabled": false},
{"label": "昌江黎族自治县","value":"461400","disabled": false},
{"label": "乐东黎族自治县","value":"461500","disabled": false},
{"label": "陵水黎族自治县","value":"461600","disabled": false},
{"label": "保亭黎族苗族自治县","value":"461700","disabled": false},
{"label": "琼中黎族苗族自治县","value":"461800","disabled": false},
{"label": "西沙群岛","value":"461900","disabled": false},
{"label": "南沙群岛","value":"462000","disabled": false},
{"label": "中沙群岛","value":"462100","disabled": false},
{"label": "万州区","value":"500100","disabled": false},
{"label": "涪陵区","value":"500200","disabled": false},
{"label": "渝中区","value":"500300","disabled": false},
{"label": "大渡口区","value":"500400","disabled": false},
{"label": "江北区","value":"500500","disabled": false},
{"label": "沙坪坝区","value":"500600","disabled": false},
{"label": "九龙坡区","value":"500700","disabled": false},
{"label": "南岸区","value":"500800","disabled": false},
{"label": "北碚区","value":"500900","disabled": false},
{"label": "万盛区","value":"501000","disabled": false},
{"label": "双桥区","value":"501100","disabled": false},
{"label": "渝北区","value":"501200","disabled": false},
{"label": "巴南区","value":"501300","disabled": false},
{"label": "黔江区","value":"501400","disabled": false},
{"label": "长寿区","value":"501500","disabled": false},
{"label": "江津区","value":"501600","disabled": false},
{"label": "合川区","value":"501700","disabled": false},
{"label": "永川区","value":"501800","disabled": false},
{"label": "南川区","value":"501900","disabled": false},
{"label": "綦江县","value":"502000","disabled": false},
{"label": "潼南县","value":"502100","disabled": false},
{"label": "铜梁县","value":"502200","disabled": false},
{"label": "大足县","value":"502300","disabled": false},
{"label": "荣昌县","value":"502400","disabled": false},
{"label": "璧山县","value":"502500","disabled": false},
{"label": "梁平县","value":"502600","disabled": false},
{"label": "城口县","value":"502700","disabled": false},
{"label": "丰都县","value":"502800","disabled": false},
{"label": "垫江县","value":"502900","disabled": false},
{"label": "武隆县","value":"503000","disabled": false},
{"label": "忠县","value":"503100","disabled": false},
{"label": "开县","value":"503200","disabled": false},
{"label": "云阳县","value":"503300","disabled": false},
{"label": "奉节县","value":"503400","disabled": false},
{"label": "巫山县","value":"503500","disabled": false},
{"label": "巫溪县","value":"503600","disabled": false},
{"label": "石柱土家族自治县","value":"503700","disabled": false},
{"label": "秀山土家族苗族自治县","value":"503800","disabled": false},
{"label": "酉阳土家族苗族自治县","value":"504000","disabled": false},
{"label": "彭水苗族土家族自治县","value":"504100","disabled": false},
{"label": "成都市","value":"510100","disabled": false},
{"label": "自贡市","value":"510300","disabled": false},
{"label": "攀枝花市","value":"510400","disabled": false},
{"label": "泸州市","value":"510500","disabled": false},
{"label": "德阳市","value":"510600","disabled": false},
{"label": "绵阳市","value":"510700","disabled": false},
{"label": "广元市","value":"510800","disabled": false},
{"label": "遂宁市","value":"510900","disabled": false},
{"label": "内江市","value":"511000","disabled": false},
{"label": "乐山市","value":"511100","disabled": false},
{"label": "南充市","value":"511300","disabled": false},
{"label": "眉山市","value":"511400","disabled": false},
{"label": "宜宾市","value":"511500","disabled": false},
{"label": "广安市","value":"511600","disabled": false},
{"label": "达州市","value":"511700","disabled": false},
{"label": "雅安市","value":"511800","disabled": false},
{"label": "巴中市","value":"511900","disabled": false},
{"label": "资阳市","value":"512000","disabled": false},
{"label": "阿坝自治州","value":"513200","disabled": false},
{"label": "甘孜自治州","value":"513300","disabled": false},
{"label": "凉山自治州","value":"513400","disabled": false},
{"label": "贵阳市","value":"520100","disabled": false},
{"label": "六盘水市","value":"520200","disabled": false},
{"label": "遵义市","value":"520300","disabled": false},
{"label": "安顺市","value":"520400","disabled": false},
{"label": "铜仁地区","value":"522200","disabled": false},
{"label": "黔西南布依族苗族自治州","value":"522300","disabled": false},
{"label": "毕节地区","value":"522400","disabled": false},
{"label": "黔东南苗族侗族自治州","value":"522600","disabled": false},
{"label": "黔南布依族苗族自治州","value":"522700","disabled": false},
{"label": "昆明市","value":"530100","disabled": false},
{"label": "曲靖市","value":"530300","disabled": false},
{"label": "玉溪市","value":"530400","disabled": false},
{"label": "保山市","value":"530500","disabled": false},
{"label": "昭通市","value":"530600","disabled": false},
{"label": "丽江市","value":"530700","disabled": false},
{"label": "普洱市","value":"530800","disabled": false},
{"label": "临沧市","value":"530900","disabled": false},
{"label": "楚雄自治州","value":"532300","disabled": false},
{"label": "红河自治州","value":"532500","disabled": false},
{"label": "文山自治州","value":"532600","disabled": false},
{"label": "西双版自治州","value":"532800","disabled": false},
{"label": "大理自治州","value":"532900","disabled": false},
{"label": "德宏自治州","value":"533100","disabled": false},
{"label": "怒江自治州","value":"533300","disabled": false},
{"label": "迪庆自治州","value":"533400","disabled": false},
{"label": "拉萨市","value":"540100","disabled": false},
{"label": "昌都地区","value":"542100","disabled": false},
{"label": "山南地区","value":"542200","disabled": false},
{"label": "日喀则地区","value":"542300","disabled": false},
{"label": "那曲地区","value":"542400","disabled": false},
{"label": "阿里地区","value":"542500","disabled": false},
{"label": "林芝地区","value":"542600","disabled": false},
{"label": "西安市","value":"610100","disabled": false},
{"label": "铜川市","value":"610200","disabled": false},
{"label": "宝鸡市","value":"610300","disabled": false},
{"label": "咸阳市","value":"610400","disabled": false},
{"label": "渭南市","value":"610500","disabled": false},
{"label": "延安市","value":"610600","disabled": false},
{"label": "汉中市","value":"610700","disabled": false},
{"label": "榆林市","value":"610800","disabled": false},
{"label": "安康市","value":"610900","disabled": false},
{"label": "商洛市","value":"611000","disabled": false},
{"label": "兰州市","value":"620100","disabled": false},
{"label": "嘉峪关市","value":"620200","disabled": false},
{"label": "金昌市","value":"620300","disabled": false},
{"label": "白银市","value":"620400","disabled": false},
{"label": "天水市","value":"620500","disabled": false},
{"label": "武威市","value":"620600","disabled": false},
{"label": "张掖市","value":"620700","disabled": false},
{"label": "平凉市","value":"620800","disabled": false},
{"label": "酒泉市","value":"620900","disabled": false},
{"label": "庆阳市","value":"621000","disabled": false},
{"label": "定西市","value":"621100","disabled": false},
{"label": "陇南市","value":"621200","disabled": false},
{"label": "临夏自治州","value":"622900","disabled": false},
{"label": "甘南自治州","value":"623000","disabled": false},
{"label": "西宁市","value":"630100","disabled": false},
{"label": "海东地区","value":"632100","disabled": false},
{"label": "海北自治州","value":"632200","disabled": false},
{"label": "黄南自治州","value":"632300","disabled": false},
{"label": "海南自治州","value":"632500","disabled": false},
{"label": "果洛自治州","value":"632600","disabled": false},
{"label": "玉树自治州","value":"632700","disabled": false},
{"label": "海西自治州","value":"632800","disabled": false},
{"label": "银川市","value":"640100","disabled": false},
{"label": "石嘴山市","value":"640200","disabled": false},
{"label": "吴忠市","value":"640300","disabled": false},
{"label": "固原市","value":"640400","disabled": false},
{"label": "中卫市","value":"640500","disabled": false},
{"label": "乌鲁木齐市","value":"650100","disabled": false},
{"label": "克拉玛依市","value":"650200","disabled": false},
{"label": "吐鲁番地区","value":"652100","disabled": false},
{"label": "哈密地区","value":"652200","disabled": false},
{"label": "昌吉自治州","value":"652300","disabled": false},
{"label": "博尔塔拉蒙古自治州","value":"652700","disabled": false},
{"label": "巴音郭楞蒙古自治州","value":"652800","disabled": false},
{"label": "阿克苏地区","value":"652900","disabled": false},
{"label": "克孜勒苏柯尔克孜自治州","value":"653000","disabled": false},
{"label": "喀什地区","value":"653100","disabled": false},
{"label": "和田地区","value":"653200","disabled": false},
{"label": "伊犁哈萨克自治州","value":"654000","disabled": false},
{"label": "塔城地区","value":"654200","disabled": false},
{"label": "阿勒泰地区","value":"654300","disabled": false},
{"label": "石河子市","value":"654400","disabled": false},
{"label": "阿拉尔市","value":"654500","disabled": false},
{"label": "图木舒克市","value":"654600","disabled": false},
{"label": "五家渠市","value":"654700","disabled": false},
{"label": "台北市","value":"710100","disabled": false},
{"label": "高雄市","value":"710200","disabled": false},
{"label": "基隆市","value":"710300","disabled": false},
{"label": "台中市","value":"710400","disabled": false},
{"label": "台南市","value":"710500","disabled": false},
{"label": "新竹市","value":"710600","disabled": false},
{"label": "嘉义市","value":"710700","disabled": false},
{"label": "台北县","value":"710800","disabled": false},
{"label": "宜兰县","value":"710900","disabled": false},
{"label": "新竹县","value":"711000","disabled": false},
{"label": "桃园县","value":"711100","disabled": false},
{"label": "苗栗县","value":"711200","disabled": false},
{"label": "台中县","value":"711300","disabled": false},
{"label": "彰化县","value":"711400","disabled": false},
{"label": "南投县","value":"711500","disabled": false},
{"label": "嘉义县","value":"711600","disabled": false},
{"label": "云林县","value":"711700","disabled": false},
{"label": "台南县","value":"711800","disabled": false},
{"label": "高雄县","value":"711900","disabled": false},
{"label": "屏东县","value":"712000","disabled": false},
{"label": "台东县","value":"712100","disabled": false},
{"label": "花莲县","value":"712200","disabled": false},
{"label": "澎湖县","value":"712300","disabled": false},
  ],
  Areas:[
    {"label": "长安区","value":"130102","disabled": false},
{"label": "桥东区","value":"130103","disabled": false},
{"label": "桥西区","value":"130104","disabled": false},
{"label": "新华区","value":"130105","disabled": false},
{"label": "井陉矿区","value":"130107","disabled": false},
{"label": "裕华区","value":"130108","disabled": false},
{"label": "井陉县","value":"130121","disabled": false},
{"label": "正定县","value":"130123","disabled": false},
{"label": "栾城县","value":"130124","disabled": false},
{"label": "行唐县","value":"130125","disabled": false},
{"label": "灵寿县","value":"130126","disabled": false},
{"label": "高邑县","value":"130127","disabled": false},
{"label": "深泽县","value":"130128","disabled": false},
{"label": "赞皇县","value":"130129","disabled": false},
{"label": "无极县","value":"130130","disabled": false},
{"label": "平山县","value":"130131","disabled": false},
{"label": "元氏县","value":"130132","disabled": false},
{"label": "赵县","value":"130133","disabled": false},
{"label": "辛集市","value":"130181","disabled": false},
{"label": "藁城市","value":"130182","disabled": false},
{"label": "晋州市","value":"130183","disabled": false},
{"label": "新乐市","value":"130184","disabled": false},
{"label": "鹿泉市","value":"130185","disabled": false},
{"label": "路南区","value":"130202","disabled": false},
{"label": "路北区","value":"130203","disabled": false},
{"label": "古冶区","value":"130204","disabled": false},
{"label": "开平区","value":"130205","disabled": false},
{"label": "丰南区","value":"130207","disabled": false},
{"label": "丰润区","value":"130208","disabled": false},
{"label": "滦县","value":"130223","disabled": false},
{"label": "滦南县","value":"130224","disabled": false},
{"label": "乐亭县","value":"130225","disabled": false},
{"label": "迁西县","value":"130227","disabled": false},
{"label": "玉田县","value":"130229","disabled": false},
{"label": "唐海县","value":"130230","disabled": false},
{"label": "遵化市","value":"130281","disabled": false},
{"label": "迁安市","value":"130283","disabled": false},
{"label": "海港区","value":"130302","disabled": false},
{"label": "山海关区","value":"130303","disabled": false},
{"label": "北戴河区","value":"130304","disabled": false},
{"label": "青龙满族自治县","value":"130321","disabled": false},
{"label": "昌黎县","value":"130322","disabled": false},
{"label": "抚宁县","value":"130323","disabled": false},
{"label": "卢龙县","value":"130324","disabled": false},
{"label": "邯山区","value":"130402","disabled": false},
{"label": "丛台区","value":"130403","disabled": false},
{"label": "复兴区","value":"130404","disabled": false},
{"label": "峰峰矿区","value":"130406","disabled": false},
{"label": "邯郸县","value":"130421","disabled": false},
{"label": "临漳县","value":"130423","disabled": false},
{"label": "成安县","value":"130424","disabled": false},
{"label": "大名县","value":"130425","disabled": false},
{"label": "涉县","value":"130426","disabled": false},
{"label": "磁县","value":"130427","disabled": false},
{"label": "肥乡县","value":"130428","disabled": false},
{"label": "永年县","value":"130429","disabled": false},
{"label": "邱县","value":"130430","disabled": false},
{"label": "鸡泽县","value":"130431","disabled": false},
{"label": "广平县","value":"130432","disabled": false},
{"label": "馆陶县","value":"130433","disabled": false},
{"label": "魏县","value":"130434","disabled": false},
{"label": "曲周县","value":"130435","disabled": false},
{"label": "武安市","value":"130481","disabled": false},
{"label": "桥东区","value":"130502","disabled": false},
{"label": "桥西区","value":"130503","disabled": false},
{"label": "邢台县","value":"130521","disabled": false},
{"label": "临城县","value":"130522","disabled": false},
{"label": "内丘县","value":"130523","disabled": false},
{"label": "柏乡县","value":"130524","disabled": false},
{"label": "隆尧县","value":"130525","disabled": false},
{"label": "任县","value":"130526","disabled": false},
{"label": "南和县","value":"130527","disabled": false},
{"label": "宁晋县","value":"130528","disabled": false},
{"label": "巨鹿县","value":"130529","disabled": false},
{"label": "新河县","value":"130530","disabled": false},
{"label": "广宗县","value":"130531","disabled": false},
{"label": "平乡县","value":"130532","disabled": false},
{"label": "威县","value":"130533","disabled": false},
{"label": "清河县","value":"130534","disabled": false},
{"label": "临西县","value":"130535","disabled": false},
{"label": "南宫市","value":"130581","disabled": false},
{"label": "沙河市","value":"130582","disabled": false},
{"label": "新市区","value":"130602","disabled": false},
{"label": "北市区","value":"130603","disabled": false},
{"label": "南市区","value":"130604","disabled": false},
{"label": "满城县","value":"130621","disabled": false},
{"label": "清苑县","value":"130622","disabled": false},
{"label": "涞水县","value":"130623","disabled": false},
{"label": "阜平县","value":"130624","disabled": false},
{"label": "徐水县","value":"130625","disabled": false},
{"label": "定兴县","value":"130626","disabled": false},
{"label": "唐县","value":"130627","disabled": false},
{"label": "高阳县","value":"130628","disabled": false},
{"label": "容城县","value":"130629","disabled": false},
{"label": "涞源县","value":"130630","disabled": false},
{"label": "望都县","value":"130631","disabled": false},
{"label": "安新县","value":"130632","disabled": false},
{"label": "易县","value":"130633","disabled": false},
{"label": "曲阳县","value":"130634","disabled": false},
{"label": "蠡县","value":"130635","disabled": false},
{"label": "顺平县","value":"130636","disabled": false},
{"label": "博野县","value":"130637","disabled": false},
{"label": "雄县","value":"130638","disabled": false},
{"label": "涿州市","value":"130681","disabled": false},
{"label": "定州市","value":"130682","disabled": false},
{"label": "安国市","value":"130683","disabled": false},
{"label": "高碑店市","value":"130684","disabled": false},
{"label": "桥东区","value":"130702","disabled": false},
{"label": "桥西区","value":"130703","disabled": false},
{"label": "宣化区","value":"130705","disabled": false},
{"label": "下花园区","value":"130706","disabled": false},
{"label": "宣化县","value":"130721","disabled": false},
{"label": "张北县","value":"130722","disabled": false},
{"label": "康保县","value":"130723","disabled": false},
{"label": "沽源县","value":"130724","disabled": false},
{"label": "尚义县","value":"130725","disabled": false},
{"label": "蔚县","value":"130726","disabled": false},
{"label": "阳原县","value":"130727","disabled": false},
{"label": "怀安县","value":"130728","disabled": false},
{"label": "万全县","value":"130729","disabled": false},
{"label": "怀来县","value":"130730","disabled": false},
{"label": "涿鹿县","value":"130731","disabled": false},
{"label": "赤城县","value":"130732","disabled": false},
{"label": "崇礼县","value":"130733","disabled": false},
{"label": "双桥区","value":"130802","disabled": false},
{"label": "双滦区","value":"130803","disabled": false},
{"label": "鹰手营子矿区","value":"130804","disabled": false},
{"label": "承德县","value":"130821","disabled": false},
{"label": "兴隆县","value":"130822","disabled": false},
{"label": "平泉县","value":"130823","disabled": false},
{"label": "滦平县","value":"130824","disabled": false},
{"label": "隆化县","value":"130825","disabled": false},
{"label": "丰宁满族自治县","value":"130826","disabled": false},
{"label": "宽城满族自治县","value":"130827","disabled": false},
{"label": "围场满族蒙古族自治县","value":"130828","disabled": false},
{"label": "新华区","value":"130902","disabled": false},
{"label": "运河区","value":"130903","disabled": false},
{"label": "沧县","value":"130921","disabled": false},
{"label": "青县","value":"130922","disabled": false},
{"label": "东光县","value":"130923","disabled": false},
{"label": "海兴县","value":"130924","disabled": false},
{"label": "盐山县","value":"130925","disabled": false},
{"label": "肃宁县","value":"130926","disabled": false},
{"label": "南皮县","value":"130927","disabled": false},
{"label": "吴桥县","value":"130928","disabled": false},
{"label": "献县","value":"130929","disabled": false},
{"label": "孟村回族自治县","value":"130930","disabled": false},
{"label": "泊头市","value":"130981","disabled": false},
{"label": "任丘市","value":"130982","disabled": false},
{"label": "黄骅市","value":"130983","disabled": false},
{"label": "河间市","value":"130984","disabled": false},
{"label": "安次区","value":"131002","disabled": false},
{"label": "广阳区","value":"131003","disabled": false},
{"label": "固安县","value":"131022","disabled": false},
{"label": "永清县","value":"131023","disabled": false},
{"label": "香河县","value":"131024","disabled": false},
{"label": "大城县","value":"131025","disabled": false},
{"label": "文安县","value":"131026","disabled": false},
{"label": "大厂回族自治县","value":"131028","disabled": false},
{"label": "霸州市","value":"131081","disabled": false},
{"label": "三河市","value":"131082","disabled": false},
{"label": "桃城区","value":"131102","disabled": false},
{"label": "枣强县","value":"131121","disabled": false},
{"label": "武邑县","value":"131122","disabled": false},
{"label": "武强县","value":"131123","disabled": false},
{"label": "饶阳县","value":"131124","disabled": false},
{"label": "安平县","value":"131125","disabled": false},
{"label": "故城县","value":"131126","disabled": false},
{"label": "景县","value":"131127","disabled": false},
{"label": "阜城县","value":"131128","disabled": false},
{"label": "冀州市","value":"131181","disabled": false},
{"label": "深州市","value":"131182","disabled": false},
{"label": "小店区","value":"140105","disabled": false},
{"label": "迎泽区","value":"140106","disabled": false},
{"label": "杏花岭区","value":"140107","disabled": false},
{"label": "尖草坪区","value":"140108","disabled": false},
{"label": "万柏林区","value":"140109","disabled": false},
{"label": "晋源区","value":"140110","disabled": false},
{"label": "清徐县","value":"140121","disabled": false},
{"label": "阳曲县","value":"140122","disabled": false},
{"label": "娄烦县","value":"140123","disabled": false},
{"label": "古交市","value":"140181","disabled": false},
{"label": "城区","value":"140202","disabled": false},
{"label": "矿区","value":"140203","disabled": false},
{"label": "南郊区","value":"140211","disabled": false},
{"label": "新荣区","value":"140212","disabled": false},
{"label": "阳高县","value":"140221","disabled": false},
{"label": "天镇县","value":"140222","disabled": false},
{"label": "广灵县","value":"140223","disabled": false},
{"label": "灵丘县","value":"140224","disabled": false},
{"label": "浑源县","value":"140225","disabled": false},
{"label": "左云县","value":"140226","disabled": false},
{"label": "大同县","value":"140227","disabled": false},
{"label": "城区","value":"140302","disabled": false},
{"label": "矿区","value":"140303","disabled": false},
{"label": "郊区","value":"140311","disabled": false},
{"label": "平定县","value":"140321","disabled": false},
{"label": "盂县","value":"140322","disabled": false},
{"label": "城区","value":"140402","disabled": false},
{"label": "郊区","value":"140411","disabled": false},
{"label": "长治县","value":"140421","disabled": false},
{"label": "襄垣县","value":"140423","disabled": false},
{"label": "屯留县","value":"140424","disabled": false},
{"label": "平顺县","value":"140425","disabled": false},
{"label": "黎城县","value":"140426","disabled": false},
{"label": "壶关县","value":"140427","disabled": false},
{"label": "长子县","value":"140428","disabled": false},
{"label": "武乡县","value":"140429","disabled": false},
{"label": "沁县","value":"140430","disabled": false},
{"label": "沁源县","value":"140431","disabled": false},
{"label": "潞城市","value":"140481","disabled": false},
{"label": "城区","value":"140502","disabled": false},
{"label": "沁水县","value":"140521","disabled": false},
{"label": "阳城县","value":"140522","disabled": false},
{"label": "陵川县","value":"140524","disabled": false},
{"label": "泽州县","value":"140525","disabled": false},
{"label": "高平市","value":"140581","disabled": false},
{"label": "朔城区","value":"140602","disabled": false},
{"label": "平鲁区","value":"140603","disabled": false},
{"label": "山阴县","value":"140621","disabled": false},
{"label": "应县","value":"140622","disabled": false},
{"label": "右玉县","value":"140623","disabled": false},
{"label": "怀仁县","value":"140624","disabled": false},
{"label": "榆次区","value":"140702","disabled": false},
{"label": "榆社县","value":"140721","disabled": false},
{"label": "左权县","value":"140722","disabled": false},
{"label": "和顺县","value":"140723","disabled": false},
{"label": "昔阳县","value":"140724","disabled": false},
{"label": "寿阳县","value":"140725","disabled": false},
{"label": "太谷县","value":"140726","disabled": false},
{"label": "祁县","value":"140727","disabled": false},
{"label": "平遥县","value":"140728","disabled": false},
{"label": "灵石县","value":"140729","disabled": false},
{"label": "介休市","value":"140781","disabled": false},
{"label": "盐湖区","value":"140802","disabled": false},
{"label": "临猗县","value":"140821","disabled": false},
{"label": "万荣县","value":"140822","disabled": false},
{"label": "闻喜县","value":"140823","disabled": false},
{"label": "稷山县","value":"140824","disabled": false},
{"label": "新绛县","value":"140825","disabled": false},
{"label": "绛县","value":"140826","disabled": false},
{"label": "垣曲县","value":"140827","disabled": false},
{"label": "夏县","value":"140828","disabled": false},
{"label": "平陆县","value":"140829","disabled": false},
{"label": "芮城县","value":"140830","disabled": false},
{"label": "永济市","value":"140881","disabled": false},
{"label": "河津市","value":"140882","disabled": false},
{"label": "忻府区","value":"140902","disabled": false},
{"label": "定襄县","value":"140921","disabled": false},
{"label": "五台县","value":"140922","disabled": false},
{"label": "代县","value":"140923","disabled": false},
{"label": "繁峙县","value":"140924","disabled": false},
{"label": "宁武县","value":"140925","disabled": false},
{"label": "静乐县","value":"140926","disabled": false},
{"label": "神池县","value":"140927","disabled": false},
{"label": "五寨县","value":"140928","disabled": false},
{"label": "岢岚县","value":"140929","disabled": false},
{"label": "河曲县","value":"140930","disabled": false},
{"label": "保德县","value":"140931","disabled": false},
{"label": "偏关县","value":"140932","disabled": false},
{"label": "原平市","value":"140981","disabled": false},
{"label": "尧都区","value":"141002","disabled": false},
{"label": "曲沃县","value":"141021","disabled": false},
{"label": "翼城县","value":"141022","disabled": false},
{"label": "襄汾县","value":"141023","disabled": false},
{"label": "洪洞县","value":"141024","disabled": false},
{"label": "古县","value":"141025","disabled": false},
{"label": "安泽县","value":"141026","disabled": false},
{"label": "浮山县","value":"141027","disabled": false},
{"label": "吉县","value":"141028","disabled": false},
{"label": "乡宁县","value":"141029","disabled": false},
{"label": "大宁县","value":"141030","disabled": false},
{"label": "隰县","value":"141031","disabled": false},
{"label": "永和县","value":"141032","disabled": false},
{"label": "蒲县","value":"141033","disabled": false},
{"label": "汾西县","value":"141034","disabled": false},
{"label": "侯马市","value":"141081","disabled": false},
{"label": "霍州市","value":"141082","disabled": false},
{"label": "离石区","value":"141102","disabled": false},
{"label": "文水县","value":"141121","disabled": false},
{"label": "交城县","value":"141122","disabled": false},
{"label": "兴县","value":"141123","disabled": false},
{"label": "临县","value":"141124","disabled": false},
{"label": "柳林县","value":"141125","disabled": false},
{"label": "石楼县","value":"141126","disabled": false},
{"label": "岚县","value":"141127","disabled": false},
{"label": "方山县","value":"141128","disabled": false},
{"label": "中阳县","value":"141129","disabled": false},
{"label": "交口县","value":"141130","disabled": false},
{"label": "孝义市","value":"141181","disabled": false},
{"label": "汾阳市","value":"141182","disabled": false},
{"label": "新城区","value":"150102","disabled": false},
{"label": "回民区","value":"150103","disabled": false},
{"label": "玉泉区","value":"150104","disabled": false},
{"label": "赛罕区","value":"150105","disabled": false},
{"label": "土默特左旗","value":"150121","disabled": false},
{"label": "托克托县","value":"150122","disabled": false},
{"label": "和林格尔县","value":"150123","disabled": false},
{"label": "清水河县","value":"150124","disabled": false},
{"label": "武川县","value":"150125","disabled": false},
{"label": "东河区","value":"150202","disabled": false},
{"label": "昆都仑区","value":"150203","disabled": false},
{"label": "青山区","value":"150204","disabled": false},
{"label": "石拐区","value":"150205","disabled": false},
{"label": "白云鄂博矿区","value":"150206","disabled": false},
{"label": "九原区","value":"150207","disabled": false},
{"label": "土默特右旗","value":"150221","disabled": false},
{"label": "固阳县","value":"150222","disabled": false},
{"label": "达尔罕茂明安联合旗","value":"150223","disabled": false},
{"label": "海勃湾区","value":"150302","disabled": false},
{"label": "海南区","value":"150303","disabled": false},
{"label": "乌达区","value":"150304","disabled": false},
{"label": "红山区","value":"150402","disabled": false},
{"label": "元宝山区","value":"150403","disabled": false},
{"label": "松山区","value":"150404","disabled": false},
{"label": "阿鲁科尔沁旗","value":"150421","disabled": false},
{"label": "巴林左旗","value":"150422","disabled": false},
{"label": "巴林右旗","value":"150423","disabled": false},
{"label": "林西县","value":"150424","disabled": false},
{"label": "克什克腾旗","value":"150425","disabled": false},
{"label": "翁牛特旗","value":"150426","disabled": false},
{"label": "喀喇沁旗","value":"150428","disabled": false},
{"label": "宁城县","value":"150429","disabled": false},
{"label": "敖汉旗","value":"150430","disabled": false},
{"label": "科尔沁区","value":"150502","disabled": false},
{"label": "科尔沁左翼中旗","value":"150521","disabled": false},
{"label": "科尔沁左翼后旗","value":"150522","disabled": false},
{"label": "开鲁县","value":"150523","disabled": false},
{"label": "库伦旗","value":"150524","disabled": false},
{"label": "奈曼旗","value":"150525","disabled": false},
{"label": "扎鲁特旗","value":"150526","disabled": false},
{"label": "霍林郭勒市","value":"150581","disabled": false},
{"label": "东胜区","value":"150602","disabled": false},
{"label": "达拉特旗","value":"150621","disabled": false},
{"label": "准格尔旗","value":"150622","disabled": false},
{"label": "鄂托克前旗","value":"150623","disabled": false},
{"label": "鄂托克旗","value":"150624","disabled": false},
{"label": "杭锦旗","value":"150625","disabled": false},
{"label": "乌审旗","value":"150626","disabled": false},
{"label": "伊金霍洛旗","value":"150627","disabled": false},
{"label": "海拉尔区","value":"150702","disabled": false},
{"label": "阿荣旗","value":"150721","disabled": false},
{"label": "莫力达瓦达斡尔族自治旗","value":"150722","disabled": false},
{"label": "鄂伦春自治旗","value":"150723","disabled": false},
{"label": "鄂温克族自治旗","value":"150724","disabled": false},
{"label": "陈巴尔虎旗","value":"150725","disabled": false},
{"label": "新巴尔虎左旗","value":"150726","disabled": false},
{"label": "新巴尔虎右旗","value":"150727","disabled": false},
{"label": "满洲里市","value":"150781","disabled": false},
{"label": "牙克石市","value":"150782","disabled": false},
{"label": "扎兰屯市","value":"150783","disabled": false},
{"label": "额尔古纳市","value":"150784","disabled": false},
{"label": "根河市","value":"150785","disabled": false},
{"label": "临河区","value":"150802","disabled": false},
{"label": "五原县","value":"150821","disabled": false},
{"label": "磴口县","value":"150822","disabled": false},
{"label": "乌拉特前旗","value":"150823","disabled": false},
{"label": "乌拉特中旗","value":"150824","disabled": false},
{"label": "乌拉特后旗","value":"150825","disabled": false},
{"label": "杭锦后旗","value":"150826","disabled": false},
{"label": "集宁区","value":"150902","disabled": false},
{"label": "卓资县","value":"150921","disabled": false},
{"label": "化德县","value":"150922","disabled": false},
{"label": "商都县","value":"150923","disabled": false},
{"label": "兴和县","value":"150924","disabled": false},
{"label": "凉城县","value":"150925","disabled": false},
{"label": "察哈尔右翼前旗","value":"150926","disabled": false},
{"label": "察哈尔右翼中旗","value":"150927","disabled": false},
{"label": "察哈尔右翼后旗","value":"150928","disabled": false},
{"label": "四子王旗","value":"150929","disabled": false},
{"label": "丰镇市","value":"150981","disabled": false},
{"label": "乌兰浩特市","value":"152201","disabled": false},
{"label": "阿尔山市","value":"152202","disabled": false},
{"label": "科尔沁右翼前旗","value":"152221","disabled": false},
{"label": "科尔沁右翼中旗","value":"152222","disabled": false},
{"label": "扎赉特旗","value":"152223","disabled": false},
{"label": "突泉县","value":"152224","disabled": false},
{"label": "二连浩特市","value":"152501","disabled": false},
{"label": "锡林浩特市","value":"152502","disabled": false},
{"label": "阿巴嘎旗","value":"152522","disabled": false},
{"label": "苏尼特左旗","value":"152523","disabled": false},
{"label": "苏尼特右旗","value":"152524","disabled": false},
{"label": "东乌珠穆沁旗","value":"152525","disabled": false},
{"label": "西乌珠穆沁旗","value":"152526","disabled": false},
{"label": "太仆寺旗","value":"152527","disabled": false},
{"label": "镶黄旗","value":"152528","disabled": false},
{"label": "正镶白旗","value":"152529","disabled": false},
{"label": "正蓝旗","value":"152530","disabled": false},
{"label": "多伦县","value":"152531","disabled": false},
{"label": "阿拉善左旗","value":"152921","disabled": false},
{"label": "阿拉善右旗","value":"152922","disabled": false},
{"label": "额济纳旗","value":"152923","disabled": false},
{"label": "和平区","value":"210102","disabled": false},
{"label": "沈河区","value":"210103","disabled": false},
{"label": "大东区","value":"210104","disabled": false},
{"label": "皇姑区","value":"210105","disabled": false},
{"label": "铁西区","value":"210106","disabled": false},
{"label": "苏家屯区","value":"210111","disabled": false},
{"label": "东陵区","value":"210112","disabled": false},
{"label": "沈北新区","value":"210113","disabled": false},
{"label": "于洪区","value":"210114","disabled": false},
{"label": "辽中县","value":"210122","disabled": false},
{"label": "康平县","value":"210123","disabled": false},
{"label": "法库县","value":"210124","disabled": false},
{"label": "新民市","value":"210181","disabled": false},
{"label": "中山区","value":"210202","disabled": false},
{"label": "西岗区","value":"210203","disabled": false},
{"label": "沙河口区","value":"210204","disabled": false},
{"label": "甘井子区","value":"210211","disabled": false},
{"label": "旅顺口区","value":"210212","disabled": false},
{"label": "金州区","value":"210213","disabled": false},
{"label": "长海县","value":"210224","disabled": false},
{"label": "瓦房店市","value":"210281","disabled": false},
{"label": "普兰店市","value":"210282","disabled": false},
{"label": "庄河市","value":"210283","disabled": false},
{"label": "铁东区","value":"210302","disabled": false},
{"label": "铁西区","value":"210303","disabled": false},
{"label": "立山区","value":"210304","disabled": false},
{"label": "千山区","value":"210311","disabled": false},
{"label": "台安县","value":"210321","disabled": false},
{"label": "岫岩满族自治县","value":"210323","disabled": false},
{"label": "海城市","value":"210381","disabled": false},
{"label": "新抚区","value":"210402","disabled": false},
{"label": "东洲区","value":"210403","disabled": false},
{"label": "望花区","value":"210404","disabled": false},
{"label": "顺城区","value":"210411","disabled": false},
{"label": "抚顺县","value":"210421","disabled": false},
{"label": "新宾满族自治县","value":"210422","disabled": false},
{"label": "清原满族自治县","value":"210423","disabled": false},
{"label": "平山区","value":"210502","disabled": false},
{"label": "溪湖区","value":"210503","disabled": false},
{"label": "明山区","value":"210504","disabled": false},
{"label": "南芬区","value":"210505","disabled": false},
{"label": "本溪满族自治县","value":"210521","disabled": false},
{"label": "桓仁满族自治县","value":"210522","disabled": false},
{"label": "元宝区","value":"210602","disabled": false},
{"label": "振兴区","value":"210603","disabled": false},
{"label": "振安区","value":"210604","disabled": false},
{"label": "宽甸满族自治县","value":"210624","disabled": false},
{"label": "东港市","value":"210681","disabled": false},
{"label": "凤城市","value":"210682","disabled": false},
{"label": "古塔区","value":"210702","disabled": false},
{"label": "凌河区","value":"210703","disabled": false},
{"label": "太和区","value":"210711","disabled": false},
{"label": "黑山县","value":"210726","disabled": false},
{"label": "义县","value":"210727","disabled": false},
{"label": "凌海市","value":"210781","disabled": false},
{"label": "北镇市","value":"210782","disabled": false},
{"label": "站前区","value":"210802","disabled": false},
{"label": "西市区","value":"210803","disabled": false},
{"label": "鲅鱼圈区","value":"210804","disabled": false},
{"label": "老边区","value":"210811","disabled": false},
{"label": "盖州市","value":"210881","disabled": false},
{"label": "大石桥市","value":"210882","disabled": false},
{"label": "海州区","value":"210902","disabled": false},
{"label": "新邱区","value":"210903","disabled": false},
{"label": "太平区","value":"210904","disabled": false},
{"label": "清河门区","value":"210905","disabled": false},
{"label": "细河区","value":"210911","disabled": false},
{"label": "阜新蒙古族自治县","value":"210921","disabled": false},
{"label": "彰武县","value":"210922","disabled": false},
{"label": "白塔区","value":"211002","disabled": false},
{"label": "文圣区","value":"211003","disabled": false},
{"label": "宏伟区","value":"211004","disabled": false},
{"label": "弓长岭区","value":"211005","disabled": false},
{"label": "太子河区","value":"211011","disabled": false},
{"label": "辽阳县","value":"211021","disabled": false},
{"label": "灯塔市","value":"211081","disabled": false},
{"label": "双台子区","value":"211102","disabled": false},
{"label": "兴隆台区","value":"211103","disabled": false},
{"label": "大洼县","value":"211121","disabled": false},
{"label": "盘山县","value":"211122","disabled": false},
{"label": "银州区","value":"211202","disabled": false},
{"label": "清河区","value":"211204","disabled": false},
{"label": "铁岭县","value":"211221","disabled": false},
{"label": "西丰县","value":"211223","disabled": false},
{"label": "昌图县","value":"211224","disabled": false},
{"label": "调兵山市","value":"211281","disabled": false},
{"label": "开原市","value":"211282","disabled": false},
{"label": "双塔区","value":"211302","disabled": false},
{"label": "龙城区","value":"211303","disabled": false},
{"label": "朝阳县","value":"211321","disabled": false},
{"label": "建平县","value":"211322","disabled": false},
{"label": "喀喇沁左翼蒙古族自治县","value":"211324","disabled": false},
{"label": "北票市","value":"211381","disabled": false},
{"label": "凌源市","value":"211382","disabled": false},
{"label": "连山区","value":"211402","disabled": false},
{"label": "龙港区","value":"211403","disabled": false},
{"label": "南票区","value":"211404","disabled": false},
{"label": "绥中县","value":"211421","disabled": false},
{"label": "建昌县","value":"211422","disabled": false},
{"label": "兴城市","value":"211481","disabled": false},
{"label": "南关区","value":"220102","disabled": false},
{"label": "宽城区","value":"220103","disabled": false},
{"label": "朝阳区","value":"220104","disabled": false},
{"label": "二道区","value":"220105","disabled": false},
{"label": "绿园区","value":"220106","disabled": false},
{"label": "双阳区","value":"220112","disabled": false},
{"label": "农安县","value":"220122","disabled": false},
{"label": "九台市","value":"220181","disabled": false},
{"label": "榆树市","value":"220182","disabled": false},
{"label": "德惠市","value":"220183","disabled": false},
{"label": "昌邑区","value":"220202","disabled": false},
{"label": "龙潭区","value":"220203","disabled": false},
{"label": "船营区","value":"220204","disabled": false},
{"label": "丰满区","value":"220211","disabled": false},
{"label": "永吉县","value":"220221","disabled": false},
{"label": "蛟河市","value":"220281","disabled": false},
{"label": "桦甸市","value":"220282","disabled": false},
{"label": "舒兰市","value":"220283","disabled": false},
{"label": "磐石市","value":"220284","disabled": false},
{"label": "铁西区","value":"220302","disabled": false},
{"label": "铁东区","value":"220303","disabled": false},
{"label": "梨树县","value":"220322","disabled": false},
{"label": "伊通满族自治县","value":"220323","disabled": false},
{"label": "公主岭市","value":"220381","disabled": false},
{"label": "双辽市","value":"220382","disabled": false},
{"label": "龙山区","value":"220402","disabled": false},
{"label": "西安区","value":"220403","disabled": false},
{"label": "东丰县","value":"220421","disabled": false},
{"label": "东辽县","value":"220422","disabled": false},
{"label": "东昌区","value":"220502","disabled": false},
{"label": "二道江区","value":"220503","disabled": false},
{"label": "通化县","value":"220521","disabled": false},
{"label": "辉南县","value":"220523","disabled": false},
{"label": "柳河县","value":"220524","disabled": false},
{"label": "梅河口市","value":"220581","disabled": false},
{"label": "集安市","value":"220582","disabled": false},
{"label": "八道江区","value":"220602","disabled": false},
{"label": "江源区","value":"220605","disabled": false},
{"label": "抚松县","value":"220621","disabled": false},
{"label": "靖宇县","value":"220622","disabled": false},
{"label": "长白朝鲜族自治县","value":"220623","disabled": false},
{"label": "临江市","value":"220681","disabled": false},
{"label": "宁江区","value":"220702","disabled": false},
{"label": "前郭尔罗斯蒙古族自治县","value":"220721","disabled": false},
{"label": "长岭县","value":"220722","disabled": false},
{"label": "乾安县","value":"220723","disabled": false},
{"label": "扶余县","value":"220724","disabled": false},
{"label": "洮北区","value":"220802","disabled": false},
{"label": "镇赉县","value":"220821","disabled": false},
{"label": "通榆县","value":"220822","disabled": false},
{"label": "洮南市","value":"220881","disabled": false},
{"label": "大安市","value":"220882","disabled": false},
{"label": "延吉市","value":"222401","disabled": false},
{"label": "图们市","value":"222402","disabled": false},
{"label": "敦化市","value":"222403","disabled": false},
{"label": "珲春市","value":"222404","disabled": false},
{"label": "龙井市","value":"222405","disabled": false},
{"label": "和龙市","value":"222406","disabled": false},
{"label": "汪清县","value":"222424","disabled": false},
{"label": "安图县","value":"222426","disabled": false},
{"label": "道里区","value":"230102","disabled": false},
{"label": "南岗区","value":"230103","disabled": false},
{"label": "道外区","value":"230104","disabled": false},
{"label": "平房区","value":"230108","disabled": false},
{"label": "松北区","value":"230109","disabled": false},
{"label": "香坊区","value":"230110","disabled": false},
{"label": "呼兰区","value":"230111","disabled": false},
{"label": "阿城区","value":"230112","disabled": false},
{"label": "依兰县","value":"230123","disabled": false},
{"label": "方正县","value":"230124","disabled": false},
{"label": "宾县","value":"230125","disabled": false},
{"label": "巴彦县","value":"230126","disabled": false},
{"label": "木兰县","value":"230127","disabled": false},
{"label": "通河县","value":"230128","disabled": false},
{"label": "延寿县","value":"230129","disabled": false},
{"label": "双城市","value":"230182","disabled": false},
{"label": "尚志市","value":"230183","disabled": false},
{"label": "五常市","value":"230184","disabled": false},
{"label": "龙沙区","value":"230202","disabled": false},
{"label": "建华区","value":"230203","disabled": false},
{"label": "铁锋区","value":"230204","disabled": false},
{"label": "昂昂溪区","value":"230205","disabled": false},
{"label": "富拉尔基区","value":"230206","disabled": false},
{"label": "碾子山区","value":"230207","disabled": false},
{"label": "梅里斯达斡尔族区","value":"230208","disabled": false},
{"label": "龙江县","value":"230221","disabled": false},
{"label": "依安县","value":"230223","disabled": false},
{"label": "泰来县","value":"230224","disabled": false},
{"label": "甘南县","value":"230225","disabled": false},
{"label": "富裕县","value":"230227","disabled": false},
{"label": "克山县","value":"230229","disabled": false},
{"label": "克东县","value":"230230","disabled": false},
{"label": "拜泉县","value":"230231","disabled": false},
{"label": "讷河市","value":"230281","disabled": false},
{"label": "鸡冠区","value":"230302","disabled": false},
{"label": "恒山区","value":"230303","disabled": false},
{"label": "滴道区","value":"230304","disabled": false},
{"label": "梨树区","value":"230305","disabled": false},
{"label": "城子河区","value":"230306","disabled": false},
{"label": "麻山区","value":"230307","disabled": false},
{"label": "鸡东县","value":"230321","disabled": false},
{"label": "虎林市","value":"230381","disabled": false},
{"label": "密山市","value":"230382","disabled": false},
{"label": "向阳区","value":"230402","disabled": false},
{"label": "工农区","value":"230403","disabled": false},
{"label": "南山区","value":"230404","disabled": false},
{"label": "兴安区","value":"230405","disabled": false},
{"label": "东山区","value":"230406","disabled": false},
{"label": "兴山区","value":"230407","disabled": false},
{"label": "萝北县","value":"230421","disabled": false},
{"label": "绥滨县","value":"230422","disabled": false},
{"label": "尖山区","value":"230502","disabled": false},
{"label": "岭东区","value":"230503","disabled": false},
{"label": "四方台区","value":"230505","disabled": false},
{"label": "宝山区","value":"230506","disabled": false},
{"label": "集贤县","value":"230521","disabled": false},
{"label": "友谊县","value":"230522","disabled": false},
{"label": "宝清县","value":"230523","disabled": false},
{"label": "饶河县","value":"230524","disabled": false},
{"label": "萨尔图区","value":"230602","disabled": false},
{"label": "龙凤区","value":"230603","disabled": false},
{"label": "让胡路区","value":"230604","disabled": false},
{"label": "红岗区","value":"230605","disabled": false},
{"label": "大同区","value":"230606","disabled": false},
{"label": "肇州县","value":"230621","disabled": false},
{"label": "肇源县","value":"230622","disabled": false},
{"label": "林甸县","value":"230623","disabled": false},
{"label": "杜尔伯特蒙古族自治县","value":"230624","disabled": false},
{"label": "伊春区","value":"230702","disabled": false},
{"label": "南岔区","value":"230703","disabled": false},
{"label": "友好区","value":"230704","disabled": false},
{"label": "西林区","value":"230705","disabled": false},
{"label": "翠峦区","value":"230706","disabled": false},
{"label": "新青区","value":"230707","disabled": false},
{"label": "美溪区","value":"230708","disabled": false},
{"label": "金山屯区","value":"230709","disabled": false},
{"label": "五营区","value":"230710","disabled": false},
{"label": "乌马河区","value":"230711","disabled": false},
{"label": "汤旺河区","value":"230712","disabled": false},
{"label": "带岭区","value":"230713","disabled": false},
{"label": "乌伊岭区","value":"230714","disabled": false},
{"label": "红星区","value":"230715","disabled": false},
{"label": "上甘岭区","value":"230716","disabled": false},
{"label": "嘉荫县","value":"230722","disabled": false},
{"label": "铁力市","value":"230781","disabled": false},
{"label": "向阳区","value":"230803","disabled": false},
{"label": "前进区","value":"230804","disabled": false},
{"label": "东风区","value":"230805","disabled": false},
{"label": "郊区","value":"230811","disabled": false},
{"label": "桦南县","value":"230822","disabled": false},
{"label": "桦川县","value":"230826","disabled": false},
{"label": "汤原县","value":"230828","disabled": false},
{"label": "抚远县","value":"230833","disabled": false},
{"label": "同江市","value":"230881","disabled": false},
{"label": "富锦市","value":"230882","disabled": false},
{"label": "新兴区","value":"230902","disabled": false},
{"label": "桃山区","value":"230903","disabled": false},
{"label": "茄子河区","value":"230904","disabled": false},
{"label": "勃利县","value":"230921","disabled": false},
{"label": "东安区","value":"231002","disabled": false},
{"label": "阳明区","value":"231003","disabled": false},
{"label": "爱民区","value":"231004","disabled": false},
{"label": "西安区","value":"231005","disabled": false},
{"label": "东宁县","value":"231024","disabled": false},
{"label": "林口县","value":"231025","disabled": false},
{"label": "绥芬河市","value":"231081","disabled": false},
{"label": "海林市","value":"231083","disabled": false},
{"label": "宁安市","value":"231084","disabled": false},
{"label": "穆棱市","value":"231085","disabled": false},
{"label": "爱辉区","value":"231102","disabled": false},
{"label": "嫩江县","value":"231121","disabled": false},
{"label": "逊克县","value":"231123","disabled": false},
{"label": "孙吴县","value":"231124","disabled": false},
{"label": "北安市","value":"231181","disabled": false},
{"label": "五大连池市","value":"231182","disabled": false},
{"label": "北林区","value":"231202","disabled": false},
{"label": "望奎县","value":"231221","disabled": false},
{"label": "兰西县","value":"231222","disabled": false},
{"label": "青冈县","value":"231223","disabled": false},
{"label": "庆安县","value":"231224","disabled": false},
{"label": "明水县","value":"231225","disabled": false},
{"label": "绥棱县","value":"231226","disabled": false},
{"label": "安达市","value":"231281","disabled": false},
{"label": "肇东市","value":"231282","disabled": false},
{"label": "海伦市","value":"231283","disabled": false},
{"label": "加格达奇区","value":"232701","disabled": false},
{"label": "松岭区","value":"232702","disabled": false},
{"label": "新林区","value":"232703","disabled": false},
{"label": "呼中区","value":"232704","disabled": false},
{"label": "呼玛县","value":"232721","disabled": false},
{"label": "塔河县","value":"232722","disabled": false},
{"label": "漠河县","value":"232723","disabled": false},
{"label": "玄武区","value":"320102","disabled": false},
{"label": "白下区","value":"320103","disabled": false},
{"label": "秦淮区","value":"320104","disabled": false},
{"label": "建邺区","value":"320105","disabled": false},
{"label": "鼓楼区","value":"320106","disabled": false},
{"label": "下关区","value":"320107","disabled": false},
{"label": "浦口区","value":"320111","disabled": false},
{"label": "栖霞区","value":"320113","disabled": false},
{"label": "雨花台区","value":"320114","disabled": false},
{"label": "江宁区","value":"320115","disabled": false},
{"label": "六合区","value":"320116","disabled": false},
{"label": "溧水县","value":"320124","disabled": false},
{"label": "高淳县","value":"320125","disabled": false},
{"label": "崇安区","value":"320202","disabled": false},
{"label": "南长区","value":"320203","disabled": false},
{"label": "北塘区","value":"320204","disabled": false},
{"label": "锡山区","value":"320205","disabled": false},
{"label": "惠山区","value":"320206","disabled": false},
{"label": "滨湖区","value":"320211","disabled": false},
{"label": "江阴市","value":"320281","disabled": false},
{"label": "宜兴市","value":"320282","disabled": false},
{"label": "鼓楼区","value":"320302","disabled": false},
{"label": "云龙区","value":"320303","disabled": false},
{"label": "九里区","value":"320304","disabled": false},
{"label": "贾汪区","value":"320305","disabled": false},
{"label": "泉山区","value":"320311","disabled": false},
{"label": "丰县","value":"320321","disabled": false},
{"label": "沛县","value":"320322","disabled": false},
{"label": "铜山县","value":"320323","disabled": false},
{"label": "睢宁县","value":"320324","disabled": false},
{"label": "新沂市","value":"320381","disabled": false},
{"label": "邳州市","value":"320382","disabled": false},
{"label": "天宁区","value":"320402","disabled": false},
{"label": "钟楼区","value":"320404","disabled": false},
{"label": "戚墅堰区","value":"320405","disabled": false},
{"label": "新北区","value":"320411","disabled": false},
{"label": "武进区","value":"320412","disabled": false},
{"label": "溧阳市","value":"320481","disabled": false},
{"label": "金坛市","value":"320482","disabled": false},
{"label": "沧浪区","value":"320502","disabled": false},
{"label": "平江区","value":"320503","disabled": false},
{"label": "金阊区","value":"320504","disabled": false},
{"label": "虎丘区","value":"320505","disabled": false},
{"label": "吴中区","value":"320506","disabled": false},
{"label": "相城区","value":"320507","disabled": false},
{"label": "姑苏区","value":"320508 ","disabled": false},
{"label": "常熟市","value":"320581","disabled": false},
{"label": "张家港市","value":"320582","disabled": false},
{"label": "昆山市","value":"320583","disabled": false},
{"label": "吴江市","value":"320584","disabled": false},
{"label": "太仓市","value":"320585","disabled": false},
{"label": "崇川区","value":"320602","disabled": false},
{"label": "港闸区","value":"320611","disabled": false},
{"label": "海安县","value":"320621","disabled": false},
{"label": "如东县","value":"320623","disabled": false},
{"label": "启东市","value":"320681","disabled": false},
{"label": "如皋市","value":"320682","disabled": false},
{"label": "通州市","value":"320683","disabled": false},
{"label": "海门市","value":"320684","disabled": false},
{"label": "连云区","value":"320703","disabled": false},
{"label": "新浦区","value":"320705","disabled": false},
{"label": "海州区","value":"320706","disabled": false},
{"label": "赣榆县","value":"320721","disabled": false},
{"label": "东海县","value":"320722","disabled": false},
{"label": "灌云县","value":"320723","disabled": false},
{"label": "灌南县","value":"320724","disabled": false},
{"label": "清河区","value":"320802","disabled": false},
{"label": "楚州区","value":"320803","disabled": false},
{"label": "淮阴区","value":"320804","disabled": false},
{"label": "清浦区","value":"320811","disabled": false},
{"label": "涟水县","value":"320826","disabled": false},
{"label": "洪泽县","value":"320829","disabled": false},
{"label": "盱眙县","value":"320830","disabled": false},
{"label": "金湖县","value":"320831","disabled": false},
{"label": "亭湖区","value":"320902","disabled": false},
{"label": "盐都区","value":"320903","disabled": false},
{"label": "响水县","value":"320921","disabled": false},
{"label": "滨海县","value":"320922","disabled": false},
{"label": "阜宁县","value":"320923","disabled": false},
{"label": "射阳县","value":"320924","disabled": false},
{"label": "建湖县","value":"320925","disabled": false},
{"label": "东台市","value":"320981","disabled": false},
{"label": "大丰市","value":"320982","disabled": false},
{"label": "广陵区","value":"321002","disabled": false},
{"label": "邗江区","value":"321003","disabled": false},
{"label": "维扬区","value":"321011","disabled": false},
{"label": "宝应县","value":"321023","disabled": false},
{"label": "仪征市","value":"321081","disabled": false},
{"label": "高邮市","value":"321084","disabled": false},
{"label": "江都市","value":"321088","disabled": false},
{"label": "京口区","value":"321102","disabled": false},
{"label": "润州区","value":"321111","disabled": false},
{"label": "丹徒区","value":"321112","disabled": false},
{"label": "丹阳市","value":"321181","disabled": false},
{"label": "扬中市","value":"321182","disabled": false},
{"label": "句容市","value":"321183","disabled": false},
{"label": "海陵区","value":"321202","disabled": false},
{"label": "高港区","value":"321203","disabled": false},
{"label": "兴化市","value":"321281","disabled": false},
{"label": "靖江市","value":"321282","disabled": false},
{"label": "泰兴市","value":"321283","disabled": false},
{"label": "姜堰市","value":"321284","disabled": false},
{"label": "宿城区","value":"321302","disabled": false},
{"label": "宿豫区","value":"321311","disabled": false},
{"label": "沭阳县","value":"321322","disabled": false},
{"label": "泗阳县","value":"321323","disabled": false},
{"label": "泗洪县","value":"321324","disabled": false},
{"label": "上城区","value":"330102","disabled": false},
{"label": "下城区","value":"330103","disabled": false},
{"label": "江干区","value":"330104","disabled": false},
{"label": "拱墅区","value":"330105","disabled": false},
{"label": "西湖区","value":"330106","disabled": false},
{"label": "滨江区","value":"330108","disabled": false},
{"label": "萧山区","value":"330109","disabled": false},
{"label": "余杭区","value":"330110","disabled": false},
{"label": "桐庐县","value":"330122","disabled": false},
{"label": "淳安县","value":"330127","disabled": false},
{"label": "建德市","value":"330182","disabled": false},
{"label": "富阳市","value":"330183","disabled": false},
{"label": "临安市","value":"330185","disabled": false},
{"label": "海曙区","value":"330203","disabled": false},
{"label": "江东区","value":"330204","disabled": false},
{"label": "江北区","value":"330205","disabled": false},
{"label": "北仑区","value":"330206","disabled": false},
{"label": "镇海区","value":"330211","disabled": false},
{"label": "鄞州区","value":"330212","disabled": false},
{"label": "象山县","value":"330225","disabled": false},
{"label": "宁海县","value":"330226","disabled": false},
{"label": "余姚市","value":"330281","disabled": false},
{"label": "慈溪市","value":"330282","disabled": false},
{"label": "奉化市","value":"330283","disabled": false},
{"label": "鹿城区","value":"330302","disabled": false},
{"label": "龙湾区","value":"330303","disabled": false},
{"label": "瓯海区","value":"330304","disabled": false},
{"label": "洞头县","value":"330322","disabled": false},
{"label": "永嘉县","value":"330324","disabled": false},
{"label": "平阳县","value":"330326","disabled": false},
{"label": "苍南县","value":"330327","disabled": false},
{"label": "文成县","value":"330328","disabled": false},
{"label": "泰顺县","value":"330329","disabled": false},
{"label": "瑞安市","value":"330381","disabled": false},
{"label": "乐清市","value":"330382","disabled": false},
{"label": "南湖区","value":"330402","disabled": false},
{"label": "秀洲区","value":"330411","disabled": false},
{"label": "嘉善县","value":"330421","disabled": false},
{"label": "海盐县","value":"330424","disabled": false},
{"label": "海宁市","value":"330481","disabled": false},
{"label": "平湖市","value":"330482","disabled": false},
{"label": "桐乡市","value":"330483","disabled": false},
{"label": "吴兴区","value":"330502","disabled": false},
{"label": "南浔区","value":"330503","disabled": false},
{"label": "德清县","value":"330521","disabled": false},
{"label": "长兴县","value":"330522","disabled": false},
{"label": "安吉县","value":"330523","disabled": false},
{"label": "越城区","value":"330602","disabled": false},
{"label": "绍兴县","value":"330621","disabled": false},
{"label": "新昌县","value":"330624","disabled": false},
{"label": "诸暨市","value":"330681","disabled": false},
{"label": "上虞市","value":"330682","disabled": false},
{"label": "嵊州市","value":"330683","disabled": false},
{"label": "婺城区","value":"330702","disabled": false},
{"label": "金东区","value":"330703","disabled": false},
{"label": "武义县","value":"330723","disabled": false},
{"label": "浦江县","value":"330726","disabled": false},
{"label": "磐安县","value":"330727","disabled": false},
{"label": "兰溪市","value":"330781","disabled": false},
{"label": "义乌市","value":"330782","disabled": false},
{"label": "东阳市","value":"330783","disabled": false},
{"label": "永康市","value":"330784","disabled": false},
{"label": "柯城区","value":"330802","disabled": false},
{"label": "衢江区","value":"330803","disabled": false},
{"label": "常山县","value":"330822","disabled": false},
{"label": "开化县","value":"330824","disabled": false},
{"label": "龙游县","value":"330825","disabled": false},
{"label": "江山市","value":"330881","disabled": false},
{"label": "定海区","value":"330902","disabled": false},
{"label": "普陀区","value":"330903","disabled": false},
{"label": "岱山县","value":"330921","disabled": false},
{"label": "嵊泗县","value":"330922","disabled": false},
{"label": "椒江区","value":"331002","disabled": false},
{"label": "黄岩区","value":"331003","disabled": false},
{"label": "路桥区","value":"331004","disabled": false},
{"label": "玉环县","value":"331021","disabled": false},
{"label": "三门县","value":"331022","disabled": false},
{"label": "天台县","value":"331023","disabled": false},
{"label": "仙居县","value":"331024","disabled": false},
{"label": "温岭市","value":"331081","disabled": false},
{"label": "临海市","value":"331082","disabled": false},
{"label": "莲都区","value":"331102","disabled": false},
{"label": "青田县","value":"331121","disabled": false},
{"label": "缙云县","value":"331122","disabled": false},
{"label": "遂昌县","value":"331123","disabled": false},
{"label": "松阳县","value":"331124","disabled": false},
{"label": "云和县","value":"331125","disabled": false},
{"label": "庆元县","value":"331126","disabled": false},
{"label": "景宁畲族自治县","value":"331127","disabled": false},
{"label": "龙泉市","value":"331181","disabled": false},
{"label": "瑶海区","value":"340102","disabled": false},
{"label": "庐阳区","value":"340103","disabled": false},
{"label": "蜀山区","value":"340104","disabled": false},
{"label": "包河区","value":"340111","disabled": false},
{"label": "长丰县","value":"340121","disabled": false},
{"label": "肥东县","value":"340122","disabled": false},
{"label": "肥西县","value":"340123","disabled": false},
{"label": "镜湖区","value":"340202","disabled": false},
{"label": "弋江区","value":"340203","disabled": false},
{"label": "鸠江区","value":"340207","disabled": false},
{"label": "三山区","value":"340208","disabled": false},
{"label": "芜湖县","value":"340221","disabled": false},
{"label": "繁昌县","value":"340222","disabled": false},
{"label": "南陵县","value":"340223","disabled": false},
{"label": "龙子湖区","value":"340302","disabled": false},
{"label": "蚌山区","value":"340303","disabled": false},
{"label": "禹会区","value":"340304","disabled": false},
{"label": "淮上区","value":"340311","disabled": false},
{"label": "怀远县","value":"340321","disabled": false},
{"label": "五河县","value":"340322","disabled": false},
{"label": "固镇县","value":"340323","disabled": false},
{"label": "大通区","value":"340402","disabled": false},
{"label": "田家庵区","value":"340403","disabled": false},
{"label": "谢家集区","value":"340404","disabled": false},
{"label": "八公山区","value":"340405","disabled": false},
{"label": "潘集区","value":"340406","disabled": false},
{"label": "凤台县","value":"340421","disabled": false},
{"label": "金家庄区","value":"340502","disabled": false},
{"label": "花山区","value":"340503","disabled": false},
{"label": "雨山区","value":"340504","disabled": false},
{"label": "当涂县","value":"340521","disabled": false},
{"label": "含山县","value":"340522 ","disabled": false},
{"label": "杜集区","value":"340602","disabled": false},
{"label": "相山区","value":"340603","disabled": false},
{"label": "烈山区","value":"340604","disabled": false},
{"label": "濉溪县","value":"340621","disabled": false},
{"label": "铜官山区","value":"340702","disabled": false},
{"label": "狮子山区","value":"340703","disabled": false},
{"label": "郊区","value":"340711","disabled": false},
{"label": "铜陵县","value":"340721","disabled": false},
{"label": "铜宫区","value":"340722","disabled": false},
{"label": "枞阳县","value":"340723","disabled": false},
{"label": "铜陵义安区","value":"340724","disabled": false},
{"label": "迎江区","value":"340802","disabled": false},
{"label": "大观区","value":"340803","disabled": false},
{"label": "宜秀区","value":"340811","disabled": false},
{"label": "怀宁县","value":"340822","disabled": false},
{"label": "枞阳县","value":"340823","disabled": false},
{"label": "潜山县","value":"340824","disabled": false},
{"label": "太湖县","value":"340825","disabled": false},
{"label": "宿松县","value":"340826","disabled": false},
{"label": "望江县","value":"340827","disabled": false},
{"label": "岳西县","value":"340828","disabled": false},
{"label": "桐城市","value":"340881","disabled": false},
{"label": "屯溪区","value":"341002","disabled": false},
{"label": "黄山区","value":"341003","disabled": false},
{"label": "徽州区","value":"341004","disabled": false},
{"label": "歙县","value":"341021","disabled": false},
{"label": "休宁县","value":"341022","disabled": false},
{"label": "黟县","value":"341023","disabled": false},
{"label": "祁门县","value":"341024","disabled": false},
{"label": "琅琊区","value":"341102","disabled": false},
{"label": "南谯区","value":"341103","disabled": false},
{"label": "来安县","value":"341122","disabled": false},
{"label": "全椒县","value":"341124","disabled": false},
{"label": "定远县","value":"341125","disabled": false},
{"label": "凤阳县","value":"341126","disabled": false},
{"label": "天长市","value":"341181","disabled": false},
{"label": "明光市","value":"341182","disabled": false},
{"label": "颍州区","value":"341202","disabled": false},
{"label": "颍东区","value":"341203","disabled": false},
{"label": "颍泉区","value":"341204","disabled": false},
{"label": "临泉县","value":"341221","disabled": false},
{"label": "太和县","value":"341222","disabled": false},
{"label": "阜南县","value":"341225","disabled": false},
{"label": "颍上县","value":"341226","disabled": false},
{"label": "界首市","value":"341282","disabled": false},
{"label": "埇桥区","value":"341302","disabled": false},
{"label": "砀山县","value":"341321","disabled": false},
{"label": "萧县","value":"341322","disabled": false},
{"label": "灵璧县","value":"341323","disabled": false},
{"label": "泗县","value":"341324","disabled": false},
{"label": "居巢区","value":"341402","disabled": false},
{"label": "庐江县","value":"341421","disabled": false},
{"label": "无为县","value":"341422","disabled": false},
{"label": "含山县","value":"341423","disabled": false},
{"label": "和县","value":"341424","disabled": false},
{"label": "金安区","value":"341502","disabled": false},
{"label": "裕安区","value":"341503","disabled": false},
{"label": "寿县","value":"341521","disabled": false},
{"label": "霍邱县","value":"341522","disabled": false},
{"label": "舒城县","value":"341523","disabled": false},
{"label": "金寨县","value":"341524","disabled": false},
{"label": "霍山县","value":"341525","disabled": false},
{"label": "谯城区","value":"341602","disabled": false},
{"label": "涡阳县","value":"341621","disabled": false},
{"label": "蒙城县","value":"341622","disabled": false},
{"label": "利辛县","value":"341623","disabled": false},
{"label": "贵池区","value":"341702","disabled": false},
{"label": "东至县","value":"341721","disabled": false},
{"label": "石台县","value":"341722","disabled": false},
{"label": "青阳县","value":"341723","disabled": false},
{"label": "宣州区","value":"341802","disabled": false},
{"label": "郎溪县","value":"341821","disabled": false},
{"label": "广德县","value":"341822","disabled": false},
{"label": "泾县","value":"341823","disabled": false},
{"label": "绩溪县","value":"341824","disabled": false},
{"label": "旌德县","value":"341825","disabled": false},
{"label": "宁国市","value":"341881","disabled": false},
{"label": "鼓楼区","value":"350102","disabled": false},
{"label": "台江区","value":"350103","disabled": false},
{"label": "仓山区","value":"350104","disabled": false},
{"label": "马尾区","value":"350105","disabled": false},
{"label": "晋安区","value":"350111","disabled": false},
{"label": "闽侯县","value":"350121","disabled": false},
{"label": "连江县","value":"350122","disabled": false},
{"label": "罗源县","value":"350123","disabled": false},
{"label": "闽清县","value":"350124","disabled": false},
{"label": "永泰县","value":"350125","disabled": false},
{"label": "平潭县","value":"350128","disabled": false},
{"label": "福清市","value":"350181","disabled": false},
{"label": "长乐市","value":"350182","disabled": false},
{"label": "思明区","value":"350203","disabled": false},
{"label": "海沧区","value":"350205","disabled": false},
{"label": "湖里区","value":"350206","disabled": false},
{"label": "集美区","value":"350211","disabled": false},
{"label": "同安区","value":"350212","disabled": false},
{"label": "翔安区","value":"350213","disabled": false},
{"label": "城厢区","value":"350302","disabled": false},
{"label": "涵江区","value":"350303","disabled": false},
{"label": "荔城区","value":"350304","disabled": false},
{"label": "秀屿区","value":"350305","disabled": false},
{"label": "仙游县","value":"350322","disabled": false},
{"label": "梅列区","value":"350402","disabled": false},
{"label": "三元区","value":"350403","disabled": false},
{"label": "明溪县","value":"350421","disabled": false},
{"label": "清流县","value":"350423","disabled": false},
{"label": "宁化县","value":"350424","disabled": false},
{"label": "大田县","value":"350425","disabled": false},
{"label": "尤溪县","value":"350426","disabled": false},
{"label": "沙县","value":"350427","disabled": false},
{"label": "将乐县","value":"350428","disabled": false},
{"label": "泰宁县","value":"350429","disabled": false},
{"label": "建宁县","value":"350430","disabled": false},
{"label": "永安市","value":"350481","disabled": false},
{"label": "鲤城区","value":"350502","disabled": false},
{"label": "丰泽区","value":"350503","disabled": false},
{"label": "洛江区","value":"350504","disabled": false},
{"label": "泉港区","value":"350505","disabled": false},
{"label": "惠安县","value":"350521","disabled": false},
{"label": "安溪县","value":"350524","disabled": false},
{"label": "永春县","value":"350525","disabled": false},
{"label": "德化县","value":"350526","disabled": false},
{"label": "金门县","value":"350527","disabled": false},
{"label": "石狮市","value":"350581","disabled": false},
{"label": "晋江市","value":"350582","disabled": false},
{"label": "南安市","value":"350583","disabled": false},
{"label": "芗城区","value":"350602","disabled": false},
{"label": "龙文区","value":"350603","disabled": false},
{"label": "云霄县","value":"350622","disabled": false},
{"label": "漳浦县","value":"350623","disabled": false},
{"label": "诏安县","value":"350624","disabled": false},
{"label": "长泰县","value":"350625","disabled": false},
{"label": "东山县","value":"350626","disabled": false},
{"label": "南靖县","value":"350627","disabled": false},
{"label": "平和县","value":"350628","disabled": false},
{"label": "华安县","value":"350629","disabled": false},
{"label": "龙海市","value":"350681","disabled": false},
{"label": "延平区","value":"350702","disabled": false},
{"label": "顺昌县","value":"350721","disabled": false},
{"label": "浦城县","value":"350722","disabled": false},
{"label": "光泽县","value":"350723","disabled": false},
{"label": "松溪县","value":"350724","disabled": false},
{"label": "政和县","value":"350725","disabled": false},
{"label": "邵武市","value":"350781","disabled": false},
{"label": "武夷山市","value":"350782","disabled": false},
{"label": "建瓯市","value":"350783","disabled": false},
{"label": "建阳市","value":"350784","disabled": false},
{"label": "新罗区","value":"350802","disabled": false},
{"label": "长汀县","value":"350821","disabled": false},
{"label": "永定县","value":"350822","disabled": false},
{"label": "上杭县","value":"350823","disabled": false},
{"label": "武平县","value":"350824","disabled": false},
{"label": "连城县","value":"350825","disabled": false},
{"label": "漳平市","value":"350881","disabled": false},
{"label": "蕉城区","value":"350902","disabled": false},
{"label": "霞浦县","value":"350921","disabled": false},
{"label": "古田县","value":"350922","disabled": false},
{"label": "屏南县","value":"350923","disabled": false},
{"label": "寿宁县","value":"350924","disabled": false},
{"label": "周宁县","value":"350925","disabled": false},
{"label": "柘荣县","value":"350926","disabled": false},
{"label": "福安市","value":"350981","disabled": false},
{"label": "福鼎市","value":"350982","disabled": false},
{"label": "东湖区","value":"360102","disabled": false},
{"label": "西湖区","value":"360103","disabled": false},
{"label": "青云谱区","value":"360104","disabled": false},
{"label": "湾里区","value":"360105","disabled": false},
{"label": "青山湖区","value":"360111","disabled": false},
{"label": "南昌县","value":"360121","disabled": false},
{"label": "新建县","value":"360122","disabled": false},
{"label": "安义县","value":"360123","disabled": false},
{"label": "进贤县","value":"360124","disabled": false},
{"label": "昌江区","value":"360202","disabled": false},
{"label": "珠山区","value":"360203","disabled": false},
{"label": "浮梁县","value":"360222","disabled": false},
{"label": "乐平市","value":"360281","disabled": false},
{"label": "安源区","value":"360302","disabled": false},
{"label": "湘东区","value":"360313","disabled": false},
{"label": "莲花县","value":"360321","disabled": false},
{"label": "上栗县","value":"360322","disabled": false},
{"label": "芦溪县","value":"360323","disabled": false},
{"label": "庐山区","value":"360402","disabled": false},
{"label": "浔阳区","value":"360403","disabled": false},
{"label": "九江县","value":"360421","disabled": false},
{"label": "武宁县","value":"360423","disabled": false},
{"label": "修水县","value":"360424","disabled": false},
{"label": "永修县","value":"360425","disabled": false},
{"label": "德安县","value":"360426","disabled": false},
{"label": "星子县","value":"360427","disabled": false},
{"label": "都昌县","value":"360428","disabled": false},
{"label": "湖口县","value":"360429","disabled": false},
{"label": "彭泽县","value":"360430","disabled": false},
{"label": "瑞昌市","value":"360481","disabled": false},
{"label": "渝水区","value":"360502","disabled": false},
{"label": "分宜县","value":"360521","disabled": false},
{"label": "月湖区","value":"360602","disabled": false},
{"label": "余江县","value":"360622","disabled": false},
{"label": "贵溪市","value":"360681","disabled": false},
{"label": "章贡区","value":"360702","disabled": false},
{"label": "赣县","value":"360721","disabled": false},
{"label": "信丰县","value":"360722","disabled": false},
{"label": "大余县","value":"360723","disabled": false},
{"label": "上犹县","value":"360724","disabled": false},
{"label": "崇义县","value":"360725","disabled": false},
{"label": "安远县","value":"360726","disabled": false},
{"label": "龙南县","value":"360727","disabled": false},
{"label": "定南县","value":"360728","disabled": false},
{"label": "全南县","value":"360729","disabled": false},
{"label": "宁都县","value":"360730","disabled": false},
{"label": "于都县","value":"360731","disabled": false},
{"label": "兴国县","value":"360732","disabled": false},
{"label": "会昌县","value":"360733","disabled": false},
{"label": "寻乌县","value":"360734","disabled": false},
{"label": "石城县","value":"360735","disabled": false},
{"label": "瑞金市","value":"360781","disabled": false},
{"label": "南康市","value":"360782","disabled": false},
{"label": "吉州区","value":"360802","disabled": false},
{"label": "青原区","value":"360803","disabled": false},
{"label": "吉安县","value":"360821","disabled": false},
{"label": "吉水县","value":"360822","disabled": false},
{"label": "峡江县","value":"360823","disabled": false},
{"label": "新干县","value":"360824","disabled": false},
{"label": "永丰县","value":"360825","disabled": false},
{"label": "泰和县","value":"360826","disabled": false},
{"label": "遂川县","value":"360827","disabled": false},
{"label": "万安县","value":"360828","disabled": false},
{"label": "安福县","value":"360829","disabled": false},
{"label": "永新县","value":"360830","disabled": false},
{"label": "井冈山市","value":"360881","disabled": false},
{"label": "袁州区","value":"360902","disabled": false},
{"label": "奉新县","value":"360921","disabled": false},
{"label": "万载县","value":"360922","disabled": false},
{"label": "上高县","value":"360923","disabled": false},
{"label": "宜丰县","value":"360924","disabled": false},
{"label": "靖安县","value":"360925","disabled": false},
{"label": "铜鼓县","value":"360926","disabled": false},
{"label": "丰城市","value":"360981","disabled": false},
{"label": "樟树市","value":"360982","disabled": false},
{"label": "高安市","value":"360983","disabled": false},
{"label": "临川区","value":"361002","disabled": false},
{"label": "南城县","value":"361021","disabled": false},
{"label": "黎川县","value":"361022","disabled": false},
{"label": "南丰县","value":"361023","disabled": false},
{"label": "崇仁县","value":"361024","disabled": false},
{"label": "乐安县","value":"361025","disabled": false},
{"label": "宜黄县","value":"361026","disabled": false},
{"label": "金溪县","value":"361027","disabled": false},
{"label": "资溪县","value":"361028","disabled": false},
{"label": "东乡县","value":"361029","disabled": false},
{"label": "广昌县","value":"361030","disabled": false},
{"label": "信州区","value":"361102","disabled": false},
{"label": "上饶县","value":"361121","disabled": false},
{"label": "广丰县","value":"361122","disabled": false},
{"label": "玉山县","value":"361123","disabled": false},
{"label": "铅山县","value":"361124","disabled": false},
{"label": "横峰县","value":"361125","disabled": false},
{"label": "弋阳县","value":"361126","disabled": false},
{"label": "余干县","value":"361127","disabled": false},
{"label": "鄱阳县","value":"361128","disabled": false},
{"label": "万年县","value":"361129","disabled": false},
{"label": "婺源县","value":"361130","disabled": false},
{"label": "德兴市","value":"361181","disabled": false},
{"label": "历下区","value":"370102","disabled": false},
{"label": "市中区","value":"370103","disabled": false},
{"label": "槐荫区","value":"370104","disabled": false},
{"label": "天桥区","value":"370105","disabled": false},
{"label": "历城区","value":"370112","disabled": false},
{"label": "长清区","value":"370113","disabled": false},
{"label": "平阴县","value":"370124","disabled": false},
{"label": "济阳县","value":"370125","disabled": false},
{"label": "商河县","value":"370126","disabled": false},
{"label": "章丘市","value":"370181","disabled": false},
{"label": "市南区","value":"370202","disabled": false},
{"label": "市北区","value":"370203","disabled": false},
{"label": "四方区","value":"370205","disabled": false},
{"label": "黄岛区","value":"370211","disabled": false},
{"label": "崂山区","value":"370212","disabled": false},
{"label": "李沧区","value":"370213","disabled": false},
{"label": "城阳区","value":"370214","disabled": false},
{"label": "胶州市","value":"370281","disabled": false},
{"label": "即墨市","value":"370282","disabled": false},
{"label": "平度市","value":"370283","disabled": false},
{"label": "胶南市","value":"370284","disabled": false},
{"label": "莱西市","value":"370285","disabled": false},
{"label": "淄川区","value":"370302","disabled": false},
{"label": "张店区","value":"370303","disabled": false},
{"label": "博山区","value":"370304","disabled": false},
{"label": "临淄区","value":"370305","disabled": false},
{"label": "周村区","value":"370306","disabled": false},
{"label": "桓台县","value":"370321","disabled": false},
{"label": "高青县","value":"370322","disabled": false},
{"label": "沂源县","value":"370323","disabled": false},
{"label": "市中区","value":"370402","disabled": false},
{"label": "薛城区","value":"370403","disabled": false},
{"label": "峄城区","value":"370404","disabled": false},
{"label": "台儿庄区","value":"370405","disabled": false},
{"label": "山亭区","value":"370406","disabled": false},
{"label": "滕州市","value":"370481","disabled": false},
{"label": "东营区","value":"370502","disabled": false},
{"label": "河口区","value":"370503","disabled": false},
{"label": "垦利县","value":"370521","disabled": false},
{"label": "利津县","value":"370522","disabled": false},
{"label": "广饶县","value":"370523","disabled": false},
{"label": "芝罘区","value":"370602","disabled": false},
{"label": "福山区","value":"370611","disabled": false},
{"label": "牟平区","value":"370612","disabled": false},
{"label": "莱山区","value":"370613","disabled": false},
{"label": "长岛县","value":"370634","disabled": false},
{"label": "龙口市","value":"370681","disabled": false},
{"label": "莱阳市","value":"370682","disabled": false},
{"label": "莱州市","value":"370683","disabled": false},
{"label": "蓬莱市","value":"370684","disabled": false},
{"label": "招远市","value":"370685","disabled": false},
{"label": "栖霞市","value":"370686","disabled": false},
{"label": "海阳市","value":"370687","disabled": false},
{"label": "潍城区","value":"370702","disabled": false},
{"label": "寒亭区","value":"370703","disabled": false},
{"label": "坊子区","value":"370704","disabled": false},
{"label": "奎文区","value":"370705","disabled": false},
{"label": "临朐县","value":"370724","disabled": false},
{"label": "昌乐县","value":"370725","disabled": false},
{"label": "青州市","value":"370781","disabled": false},
{"label": "诸城市","value":"370782","disabled": false},
{"label": "寿光市","value":"370783","disabled": false},
{"label": "安丘市","value":"370784","disabled": false},
{"label": "高密市","value":"370785","disabled": false},
{"label": "昌邑市","value":"370786","disabled": false},
{"label": "市中区","value":"370802","disabled": false},
{"label": "任城区","value":"370811","disabled": false},
{"label": "微山县","value":"370826","disabled": false},
{"label": "鱼台县","value":"370827","disabled": false},
{"label": "金乡县","value":"370828","disabled": false},
{"label": "嘉祥县","value":"370829","disabled": false},
{"label": "汶上县","value":"370830","disabled": false},
{"label": "泗水县","value":"370831","disabled": false},
{"label": "梁山县","value":"370832","disabled": false},
{"label": "曲阜市","value":"370881","disabled": false},
{"label": "兖州市","value":"370882","disabled": false},
{"label": "邹城市","value":"370883","disabled": false},
{"label": "泰山区","value":"370902","disabled": false},
{"label": "岱岳区","value":"370911","disabled": false},
{"label": "宁阳县","value":"370921","disabled": false},
{"label": "东平县","value":"370923","disabled": false},
{"label": "新泰市","value":"370982","disabled": false},
{"label": "肥城市","value":"370983","disabled": false},
{"label": "环翠区","value":"371002","disabled": false},
{"label": "文登市","value":"371081","disabled": false},
{"label": "荣成市","value":"371082","disabled": false},
{"label": "乳山市","value":"371083","disabled": false},
{"label": "东港区","value":"371102","disabled": false},
{"label": "岚山区","value":"371103","disabled": false},
{"label": "五莲县","value":"371121","disabled": false},
{"label": "莒县","value":"371122","disabled": false},
{"label": "莱城区","value":"371202","disabled": false},
{"label": "钢城区","value":"371203","disabled": false},
{"label": "兰山区","value":"371302","disabled": false},
{"label": "罗庄区","value":"371311","disabled": false},
{"label": "河东区","value":"371312","disabled": false},
{"label": "沂南县","value":"371321","disabled": false},
{"label": "郯城县","value":"371322","disabled": false},
{"label": "沂水县","value":"371323","disabled": false},
{"label": "苍山县","value":"371324","disabled": false},
{"label": "费县","value":"371325","disabled": false},
{"label": "平邑县","value":"371326","disabled": false},
{"label": "莒南县","value":"371327","disabled": false},
{"label": "蒙阴县","value":"371328","disabled": false},
{"label": "临沭县","value":"371329","disabled": false},
{"label": "德城区","value":"371402","disabled": false},
{"label": "陵县","value":"371421","disabled": false},
{"label": "宁津县","value":"371422","disabled": false},
{"label": "庆云县","value":"371423","disabled": false},
{"label": "临邑县","value":"371424","disabled": false},
{"label": "齐河县","value":"371425","disabled": false},
{"label": "平原县","value":"371426","disabled": false},
{"label": "夏津县","value":"371427","disabled": false},
{"label": "武城县","value":"371428","disabled": false},
{"label": "乐陵市","value":"371481","disabled": false},
{"label": "禹城市","value":"371482","disabled": false},
{"label": "东昌府区","value":"371502","disabled": false},
{"label": "阳谷县","value":"371521","disabled": false},
{"label": "莘县","value":"371522","disabled": false},
{"label": "茌平县","value":"371523","disabled": false},
{"label": "东阿县","value":"371524","disabled": false},
{"label": "冠县","value":"371525","disabled": false},
{"label": "高唐县","value":"371526","disabled": false},
{"label": "临清市","value":"371581","disabled": false},
{"label": "滨城区","value":"371602","disabled": false},
{"label": "惠民县","value":"371621","disabled": false},
{"label": "阳信县","value":"371622","disabled": false},
{"label": "无棣县","value":"371623","disabled": false},
{"label": "沾化县","value":"371624","disabled": false},
{"label": "博兴县","value":"371625","disabled": false},
{"label": "邹平县","value":"371626","disabled": false},
{"label": "牡丹区","value":"371702","disabled": false},
{"label": "曹县","value":"371721","disabled": false},
{"label": "单县","value":"371722","disabled": false},
{"label": "成武县","value":"371723","disabled": false},
{"label": "巨野县","value":"371724","disabled": false},
{"label": "郓城县","value":"371725","disabled": false},
{"label": "鄄城县","value":"371726","disabled": false},
{"label": "定陶县","value":"371727","disabled": false},
{"label": "东明县","value":"371728","disabled": false},
{"label": "中原区","value":"410102","disabled": false},
{"label": "二七区","value":"410103","disabled": false},
{"label": "管城回族区","value":"410104","disabled": false},
{"label": "金水区","value":"410105","disabled": false},
{"label": "上街区","value":"410106","disabled": false},
{"label": "惠济区","value":"410108","disabled": false},
{"label": "中牟县","value":"410122","disabled": false},
{"label": "巩义市","value":"410181","disabled": false},
{"label": "荥阳市","value":"410182","disabled": false},
{"label": "新密市","value":"410183","disabled": false},
{"label": "新郑市","value":"410184","disabled": false},
{"label": "登封市","value":"410185","disabled": false},
{"label": "龙亭区","value":"410202","disabled": false},
{"label": "顺河回族区","value":"410203","disabled": false},
{"label": "鼓楼区","value":"410204","disabled": false},
{"label": "禹王台区","value":"410205","disabled": false},
{"label": "金明区","value":"410211","disabled": false},
{"label": "杞县","value":"410221","disabled": false},
{"label": "通许县","value":"410222","disabled": false},
{"label": "尉氏县","value":"410223","disabled": false},
{"label": "开封县","value":"410224","disabled": false},
{"label": "兰考县","value":"410225","disabled": false},
{"label": "老城区","value":"410302","disabled": false},
{"label": "西工区","value":"410303","disabled": false},
{"label": "瀍河回族区","value":"410304","disabled": false},
{"label": "涧西区","value":"410305","disabled": false},
{"label": "吉利区","value":"410306","disabled": false},
{"label": "洛龙区","value":"410311","disabled": false},
{"label": "孟津县","value":"410322","disabled": false},
{"label": "新安县","value":"410323","disabled": false},
{"label": "栾川县","value":"410324","disabled": false},
{"label": "嵩县","value":"410325","disabled": false},
{"label": "汝阳县","value":"410326","disabled": false},
{"label": "宜阳县","value":"410327","disabled": false},
{"label": "洛宁县","value":"410328","disabled": false},
{"label": "伊川县","value":"410329","disabled": false},
{"label": "偃师市","value":"410381","disabled": false},
{"label": "新华区","value":"410402","disabled": false},
{"label": "卫东区","value":"410403","disabled": false},
{"label": "石龙区","value":"410404","disabled": false},
{"label": "湛河区","value":"410411","disabled": false},
{"label": "宝丰县","value":"410421","disabled": false},
{"label": "叶县","value":"410422","disabled": false},
{"label": "鲁山县","value":"410423","disabled": false},
{"label": "郏县","value":"410425","disabled": false},
{"label": "舞钢市","value":"410481","disabled": false},
{"label": "汝州市","value":"410482","disabled": false},
{"label": "文峰区","value":"410502","disabled": false},
{"label": "北关区","value":"410503","disabled": false},
{"label": "殷都区","value":"410505","disabled": false},
{"label": "龙安区","value":"410506","disabled": false},
{"label": "安阳县","value":"410522","disabled": false},
{"label": "汤阴县","value":"410523","disabled": false},
{"label": "滑县","value":"410526","disabled": false},
{"label": "内黄县","value":"410527","disabled": false},
{"label": "林州市","value":"410581","disabled": false},
{"label": "鹤山区","value":"410602","disabled": false},
{"label": "山城区","value":"410603","disabled": false},
{"label": "淇滨区","value":"410611","disabled": false},
{"label": "浚县","value":"410621","disabled": false},
{"label": "淇县","value":"410622","disabled": false},
{"label": "红旗区","value":"410702","disabled": false},
{"label": "卫滨区","value":"410703","disabled": false},
{"label": "凤泉区","value":"410704","disabled": false},
{"label": "牧野区","value":"410711","disabled": false},
{"label": "新乡县","value":"410721","disabled": false},
{"label": "获嘉县","value":"410724","disabled": false},
{"label": "原阳县","value":"410725","disabled": false},
{"label": "延津县","value":"410726","disabled": false},
{"label": "封丘县","value":"410727","disabled": false},
{"label": "长垣县","value":"410728","disabled": false},
{"label": "卫辉市","value":"410781","disabled": false},
{"label": "辉县市","value":"410782","disabled": false},
{"label": "解放区","value":"410802","disabled": false},
{"label": "中站区","value":"410803","disabled": false},
{"label": "马村区","value":"410804","disabled": false},
{"label": "山阳区","value":"410811","disabled": false},
{"label": "修武县","value":"410821","disabled": false},
{"label": "博爱县","value":"410822","disabled": false},
{"label": "武陟县","value":"410823","disabled": false},
{"label": "温县","value":"410825","disabled": false},
{"label": "沁阳市","value":"410882","disabled": false},
{"label": "孟州市","value":"410883","disabled": false},
{"label": "华龙区","value":"410902","disabled": false},
{"label": "清丰县","value":"410922","disabled": false},
{"label": "南乐县","value":"410923","disabled": false},
{"label": "范县","value":"410926","disabled": false},
{"label": "台前县","value":"410927","disabled": false},
{"label": "濮阳县","value":"410928","disabled": false},
{"label": "魏都区","value":"411002","disabled": false},
{"label": "许昌县","value":"411023","disabled": false},
{"label": "鄢陵县","value":"411024","disabled": false},
{"label": "襄城县","value":"411025","disabled": false},
{"label": "禹州市","value":"411081","disabled": false},
{"label": "长葛市","value":"411082","disabled": false},
{"label": "源汇区","value":"411102","disabled": false},
{"label": "郾城区","value":"411103","disabled": false},
{"label": "召陵区","value":"411104","disabled": false},
{"label": "舞阳县","value":"411121","disabled": false},
{"label": "临颍县","value":"411122","disabled": false},
{"label": "湖滨区","value":"411202","disabled": false},
{"label": "渑池县","value":"411221","disabled": false},
{"label": "陕县","value":"411222","disabled": false},
{"label": "卢氏县","value":"411224","disabled": false},
{"label": "义马市","value":"411281","disabled": false},
{"label": "灵宝市","value":"411282","disabled": false},
{"label": "宛城区","value":"411302","disabled": false},
{"label": "卧龙区","value":"411303","disabled": false},
{"label": "南召县","value":"411321","disabled": false},
{"label": "方城县","value":"411322","disabled": false},
{"label": "西峡县","value":"411323","disabled": false},
{"label": "镇平县","value":"411324","disabled": false},
{"label": "内乡县","value":"411325","disabled": false},
{"label": "淅川县","value":"411326","disabled": false},
{"label": "社旗县","value":"411327","disabled": false},
{"label": "唐河县","value":"411328","disabled": false},
{"label": "新野县","value":"411329","disabled": false},
{"label": "桐柏县","value":"411330","disabled": false},
{"label": "邓州市","value":"411381","disabled": false},
{"label": "梁园区","value":"411402","disabled": false},
{"label": "睢阳区","value":"411403","disabled": false},
{"label": "民权县","value":"411421","disabled": false},
{"label": "睢县","value":"411422","disabled": false},
{"label": "宁陵县","value":"411423","disabled": false},
{"label": "柘城县","value":"411424","disabled": false},
{"label": "虞城县","value":"411425","disabled": false},
{"label": "夏邑县","value":"411426","disabled": false},
{"label": "永城市","value":"411481","disabled": false},
{"label": "浉河区","value":"411502","disabled": false},
{"label": "平桥区","value":"411503","disabled": false},
{"label": "罗山县","value":"411521","disabled": false},
{"label": "光山县","value":"411522","disabled": false},
{"label": "新县","value":"411523","disabled": false},
{"label": "商城县","value":"411524","disabled": false},
{"label": "固始县","value":"411525","disabled": false},
{"label": "潢川县","value":"411526","disabled": false},
{"label": "淮滨县","value":"411527","disabled": false},
{"label": "息县","value":"411528","disabled": false},
{"label": "川汇区","value":"411602","disabled": false},
{"label": "扶沟县","value":"411621","disabled": false},
{"label": "西华县","value":"411622","disabled": false},
{"label": "商水县","value":"411623","disabled": false},
{"label": "沈丘县","value":"411624","disabled": false},
{"label": "郸城县","value":"411625","disabled": false},
{"label": "淮阳县","value":"411626","disabled": false},
{"label": "太康县","value":"411627","disabled": false},
{"label": "鹿邑县","value":"411628","disabled": false},
{"label": "项城市","value":"411681","disabled": false},
{"label": "驿城区","value":"411702","disabled": false},
{"label": "西平县","value":"411721","disabled": false},
{"label": "上蔡县","value":"411722","disabled": false},
{"label": "平舆县","value":"411723","disabled": false},
{"label": "正阳县","value":"411724","disabled": false},
{"label": "确山县","value":"411725","disabled": false},
{"label": "泌阳县","value":"411726","disabled": false},
{"label": "汝南县","value":"411727","disabled": false},
{"label": "遂平县","value":"411728","disabled": false},
{"label": "新蔡县","value":"411729","disabled": false},
{"label": "江岸区","value":"420102","disabled": false},
{"label": "江汉区","value":"420103","disabled": false},
{"label": "硚口区","value":"420104","disabled": false},
{"label": "汉阳区","value":"420105","disabled": false},
{"label": "武昌区","value":"420106","disabled": false},
{"label": "青山区","value":"420107","disabled": false},
{"label": "洪山区","value":"420111","disabled": false},
{"label": "东西湖区","value":"420112","disabled": false},
{"label": "汉南区","value":"420113","disabled": false},
{"label": "蔡甸区","value":"420114","disabled": false},
{"label": "江夏区","value":"420115","disabled": false},
{"label": "黄陂区","value":"420116","disabled": false},
{"label": "新洲区","value":"420117","disabled": false},
{"label": "黄石港区","value":"420202","disabled": false},
{"label": "西塞山区","value":"420203","disabled": false},
{"label": "下陆区","value":"420204","disabled": false},
{"label": "铁山区","value":"420205","disabled": false},
{"label": "阳新县","value":"420222","disabled": false},
{"label": "大冶市","value":"420281","disabled": false},
{"label": "茅箭区","value":"420302","disabled": false},
{"label": "张湾区","value":"420303","disabled": false},
{"label": "郧县","value":"420321","disabled": false},
{"label": "郧西县","value":"420322","disabled": false},
{"label": "竹山县","value":"420323","disabled": false},
{"label": "竹溪县","value":"420324","disabled": false},
{"label": "房县","value":"420325","disabled": false},
{"label": "丹江口市","value":"420381","disabled": false},
{"label": "西陵区","value":"420502","disabled": false},
{"label": "伍家岗区","value":"420503","disabled": false},
{"label": "点军区","value":"420504","disabled": false},
{"label": "猇亭区","value":"420505","disabled": false},
{"label": "夷陵区","value":"420506","disabled": false},
{"label": "远安县","value":"420525","disabled": false},
{"label": "兴山县","value":"420526","disabled": false},
{"label": "秭归县","value":"420527","disabled": false},
{"label": "长阳土家族自治县","value":"420528","disabled": false},
{"label": "五峰土家族自治县","value":"420529","disabled": false},
{"label": "宜都市","value":"420581","disabled": false},
{"label": "当阳市","value":"420582","disabled": false},
{"label": "枝江市","value":"420583","disabled": false},
{"label": "襄城区","value":"420602","disabled": false},
{"label": "樊城区","value":"420606","disabled": false},
{"label": "襄州区","value":"420607","disabled": false},
{"label": "南漳县","value":"420624","disabled": false},
{"label": "谷城县","value":"420625","disabled": false},
{"label": "保康县","value":"420626","disabled": false},
{"label": "老河口市","value":"420682","disabled": false},
{"label": "枣阳市","value":"420683","disabled": false},
{"label": "宜城市","value":"420684","disabled": false},
{"label": "梁子湖区","value":"420702","disabled": false},
{"label": "华容区","value":"420703","disabled": false},
{"label": "鄂城区","value":"420704","disabled": false},
{"label": "东宝区","value":"420802","disabled": false},
{"label": "掇刀区","value":"420804","disabled": false},
{"label": "京山县","value":"420821","disabled": false},
{"label": "沙洋县","value":"420822","disabled": false},
{"label": "钟祥市","value":"420881","disabled": false},
{"label": "孝南区","value":"420902","disabled": false},
{"label": "孝昌县","value":"420921","disabled": false},
{"label": "大悟县","value":"420922","disabled": false},
{"label": "云梦县","value":"420923","disabled": false},
{"label": "应城市","value":"420981","disabled": false},
{"label": "安陆市","value":"420982","disabled": false},
{"label": "汉川市","value":"420984","disabled": false},
{"label": "沙市区","value":"421002","disabled": false},
{"label": "荆州区","value":"421003","disabled": false},
{"label": "公安县","value":"421022","disabled": false},
{"label": "监利县","value":"421023","disabled": false},
{"label": "江陵县","value":"421024","disabled": false},
{"label": "石首市","value":"421081","disabled": false},
{"label": "洪湖市","value":"421083","disabled": false},
{"label": "松滋市","value":"421087","disabled": false},
{"label": "黄州区","value":"421102","disabled": false},
{"label": "团风县","value":"421121","disabled": false},
{"label": "红安县","value":"421122","disabled": false},
{"label": "罗田县","value":"421123","disabled": false},
{"label": "英山县","value":"421124","disabled": false},
{"label": "浠水县","value":"421125","disabled": false},
{"label": "蕲春县","value":"421126","disabled": false},
{"label": "黄梅县","value":"421127","disabled": false},
{"label": "麻城市","value":"421181","disabled": false},
{"label": "武穴市","value":"421182","disabled": false},
{"label": "咸安区","value":"421202","disabled": false},
{"label": "嘉鱼县","value":"421221","disabled": false},
{"label": "通城县","value":"421222","disabled": false},
{"label": "崇阳县","value":"421223","disabled": false},
{"label": "通山县","value":"421224","disabled": false},
{"label": "赤壁市","value":"421281","disabled": false},
{"label": "曾都区","value":"421302","disabled": false},
{"label": "广水市","value":"421381","disabled": false},
{"label": "恩施市","value":"422801","disabled": false},
{"label": "利川市","value":"422802","disabled": false},
{"label": "建始县","value":"422822","disabled": false},
{"label": "巴东县","value":"422823","disabled": false},
{"label": "宣恩县","value":"422825","disabled": false},
{"label": "咸丰县","value":"422826","disabled": false},
{"label": "来凤县","value":"422827","disabled": false},
{"label": "鹤峰县","value":"422828","disabled": false},
{"label": "芙蓉区","value":"430102","disabled": false},
{"label": "天心区","value":"430103","disabled": false},
{"label": "岳麓区","value":"430104","disabled": false},
{"label": "开福区","value":"430105","disabled": false},
{"label": "雨花区","value":"430111","disabled": false},
{"label": "长沙县","value":"430121","disabled": false},
{"label": "望城县","value":"430122","disabled": false},
{"label": "宁乡县","value":"430124","disabled": false},
{"label": "浏阳市","value":"430181","disabled": false},
{"label": "荷塘区","value":"430202","disabled": false},
{"label": "芦淞区","value":"430203","disabled": false},
{"label": "石峰区","value":"430204","disabled": false},
{"label": "天元区","value":"430211","disabled": false},
{"label": "株洲县","value":"430221","disabled": false},
{"label": "攸县","value":"430223","disabled": false},
{"label": "茶陵县","value":"430224","disabled": false},
{"label": "炎陵县","value":"430225","disabled": false},
{"label": "醴陵市","value":"430281","disabled": false},
{"label": "雨湖区","value":"430302","disabled": false},
{"label": "岳塘区","value":"430304","disabled": false},
{"label": "湘潭县","value":"430321","disabled": false},
{"label": "湘乡市","value":"430381","disabled": false},
{"label": "韶山市","value":"430382","disabled": false},
{"label": "珠晖区","value":"430405","disabled": false},
{"label": "雁峰区","value":"430406","disabled": false},
{"label": "石鼓区","value":"430407","disabled": false},
{"label": "蒸湘区","value":"430408","disabled": false},
{"label": "南岳区","value":"430412","disabled": false},
{"label": "衡阳县","value":"430421","disabled": false},
{"label": "衡南县","value":"430422","disabled": false},
{"label": "衡山县","value":"430423","disabled": false},
{"label": "衡东县","value":"430424","disabled": false},
{"label": "祁东县","value":"430426","disabled": false},
{"label": "耒阳市","value":"430481","disabled": false},
{"label": "常宁市","value":"430482","disabled": false},
{"label": "双清区","value":"430502","disabled": false},
{"label": "大祥区","value":"430503","disabled": false},
{"label": "北塔区","value":"430511","disabled": false},
{"label": "邵东县","value":"430521","disabled": false},
{"label": "新邵县","value":"430522","disabled": false},
{"label": "邵阳县","value":"430523","disabled": false},
{"label": "隆回县","value":"430524","disabled": false},
{"label": "洞口县","value":"430525","disabled": false},
{"label": "绥宁县","value":"430527","disabled": false},
{"label": "新宁县","value":"430528","disabled": false},
{"label": "城步苗族自治县","value":"430529","disabled": false},
{"label": "武冈市","value":"430581","disabled": false},
{"label": "岳阳楼区","value":"430602","disabled": false},
{"label": "云溪区","value":"430603","disabled": false},
{"label": "君山区","value":"430611","disabled": false},
{"label": "岳阳县","value":"430621","disabled": false},
{"label": "华容县","value":"430623","disabled": false},
{"label": "湘阴县","value":"430624","disabled": false},
{"label": "平江县","value":"430626","disabled": false},
{"label": "汨罗市","value":"430681","disabled": false},
{"label": "临湘市","value":"430682","disabled": false},
{"label": "武陵区","value":"430702","disabled": false},
{"label": "鼎城区","value":"430703","disabled": false},
{"label": "安乡县","value":"430721","disabled": false},
{"label": "汉寿县","value":"430722","disabled": false},
{"label": "澧县","value":"430723","disabled": false},
{"label": "临澧县","value":"430724","disabled": false},
{"label": "桃源县","value":"430725","disabled": false},
{"label": "石门县","value":"430726","disabled": false},
{"label": "津市市","value":"430781","disabled": false},
{"label": "永定区","value":"430802","disabled": false},
{"label": "武陵源区","value":"430811","disabled": false},
{"label": "慈利县","value":"430821","disabled": false},
{"label": "桑植县","value":"430822","disabled": false},
{"label": "资阳区","value":"430902","disabled": false},
{"label": "赫山区","value":"430903","disabled": false},
{"label": "南县","value":"430921","disabled": false},
{"label": "桃江县","value":"430922","disabled": false},
{"label": "安化县","value":"430923","disabled": false},
{"label": "沅江市","value":"430981","disabled": false},
{"label": "北湖区","value":"431002","disabled": false},
{"label": "苏仙区","value":"431003","disabled": false},
{"label": "桂阳县","value":"431021","disabled": false},
{"label": "宜章县","value":"431022","disabled": false},
{"label": "永兴县","value":"431023","disabled": false},
{"label": "嘉禾县","value":"431024","disabled": false},
{"label": "临武县","value":"431025","disabled": false},
{"label": "汝城县","value":"431026","disabled": false},
{"label": "桂东县","value":"431027","disabled": false},
{"label": "安仁县","value":"431028","disabled": false},
{"label": "资兴市","value":"431081","disabled": false},
{"label": "零陵区","value":"431102","disabled": false},
{"label": "冷水滩区","value":"431103","disabled": false},
{"label": "祁阳县","value":"431121","disabled": false},
{"label": "东安县","value":"431122","disabled": false},
{"label": "双牌县","value":"431123","disabled": false},
{"label": "道县","value":"431124","disabled": false},
{"label": "江永县","value":"431125","disabled": false},
{"label": "宁远县","value":"431126","disabled": false},
{"label": "蓝山县","value":"431127","disabled": false},
{"label": "新田县","value":"431128","disabled": false},
{"label": "江华瑶族自治县","value":"431129","disabled": false},
{"label": "鹤城区","value":"431202","disabled": false},
{"label": "中方县","value":"431221","disabled": false},
{"label": "沅陵县","value":"431222","disabled": false},
{"label": "辰溪县","value":"431223","disabled": false},
{"label": "溆浦县","value":"431224","disabled": false},
{"label": "会同县","value":"431225","disabled": false},
{"label": "麻阳苗族自治县","value":"431226","disabled": false},
{"label": "新晃侗族自治县","value":"431227","disabled": false},
{"label": "芷江侗族自治县","value":"431228","disabled": false},
{"label": "靖州苗族侗族自治县","value":"431229","disabled": false},
{"label": "通道侗族自治县","value":"431230","disabled": false},
{"label": "洪江市","value":"431281","disabled": false},
{"label": "娄星区","value":"431302","disabled": false},
{"label": "双峰县","value":"431321","disabled": false},
{"label": "新化县","value":"431322","disabled": false},
{"label": "冷水江市","value":"431381","disabled": false},
{"label": "涟源市","value":"431382","disabled": false},
{"label": "吉首市","value":"433101","disabled": false},
{"label": "泸溪县","value":"433122","disabled": false},
{"label": "凤凰县","value":"433123","disabled": false},
{"label": "花垣县","value":"433124","disabled": false},
{"label": "保靖县","value":"433125","disabled": false},
{"label": "古丈县","value":"433126","disabled": false},
{"label": "永顺县","value":"433127","disabled": false},
{"label": "龙山县","value":"433130","disabled": false},
{"label": "荔湾区","value":"440103","disabled": false},
{"label": "越秀区","value":"440104","disabled": false},
{"label": "海珠区","value":"440105","disabled": false},
{"label": "天河区","value":"440106","disabled": false},
{"label": "白云区","value":"440111","disabled": false},
{"label": "黄埔区","value":"440112","disabled": false},
{"label": "番禺区","value":"440113","disabled": false},
{"label": "花都区","value":"440114","disabled": false},
{"label": "南沙区","value":"440115","disabled": false},
{"label": "萝岗区","value":"440116","disabled": false},
{"label": "增城区","value":"440183","disabled": false},
{"label": "从化市","value":"440184","disabled": false},
{"label": "武江区","value":"440203","disabled": false},
{"label": "浈江区","value":"440204","disabled": false},
{"label": "曲江区","value":"440205","disabled": false},
{"label": "始兴县","value":"440222","disabled": false},
{"label": "仁化县","value":"440224","disabled": false},
{"label": "翁源县","value":"440229","disabled": false},
{"label": "乳源瑶族自治县","value":"440232","disabled": false},
{"label": "新丰县","value":"440233","disabled": false},
{"label": "乐昌市","value":"440281","disabled": false},
{"label": "南雄市","value":"440282","disabled": false},
{"label": "罗湖区","value":"440303","disabled": false},
{"label": "福田区","value":"440304","disabled": false},
{"label": "南山区","value":"440305","disabled": false},
{"label": "宝安区","value":"440306","disabled": false},
{"label": "龙岗区","value":"440307","disabled": false},
{"label": "盐田区","value":"440308","disabled": false},
{"label": "光明新区","value":"440309","disabled": false},
{"label": "龙华新区","value":"440310","disabled": false},
{"label": "坪山新区","value":"440311","disabled": false},
{"label": "大鹏新区","value":"440312","disabled": false},
{"label": "香洲区","value":"440402","disabled": false},
{"label": "斗门区","value":"440403","disabled": false},
{"label": "金湾区","value":"440404","disabled": false},
{"label": "龙湖区","value":"440507","disabled": false},
{"label": "金平区","value":"440511","disabled": false},
{"label": "濠江区","value":"440512","disabled": false},
{"label": "潮阳区","value":"440513","disabled": false},
{"label": "潮南区","value":"440514","disabled": false},
{"label": "澄海区","value":"440515","disabled": false},
{"label": "南澳县","value":"440523","disabled": false},
{"label": "禅城区","value":"440604","disabled": false},
{"label": "南海区","value":"440605","disabled": false},
{"label": "顺德区","value":"440606","disabled": false},
{"label": "三水区","value":"440607","disabled": false},
{"label": "高明区","value":"440608","disabled": false},
{"label": "蓬江区","value":"440703","disabled": false},
{"label": "江海区","value":"440704","disabled": false},
{"label": "新会区","value":"440705","disabled": false},
{"label": "台山市","value":"440781","disabled": false},
{"label": "开平市","value":"440783","disabled": false},
{"label": "鹤山市","value":"440784","disabled": false},
{"label": "恩平市","value":"440785","disabled": false},
{"label": "赤坎区","value":"440802","disabled": false},
{"label": "霞山区","value":"440803","disabled": false},
{"label": "坡头区","value":"440804","disabled": false},
{"label": "麻章区","value":"440811","disabled": false},
{"label": "遂溪县","value":"440823","disabled": false},
{"label": "徐闻县","value":"440825","disabled": false},
{"label": "廉江市","value":"440881","disabled": false},
{"label": "雷州市","value":"440882","disabled": false},
{"label": "吴川市","value":"440883","disabled": false},
{"label": "茂南区","value":"440902","disabled": false},
{"label": "茂港区","value":"440903","disabled": false},
{"label": "电白县","value":"440923","disabled": false},
{"label": "高州市","value":"440981","disabled": false},
{"label": "化州市","value":"440982","disabled": false},
{"label": "信宜市","value":"440983","disabled": false},
{"label": "端州区","value":"441202","disabled": false},
{"label": "鼎湖区","value":"441203","disabled": false},
{"label": "广宁县","value":"441223","disabled": false},
{"label": "怀集县","value":"441224","disabled": false},
{"label": "封开县","value":"441225","disabled": false},
{"label": "德庆县","value":"441226","disabled": false},
{"label": "高要市","value":"441283","disabled": false},
{"label": "四会市","value":"441284","disabled": false},
{"label": "惠城区","value":"441302","disabled": false},
{"label": "惠阳区","value":"441303","disabled": false},
{"label": "博罗县","value":"441322","disabled": false},
{"label": "惠东县","value":"441323","disabled": false},
{"label": "龙门县","value":"441324","disabled": false},
{"label": "梅江区","value":"441402","disabled": false},
{"label": "梅县","value":"441421","disabled": false},
{"label": "大埔县","value":"441422","disabled": false},
{"label": "丰顺县","value":"441423","disabled": false},
{"label": "五华县","value":"441424","disabled": false},
{"label": "平远县","value":"441426","disabled": false},
{"label": "蕉岭县","value":"441427","disabled": false},
{"label": "兴宁市","value":"441481","disabled": false},
{"label": "城区","value":"441502","disabled": false},
{"label": "海丰县","value":"441521","disabled": false},
{"label": "陆河县","value":"441523","disabled": false},
{"label": "陆丰市","value":"441581","disabled": false},
{"label": "源城区","value":"441602","disabled": false},
{"label": "紫金县","value":"441621","disabled": false},
{"label": "龙川县","value":"441622","disabled": false},
{"label": "连平县","value":"441623","disabled": false},
{"label": "和平县","value":"441624","disabled": false},
{"label": "东源县","value":"441625","disabled": false},
{"label": "江城区","value":"441702","disabled": false},
{"label": "阳西县","value":"441721","disabled": false},
{"label": "阳东县","value":"441723","disabled": false},
{"label": "阳春市","value":"441781","disabled": false},
{"label": "清城区","value":"441802","disabled": false},
{"label": "佛冈县","value":"441821","disabled": false},
{"label": "阳山县","value":"441823","disabled": false},
{"label": "连山壮族瑶族自治县","value":"441825","disabled": false},
{"label": "连南瑶族自治县","value":"441826","disabled": false},
{"label": "清新县","value":"441827","disabled": false},
{"label": "英德市","value":"441881","disabled": false},
{"label": "连州市","value":"441882","disabled": false},
{"label": "莞城区","value":"441901","disabled": false},
{"label": "南城区","value":"441902","disabled": false},
{"label": "东城区","value":"441903","disabled": false},
{"label": "万江区","value":"441904","disabled": false},
{"label": "寮步镇","value":"441905","disabled": false},
{"label": "厚街镇","value":"441906","disabled": false},
{"label": "虎门镇","value":"441907","disabled": false},
{"label": "长安镇","value":"441908","disabled": false},
{"label": "常平镇","value":"441909","disabled": false},
{"label": "樟木头镇","value":"441910","disabled": false},
{"label": "大朗镇","value":"441911","disabled": false},
{"label": "石排镇","value":"441912","disabled": false},
{"label": "石龙镇","value":"441913","disabled": false},
{"label": "石碣镇","value":"441914","disabled": false},
{"label": "桥头镇","value":"441915","disabled": false},
{"label": "大岭山镇","value":"441916","disabled": false},
{"label": "清溪镇","value":"441917","disabled": false},
{"label": "企石镇","value":"441918","disabled": false},
{"label": "横沥镇","value":"441919","disabled": false},
{"label": "东坑镇","value":"441920","disabled": false},
{"label": "谢岗镇","value":"441921","disabled": false},
{"label": "黄江镇","value":"441922","disabled": false},
{"label": "塘厦镇","value":"441923","disabled": false},
{"label": "凤岗镇","value":"441924","disabled": false},
{"label": "沙田镇","value":"441925","disabled": false},
{"label": "洪梅镇","value":"441926","disabled": false},
{"label": "道滘镇","value":"441927","disabled": false},
{"label": "麻涌镇","value":"441928","disabled": false},
{"label": "高埗镇","value":"441929","disabled": false},
{"label": "茶山镇","value":"441930","disabled": false},
{"label": "望牛墩镇","value":"441931","disabled": false},
{"label": "中堂镇","value":"441932","disabled": false},
{"label": "松山湖","value":"441933","disabled": false},
{"label": "石岐区","value":"442001","disabled": false},
{"label": "东区","value":"442002","disabled": false},
{"label": "西区","value":"442003","disabled": false},
{"label": "南区","value":"442004","disabled": false},
{"label": "火炬区","value":"442005","disabled": false},
{"label": "南朗镇","value":"442006","disabled": false},
{"label": "东升镇","value":"442007","disabled": false},
{"label": "民众镇","value":"442008","disabled": false},
{"label": "五桂山镇","value":"442009","disabled": false},
{"label": "港口镇","value":"442010","disabled": false},
{"label": "古镇镇","value":"442011","disabled": false},
{"label": "小榄镇","value":"442012","disabled": false},
{"label": "三乡镇","value":"442013","disabled": false},
{"label": "沙溪镇","value":"442014","disabled": false},
{"label": "坦洲镇","value":"442015","disabled": false},
{"label": "大涌镇","value":"442016","disabled": false},
{"label": "黄圃镇","value":"442017","disabled": false},
{"label": "三角镇","value":"442018","disabled": false},
{"label": "东凤镇","value":"442019","disabled": false},
{"label": "阜沙镇","value":"442020","disabled": false},
{"label": "南头镇","value":"442021","disabled": false},
{"label": "横栏镇","value":"442022","disabled": false},
{"label": "板芙镇","value":"442023","disabled": false},
{"label": "神湾镇","value":"442024","disabled": false},
{"label": "湘桥区","value":"445102","disabled": false},
{"label": "潮安县","value":"445121","disabled": false},
{"label": "饶平县","value":"445122","disabled": false},
{"label": "榕城区","value":"445202","disabled": false},
{"label": "揭东县","value":"445221","disabled": false},
{"label": "揭西县","value":"445222","disabled": false},
{"label": "惠来县","value":"445224","disabled": false},
{"label": "普宁市","value":"445281","disabled": false},
{"label": "云城区","value":"445302","disabled": false},
{"label": "新兴县","value":"445321","disabled": false},
{"label": "郁南县","value":"445322","disabled": false},
{"label": "云安县","value":"445323","disabled": false},
{"label": "罗定市","value":"445381","disabled": false},
{"label": "兴宁区","value":"450102","disabled": false},
{"label": "青秀区","value":"450103","disabled": false},
{"label": "江南区","value":"450105","disabled": false},
{"label": "西乡塘区","value":"450107","disabled": false},
{"label": "良庆区","value":"450108","disabled": false},
{"label": "邕宁区","value":"450109","disabled": false},
{"label": "武鸣县","value":"450122","disabled": false},
{"label": "隆安县","value":"450123","disabled": false},
{"label": "马山县","value":"450124","disabled": false},
{"label": "上林县","value":"450125","disabled": false},
{"label": "宾阳县","value":"450126","disabled": false},
{"label": "横县","value":"450127","disabled": false},
{"label": "城中区","value":"450202","disabled": false},
{"label": "鱼峰区","value":"450203","disabled": false},
{"label": "柳南区","value":"450204","disabled": false},
{"label": "柳北区","value":"450205","disabled": false},
{"label": "柳江县","value":"450221","disabled": false},
{"label": "柳城县","value":"450222","disabled": false},
{"label": "鹿寨县","value":"450223","disabled": false},
{"label": "融安县","value":"450224","disabled": false},
{"label": "融水苗族自治县","value":"450225","disabled": false},
{"label": "三江侗族自治县","value":"450226","disabled": false},
{"label": "秀峰区","value":"450302","disabled": false},
{"label": "叠彩区","value":"450303","disabled": false},
{"label": "象山区","value":"450304","disabled": false},
{"label": "七星区","value":"450305","disabled": false},
{"label": "雁山区","value":"450311","disabled": false},
{"label": "阳朔县","value":"450321","disabled": false},
{"label": "临桂县","value":"450322","disabled": false},
{"label": "灵川县","value":"450323","disabled": false},
{"label": "全州县","value":"450324","disabled": false},
{"label": "兴安县","value":"450325","disabled": false},
{"label": "永福县","value":"450326","disabled": false},
{"label": "灌阳县","value":"450327","disabled": false},
{"label": "龙胜各族自治县","value":"450328","disabled": false},
{"label": "资源县","value":"450329","disabled": false},
{"label": "平乐县","value":"450330","disabled": false},
{"label": "荔蒲县","value":"450331","disabled": false},
{"label": "恭城瑶族自治县","value":"450332","disabled": false},
{"label": "万秀区","value":"450403","disabled": false},
{"label": "蝶山区","value":"450404","disabled": false},
{"label": "长洲区","value":"450405","disabled": false},
{"label": "苍梧县","value":"450421","disabled": false},
{"label": "藤县","value":"450422","disabled": false},
{"label": "蒙山县","value":"450423","disabled": false},
{"label": "岑溪市","value":"450481","disabled": false},
{"label": "海城区","value":"450502","disabled": false},
{"label": "银海区","value":"450503","disabled": false},
{"label": "铁山港区","value":"450512","disabled": false},
{"label": "合浦县","value":"450521","disabled": false},
{"label": "港口区","value":"450602","disabled": false},
{"label": "防城区","value":"450603","disabled": false},
{"label": "上思县","value":"450621","disabled": false},
{"label": "东兴市","value":"450681","disabled": false},
{"label": "钦南区","value":"450702","disabled": false},
{"label": "钦北区","value":"450703","disabled": false},
{"label": "灵山县","value":"450721","disabled": false},
{"label": "浦北县","value":"450722","disabled": false},
{"label": "港北区","value":"450802","disabled": false},
{"label": "港南区","value":"450803","disabled": false},
{"label": "覃塘区","value":"450804","disabled": false},
{"label": "平南县","value":"450821","disabled": false},
{"label": "桂平市","value":"450881","disabled": false},
{"label": "玉州区","value":"450902","disabled": false},
{"label": "容县","value":"450921","disabled": false},
{"label": "陆川县","value":"450922","disabled": false},
{"label": "博白县","value":"450923","disabled": false},
{"label": "兴业县","value":"450924","disabled": false},
{"label": "北流市","value":"450981","disabled": false},
{"label": "右江区","value":"451002","disabled": false},
{"label": "田阳县","value":"451021","disabled": false},
{"label": "田东县","value":"451022","disabled": false},
{"label": "平果县","value":"451023","disabled": false},
{"label": "德保县","value":"451024","disabled": false},
{"label": "靖西县","value":"451025","disabled": false},
{"label": "那坡县","value":"451026","disabled": false},
{"label": "凌云县","value":"451027","disabled": false},
{"label": "乐业县","value":"451028","disabled": false},
{"label": "田林县","value":"451029","disabled": false},
{"label": "西林县","value":"451030","disabled": false},
{"label": "隆林各族自治县","value":"451031","disabled": false},
{"label": "八步区","value":"451102","disabled": false},
{"label": "昭平县","value":"451121","disabled": false},
{"label": "钟山县","value":"451122","disabled": false},
{"label": "富川瑶族自治县","value":"451123","disabled": false},
{"label": "金城江区","value":"451202","disabled": false},
{"label": "南丹县","value":"451221","disabled": false},
{"label": "天峨县","value":"451222","disabled": false},
{"label": "凤山县","value":"451223","disabled": false},
{"label": "东兰县","value":"451224","disabled": false},
{"label": "罗城仫佬族自治县","value":"451225","disabled": false},
{"label": "环江毛南族自治县","value":"451226","disabled": false},
{"label": "巴马瑶族自治县","value":"451227","disabled": false},
{"label": "都安瑶族自治县","value":"451228","disabled": false},
{"label": "大化瑶族自治县","value":"451229","disabled": false},
{"label": "宜州市","value":"451281","disabled": false},
{"label": "兴宾区","value":"451302","disabled": false},
{"label": "忻城县","value":"451321","disabled": false},
{"label": "象州县","value":"451322","disabled": false},
{"label": "武宣县","value":"451323","disabled": false},
{"label": "金秀瑶族自治县","value":"451324","disabled": false},
{"label": "合山市","value":"451381","disabled": false},
{"label": "江洲区","value":"451402","disabled": false},
{"label": "扶绥县","value":"451421","disabled": false},
{"label": "宁明县","value":"451422","disabled": false},
{"label": "龙州县","value":"451423","disabled": false},
{"label": "大新县","value":"451424","disabled": false},
{"label": "天等县","value":"451425","disabled": false},
{"label": "凭祥市","value":"451481","disabled": false},
{"label": "秀英区","value":"460105","disabled": false},
{"label": "龙华区","value":"460106","disabled": false},
{"label": "琼山区","value":"460107","disabled": false},
{"label": "美兰区","value":"460108","disabled": false},
{"label": "锦江区","value":"510104","disabled": false},
{"label": "青羊区","value":"510105","disabled": false},
{"label": "金牛区","value":"510106","disabled": false},
{"label": "武侯区","value":"510107","disabled": false},
{"label": "成华区","value":"510108","disabled": false},
{"label": "龙泉驿区","value":"510112","disabled": false},
{"label": "青白江区","value":"510113","disabled": false},
{"label": "新都区","value":"510114","disabled": false},
{"label": "温江区","value":"510115","disabled": false},
{"label": "金堂县","value":"510121","disabled": false},
{"label": "双流县","value":"510122","disabled": false},
{"label": "郫县","value":"510124","disabled": false},
{"label": "大邑县","value":"510129","disabled": false},
{"label": "蒲江县","value":"510131","disabled": false},
{"label": "新津县","value":"510132","disabled": false},
{"label": "都江堰市","value":"510181","disabled": false},
{"label": "彭州市","value":"510182","disabled": false},
{"label": "邛崃市","value":"510183","disabled": false},
{"label": "崇州市","value":"510184","disabled": false},
{"label": "自流井区","value":"510302","disabled": false},
{"label": "贡井区","value":"510303","disabled": false},
{"label": "大安区","value":"510304","disabled": false},
{"label": "沿滩区","value":"510311","disabled": false},
{"label": "荣县","value":"510321","disabled": false},
{"label": "富顺县","value":"510322","disabled": false},
{"label": "东区","value":"510402","disabled": false},
{"label": "西区","value":"510403","disabled": false},
{"label": "仁和区","value":"510411","disabled": false},
{"label": "米易县","value":"510421","disabled": false},
{"label": "盐边县","value":"510422","disabled": false},
{"label": "江阳区","value":"510502","disabled": false},
{"label": "纳溪区","value":"510503","disabled": false},
{"label": "龙马潭区","value":"510504","disabled": false},
{"label": "泸县","value":"510521","disabled": false},
{"label": "合江县","value":"510522","disabled": false},
{"label": "叙永县","value":"510524","disabled": false},
{"label": "古蔺县","value":"510525","disabled": false},
{"label": "旌阳区","value":"510603","disabled": false},
{"label": "中江县","value":"510623","disabled": false},
{"label": "罗江县","value":"510626","disabled": false},
{"label": "广汉市","value":"510681","disabled": false},
{"label": "什邡市","value":"510682","disabled": false},
{"label": "绵竹市","value":"510683","disabled": false},
{"label": "涪城区","value":"510703","disabled": false},
{"label": "游仙区","value":"510704","disabled": false},
{"label": "三台县","value":"510722","disabled": false},
{"label": "盐亭县","value":"510723","disabled": false},
{"label": "安县","value":"510724","disabled": false},
{"label": "梓潼县","value":"510725","disabled": false},
{"label": "北川羌族自治县","value":"510726","disabled": false},
{"label": "平武县","value":"510727","disabled": false},
{"label": "江油市","value":"510781","disabled": false},
{"label": "市中区","value":"510802","disabled": false},
{"label": "元坝区","value":"510811","disabled": false},
{"label": "朝天区","value":"510812","disabled": false},
{"label": "旺苍县","value":"510821","disabled": false},
{"label": "青川县","value":"510822","disabled": false},
{"label": "剑阁县","value":"510823","disabled": false},
{"label": "苍溪县","value":"510824","disabled": false},
{"label": "船山区","value":"510903","disabled": false},
{"label": "安居区","value":"510904","disabled": false},
{"label": "蓬溪县","value":"510921","disabled": false},
{"label": "射洪县","value":"510922","disabled": false},
{"label": "大英县","value":"510923","disabled": false},
{"label": "市中区","value":"511002","disabled": false},
{"label": "东兴区","value":"511011","disabled": false},
{"label": "威远县","value":"511024","disabled": false},
{"label": "资中县","value":"511025","disabled": false},
{"label": "隆昌县","value":"511028","disabled": false},
{"label": "市中区","value":"511102","disabled": false},
{"label": "沙湾区","value":"511111","disabled": false},
{"label": "五通桥区","value":"511112","disabled": false},
{"label": "金口河区","value":"511113","disabled": false},
{"label": "犍为县","value":"511123","disabled": false},
{"label": "井研县","value":"511124","disabled": false},
{"label": "夹江县","value":"511126","disabled": false},
{"label": "沐川县","value":"511129","disabled": false},
{"label": "峨边彝族自治县","value":"511132","disabled": false},
{"label": "马边彝族自治县","value":"511133","disabled": false},
{"label": "峨眉山市","value":"511181","disabled": false},
{"label": "顺庆区","value":"511302","disabled": false},
{"label": "高坪区","value":"511303","disabled": false},
{"label": "嘉陵区","value":"511304","disabled": false},
{"label": "南部县","value":"511321","disabled": false},
{"label": "营山县","value":"511322","disabled": false},
{"label": "蓬安县","value":"511323","disabled": false},
{"label": "仪陇县","value":"511324","disabled": false},
{"label": "西充县","value":"511325","disabled": false},
{"label": "阆中市","value":"511381","disabled": false},
{"label": "东坡区","value":"511402","disabled": false},
{"label": "仁寿县","value":"511421","disabled": false},
{"label": "彭山县","value":"511422","disabled": false},
{"label": "洪雅县","value":"511423","disabled": false},
{"label": "丹棱县","value":"511424","disabled": false},
{"label": "青神县","value":"511425","disabled": false},
{"label": "翠屏区","value":"511502","disabled": false},
{"label": "宜宾县","value":"511521","disabled": false},
{"label": "南溪县","value":"511522","disabled": false},
{"label": "江安县","value":"511523","disabled": false},
{"label": "长宁县","value":"511524","disabled": false},
{"label": "高县","value":"511525","disabled": false},
{"label": "珙县","value":"511526","disabled": false},
{"label": "筠连县","value":"511527","disabled": false},
{"label": "兴文县","value":"511528","disabled": false},
{"label": "屏山县","value":"511529","disabled": false},
{"label": "广安区","value":"511602","disabled": false},
{"label": "岳池县","value":"511621","disabled": false},
{"label": "武胜县","value":"511622","disabled": false},
{"label": "邻水县","value":"511623","disabled": false},
{"label": "华蓥市","value":"511681","disabled": false},
{"label": "通川区","value":"511702","disabled": false},
{"label": "达县","value":"511721","disabled": false},
{"label": "宣汉县","value":"511722","disabled": false},
{"label": "开江县","value":"511723","disabled": false},
{"label": "大竹县","value":"511724","disabled": false},
{"label": "渠县","value":"511725","disabled": false},
{"label": "万源市","value":"511781","disabled": false},
{"label": "雨城区","value":"511802","disabled": false},
{"label": "名山县","value":"511821","disabled": false},
{"label": "荥经县","value":"511822","disabled": false},
{"label": "汉源县","value":"511823","disabled": false},
{"label": "石棉县","value":"511824","disabled": false},
{"label": "天全县","value":"511825","disabled": false},
{"label": "芦山县","value":"511826","disabled": false},
{"label": "宝兴县","value":"511827","disabled": false},
{"label": "巴州区","value":"511902","disabled": false},
{"label": "通江县","value":"511921","disabled": false},
{"label": "南江县","value":"511922","disabled": false},
{"label": "平昌县","value":"511923","disabled": false},
{"label": "雁江区","value":"512002","disabled": false},
{"label": "安岳县","value":"512021","disabled": false},
{"label": "乐至县","value":"512022","disabled": false},
{"label": "简阳市","value":"512081","disabled": false},
{"label": "汶川县","value":"513221","disabled": false},
{"label": "理县","value":"513222","disabled": false},
{"label": "茂县","value":"513223","disabled": false},
{"label": "松潘县","value":"513224","disabled": false},
{"label": "九寨沟县","value":"513225","disabled": false},
{"label": "金川县","value":"513226","disabled": false},
{"label": "小金县","value":"513227","disabled": false},
{"label": "黑水县","value":"513228","disabled": false},
{"label": "马尔康县","value":"513229","disabled": false},
{"label": "壤塘县","value":"513230","disabled": false},
{"label": "阿坝县","value":"513231","disabled": false},
{"label": "若尔盖县","value":"513232","disabled": false},
{"label": "红原县","value":"513233","disabled": false},
{"label": "康定县","value":"513321","disabled": false},
{"label": "泸定县","value":"513322","disabled": false},
{"label": "丹巴县","value":"513323","disabled": false},
{"label": "九龙县","value":"513324","disabled": false},
{"label": "雅江县","value":"513325","disabled": false},
{"label": "道孚县","value":"513326","disabled": false},
{"label": "炉霍县","value":"513327","disabled": false},
{"label": "甘孜县","value":"513328","disabled": false},
{"label": "新龙县","value":"513329","disabled": false},
{"label": "德格县","value":"513330","disabled": false},
{"label": "白玉县","value":"513331","disabled": false},
{"label": "石渠县","value":"513332","disabled": false},
{"label": "色达县","value":"513333","disabled": false},
{"label": "理塘县","value":"513334","disabled": false},
{"label": "巴塘县","value":"513335","disabled": false},
{"label": "乡城县","value":"513336","disabled": false},
{"label": "稻城县","value":"513337","disabled": false},
{"label": "得荣县","value":"513338","disabled": false},
{"label": "西昌市","value":"513401","disabled": false},
{"label": "木里藏族自治县","value":"513422","disabled": false},
{"label": "盐源县","value":"513423","disabled": false},
{"label": "德昌县","value":"513424","disabled": false},
{"label": "会理县","value":"513425","disabled": false},
{"label": "会东县","value":"513426","disabled": false},
{"label": "宁南县","value":"513427","disabled": false},
{"label": "普格县","value":"513428","disabled": false},
{"label": "布拖县","value":"513429","disabled": false},
{"label": "金阳县","value":"513430","disabled": false},
{"label": "昭觉县","value":"513431","disabled": false},
{"label": "喜德县","value":"513432","disabled": false},
{"label": "冕宁县","value":"513433","disabled": false},
{"label": "越西县","value":"513434","disabled": false},
{"label": "甘洛县","value":"513435","disabled": false},
{"label": "美姑县","value":"513436","disabled": false},
{"label": "雷波县","value":"513437","disabled": false},
{"label": "南明区","value":"520102","disabled": false},
{"label": "云岩区","value":"520103","disabled": false},
{"label": "花溪区","value":"520111","disabled": false},
{"label": "乌当区","value":"520112","disabled": false},
{"label": "白云区","value":"520113","disabled": false},
{"label": "小河区","value":"520114","disabled": false},
{"label": "开阳县","value":"520121","disabled": false},
{"label": "息烽县","value":"520122","disabled": false},
{"label": "修文县","value":"520123","disabled": false},
{"label": "清镇市","value":"520181","disabled": false},
{"label": "钟山区","value":"520201","disabled": false},
{"label": "六枝特区","value":"520203","disabled": false},
{"label": "水城县","value":"520221","disabled": false},
{"label": "盘县","value":"520222","disabled": false},
{"label": "红花岗区","value":"520302","disabled": false},
{"label": "汇川区","value":"520303","disabled": false},
{"label": "遵义县","value":"520321","disabled": false},
{"label": "桐梓县","value":"520322","disabled": false},
{"label": "绥阳县","value":"520323","disabled": false},
{"label": "正安县","value":"520324","disabled": false},
{"label": "道真仡佬族苗族自治县","value":"520325","disabled": false},
{"label": "务川仡佬族苗族自治县","value":"520326","disabled": false},
{"label": "凤冈县","value":"520327","disabled": false},
{"label": "湄潭县","value":"520328","disabled": false},
{"label": "余庆县","value":"520329","disabled": false},
{"label": "习水县","value":"520330","disabled": false},
{"label": "赤水市","value":"520381","disabled": false},
{"label": "仁怀市","value":"520382","disabled": false},
{"label": "西秀区","value":"520402","disabled": false},
{"label": "平坝县","value":"520421","disabled": false},
{"label": "普定县","value":"520422","disabled": false},
{"label": "镇宁布依族苗族自治县","value":"520423","disabled": false},
{"label": "关岭布依族苗族自治县","value":"520424","disabled": false},
{"label": "紫云苗族布依族自治县","value":"520425","disabled": false},
{"label": "铜仁市","value":"522201","disabled": false},
{"label": "江口县","value":"522222","disabled": false},
{"label": "玉屏侗族自治县","value":"522223","disabled": false},
{"label": "石阡县","value":"522224","disabled": false},
{"label": "思南县","value":"522225","disabled": false},
{"label": "印江土家族苗族自治县","value":"522226","disabled": false},
{"label": "德江县","value":"522227","disabled": false},
{"label": "沿河土家族自治县","value":"522228","disabled": false},
{"label": "松桃苗族自治县","value":"522229","disabled": false},
{"label": "万山特区","value":"522230","disabled": false},
{"label": "兴义市","value":"522301","disabled": false},
{"label": "兴仁县","value":"522322","disabled": false},
{"label": "普安县","value":"522323","disabled": false},
{"label": "晴隆县","value":"522324","disabled": false},
{"label": "贞丰县","value":"522325","disabled": false},
{"label": "望谟县","value":"522326","disabled": false},
{"label": "册亨县","value":"522327","disabled": false},
{"label": "安龙县","value":"522328","disabled": false},
{"label": "毕节市","value":"522401","disabled": false},
{"label": "大方县","value":"522422","disabled": false},
{"label": "黔西县","value":"522423","disabled": false},
{"label": "金沙县","value":"522424","disabled": false},
{"label": "织金县","value":"522425","disabled": false},
{"label": "纳雍县","value":"522426","disabled": false},
{"label": "威宁彝族回族苗族自治县","value":"522427","disabled": false},
{"label": "赫章县","value":"522428","disabled": false},
{"label": "凯里市","value":"522601","disabled": false},
{"label": "黄平县","value":"522622","disabled": false},
{"label": "施秉县","value":"522623","disabled": false},
{"label": "三穗县","value":"522624","disabled": false},
{"label": "镇远县","value":"522625","disabled": false},
{"label": "岑巩县","value":"522626","disabled": false},
{"label": "天柱县","value":"522627","disabled": false},
{"label": "锦屏县","value":"522628","disabled": false},
{"label": "剑河县","value":"522629","disabled": false},
{"label": "台江县","value":"522630","disabled": false},
{"label": "黎平县","value":"522631","disabled": false},
{"label": "榕江县","value":"522632","disabled": false},
{"label": "从江县","value":"522633","disabled": false},
{"label": "雷山县","value":"522634","disabled": false},
{"label": "麻江县","value":"522635","disabled": false},
{"label": "丹寨县","value":"522636","disabled": false},
{"label": "都匀市","value":"522701","disabled": false},
{"label": "福泉市","value":"522702","disabled": false},
{"label": "荔波县","value":"522722","disabled": false},
{"label": "贵定县","value":"522723","disabled": false},
{"label": "瓮安县","value":"522725","disabled": false},
{"label": "独山县","value":"522726","disabled": false},
{"label": "平塘县","value":"522727","disabled": false},
{"label": "罗甸县","value":"522728","disabled": false},
{"label": "长顺县","value":"522729","disabled": false},
{"label": "龙里县","value":"522730","disabled": false},
{"label": "惠水县","value":"522731","disabled": false},
{"label": "三都水族自治县","value":"522732","disabled": false},
{"label": "五华区","value":"530102","disabled": false},
{"label": "盘龙区","value":"530103","disabled": false},
{"label": "官渡区","value":"530111","disabled": false},
{"label": "西山区","value":"530112","disabled": false},
{"label": "东川区","value":"530113","disabled": false},
{"label": "呈贡县","value":"530121","disabled": false},
{"label": "晋宁县","value":"530122","disabled": false},
{"label": "富民县","value":"530124","disabled": false},
{"label": "宜良县","value":"530125","disabled": false},
{"label": "石林彝族自治县","value":"530126","disabled": false},
{"label": "嵩明县","value":"530127","disabled": false},
{"label": "禄劝彝族苗族自治县","value":"530128","disabled": false},
{"label": "寻甸回族彝族自治县","value":"530129","disabled": false},
{"label": "安宁市","value":"530181","disabled": false},
{"label": "麒麟区","value":"530302","disabled": false},
{"label": "马龙县","value":"530321","disabled": false},
{"label": "陆良县","value":"530322","disabled": false},
{"label": "师宗县","value":"530323","disabled": false},
{"label": "罗平县","value":"530324","disabled": false},
{"label": "富源县","value":"530325","disabled": false},
{"label": "会泽县","value":"530326","disabled": false},
{"label": "沾益县","value":"530328","disabled": false},
{"label": "宣威市","value":"530381","disabled": false},
{"label": "红塔区","value":"530402","disabled": false},
{"label": "江川县","value":"530421","disabled": false},
{"label": "澄江县","value":"530422","disabled": false},
{"label": "通海县","value":"530423","disabled": false},
{"label": "华宁县","value":"530424","disabled": false},
{"label": "易门县","value":"530425","disabled": false},
{"label": "峨山彝族自治县","value":"530426","disabled": false},
{"label": "新平彝族傣族自治县","value":"530427","disabled": false},
{"label": "元江哈尼族彝族傣族自治县","value":"530428","disabled": false},
{"label": "隆阳区","value":"530502","disabled": false},
{"label": "施甸县","value":"530521","disabled": false},
{"label": "腾冲县","value":"530522","disabled": false},
{"label": "龙陵县","value":"530523","disabled": false},
{"label": "昌宁县","value":"530524","disabled": false},
{"label": "昭阳区","value":"530602","disabled": false},
{"label": "鲁甸县","value":"530621","disabled": false},
{"label": "巧家县","value":"530622","disabled": false},
{"label": "盐津县","value":"530623","disabled": false},
{"label": "大关县","value":"530624","disabled": false},
{"label": "永善县","value":"530625","disabled": false},
{"label": "绥江县","value":"530626","disabled": false},
{"label": "镇雄县","value":"530627","disabled": false},
{"label": "彝良县","value":"530628","disabled": false},
{"label": "威信县","value":"530629","disabled": false},
{"label": "水富县","value":"530630","disabled": false},
{"label": "古城区","value":"530702","disabled": false},
{"label": "玉龙纳西族自治县","value":"530721","disabled": false},
{"label": "永胜县","value":"530722","disabled": false},
{"label": "华坪县","value":"530723","disabled": false},
{"label": "宁蒗彝族自治县","value":"530724","disabled": false},
{"label": "思茅区","value":"530802","disabled": false},
{"label": "宁洱哈尼族彝族自治县","value":"530821","disabled": false},
{"label": "墨江哈尼族自治县","value":"530822","disabled": false},
{"label": "景东彝族自治县","value":"530823","disabled": false},
{"label": "景谷傣族彝族自治县","value":"530824","disabled": false},
{"label": "镇沅彝族哈尼族拉祜族自治县","value":"530825","disabled": false},
{"label": "江城哈尼族彝族自治县","value":"530826","disabled": false},
{"label": "孟连傣族拉祜族佤族自治县","value":"530827","disabled": false},
{"label": "澜沧拉祜族自治县","value":"530828","disabled": false},
{"label": "西盟佤族自治县","value":"530829","disabled": false},
{"label": "临翔区","value":"530902","disabled": false},
{"label": "凤庆县","value":"530921","disabled": false},
{"label": "云县","value":"530922","disabled": false},
{"label": "永德县","value":"530923","disabled": false},
{"label": "镇康县","value":"530924","disabled": false},
{"label": "双江拉祜族佤族布朗族傣族自治县","value":"530925","disabled": false},
{"label": "耿马傣族佤族自治县","value":"530926","disabled": false},
{"label": "沧源佤族自治县","value":"530927","disabled": false},
{"label": "楚雄市","value":"532301","disabled": false},
{"label": "双柏县","value":"532322","disabled": false},
{"label": "牟定县","value":"532323","disabled": false},
{"label": "南华县","value":"532324","disabled": false},
{"label": "姚安县","value":"532325","disabled": false},
{"label": "大姚县","value":"532326","disabled": false},
{"label": "永仁县","value":"532327","disabled": false},
{"label": "元谋县","value":"532328","disabled": false},
{"label": "武定县","value":"532329","disabled": false},
{"label": "禄丰县","value":"532331","disabled": false},
{"label": "个旧市","value":"532501","disabled": false},
{"label": "开远市","value":"532502","disabled": false},
{"label": "蒙自县","value":"532522","disabled": false},
{"label": "屏边苗族自治县","value":"532523","disabled": false},
{"label": "建水县","value":"532524","disabled": false},
{"label": "石屏县","value":"532525","disabled": false},
{"label": "弥勒县","value":"532526","disabled": false},
{"label": "泸西县","value":"532527","disabled": false},
{"label": "元阳县","value":"532528","disabled": false},
{"label": "红河县","value":"532529","disabled": false},
{"label": "金平苗族瑶族傣族自治县","value":"532530","disabled": false},
{"label": "绿春县","value":"532531","disabled": false},
{"label": "河口瑶族自治县","value":"532532","disabled": false},
{"label": "文山县","value":"532621","disabled": false},
{"label": "砚山县","value":"532622","disabled": false},
{"label": "西畴县","value":"532623","disabled": false},
{"label": "麻栗坡县","value":"532624","disabled": false},
{"label": "马关县","value":"532625","disabled": false},
{"label": "丘北县","value":"532626","disabled": false},
{"label": "广南县","value":"532627","disabled": false},
{"label": "富宁县","value":"532628","disabled": false},
{"label": "景洪市","value":"532801","disabled": false},
{"label": "勐海县","value":"532822","disabled": false},
{"label": "勐腊县","value":"532823","disabled": false},
{"label": "大理市","value":"532901","disabled": false},
{"label": "漾濞彝族自治县","value":"532922","disabled": false},
{"label": "祥云县","value":"532923","disabled": false},
{"label": "宾川县","value":"532924","disabled": false},
{"label": "弥渡县","value":"532925","disabled": false},
{"label": "南涧彝族自治县","value":"532926","disabled": false},
{"label": "巍山彝族回族自治县","value":"532927","disabled": false},
{"label": "永平县","value":"532928","disabled": false},
{"label": "云龙县","value":"532929","disabled": false},
{"label": "洱源县","value":"532930","disabled": false},
{"label": "剑川县","value":"532931","disabled": false},
{"label": "鹤庆县","value":"532932","disabled": false},
{"label": "瑞丽市","value":"533102","disabled": false},
{"label": "潞西市","value":"533103","disabled": false},
{"label": "梁河县","value":"533122","disabled": false},
{"label": "盈江县","value":"533123","disabled": false},
{"label": "陇川县","value":"533124","disabled": false},
{"label": "泸水县","value":"533321","disabled": false},
{"label": "福贡县","value":"533323","disabled": false},
{"label": "贡山独龙族怒族自治县","value":"533324","disabled": false},
{"label": "兰坪白族普米族自治县","value":"533325","disabled": false},
{"label": "香格里拉县","value":"533421","disabled": false},
{"label": "德钦县","value":"533422","disabled": false},
{"label": "维西傈僳族自治县","value":"533423","disabled": false},
{"label": "城关区","value":"540102","disabled": false},
{"label": "林周县","value":"540121","disabled": false},
{"label": "当雄县","value":"540122","disabled": false},
{"label": "尼木县","value":"540123","disabled": false},
{"label": "曲水县","value":"540124","disabled": false},
{"label": "堆龙德庆县","value":"540125","disabled": false},
{"label": "达孜县","value":"540126","disabled": false},
{"label": "墨竹工卡县","value":"540127","disabled": false},
{"label": "昌都县","value":"542121","disabled": false},
{"label": "江达县","value":"542122","disabled": false},
{"label": "贡觉县","value":"542123","disabled": false},
{"label": "类乌齐县","value":"542124","disabled": false},
{"label": "丁青县","value":"542125","disabled": false},
{"label": "察雅县","value":"542126","disabled": false},
{"label": "八宿县","value":"542127","disabled": false},
{"label": "左贡县","value":"542128","disabled": false},
{"label": "芒康县","value":"542129","disabled": false},
{"label": "洛隆县","value":"542132","disabled": false},
{"label": "边坝县","value":"542133","disabled": false},
{"label": "乃东县","value":"542221","disabled": false},
{"label": "扎囊县","value":"542222","disabled": false},
{"label": "贡嘎县","value":"542223","disabled": false},
{"label": "桑日县","value":"542224","disabled": false},
{"label": "琼结县","value":"542225","disabled": false},
{"label": "曲松县","value":"542226","disabled": false},
{"label": "措美县","value":"542227","disabled": false},
{"label": "洛扎县","value":"542228","disabled": false},
{"label": "加查县","value":"542229","disabled": false},
{"label": "隆子县","value":"542231","disabled": false},
{"label": "错那县","value":"542232","disabled": false},
{"label": "浪卡子县","value":"542233","disabled": false},
{"label": "日喀则市","value":"542301","disabled": false},
{"label": "南木林县","value":"542322","disabled": false},
{"label": "江孜县","value":"542323","disabled": false},
{"label": "定日县","value":"542324","disabled": false},
{"label": "萨迦县","value":"542325","disabled": false},
{"label": "拉孜县","value":"542326","disabled": false},
{"label": "昂仁县","value":"542327","disabled": false},
{"label": "谢通门县","value":"542328","disabled": false},
{"label": "白朗县","value":"542329","disabled": false},
{"label": "仁布县","value":"542330","disabled": false},
{"label": "康马县","value":"542331","disabled": false},
{"label": "定结县","value":"542332","disabled": false},
{"label": "仲巴县","value":"542333","disabled": false},
{"label": "亚东县","value":"542334","disabled": false},
{"label": "吉隆县","value":"542335","disabled": false},
{"label": "聂拉木县","value":"542336","disabled": false},
{"label": "萨嘎县","value":"542337","disabled": false},
{"label": "岗巴县","value":"542338","disabled": false},
{"label": "那曲县","value":"542421","disabled": false},
{"label": "嘉黎县","value":"542422","disabled": false},
{"label": "比如县","value":"542423","disabled": false},
{"label": "聂荣县","value":"542424","disabled": false},
{"label": "安多县","value":"542425","disabled": false},
{"label": "申扎县","value":"542426","disabled": false},
{"label": "索县","value":"542427","disabled": false},
{"label": "班戈县","value":"542428","disabled": false},
{"label": "巴青县","value":"542429","disabled": false},
{"label": "尼玛县","value":"542430","disabled": false},
{"label": "普兰县","value":"542521","disabled": false},
{"label": "札达县","value":"542522","disabled": false},
{"label": "噶尔县","value":"542523","disabled": false},
{"label": "日土县","value":"542524","disabled": false},
{"label": "革吉县","value":"542525","disabled": false},
{"label": "改则县","value":"542526","disabled": false},
{"label": "措勤县","value":"542527","disabled": false},
{"label": "林芝县","value":"542621","disabled": false},
{"label": "工布江达县","value":"542622","disabled": false},
{"label": "米林县","value":"542623","disabled": false},
{"label": "墨脱县","value":"542624","disabled": false},
{"label": "波密县","value":"542625","disabled": false},
{"label": "察隅县","value":"542626","disabled": false},
{"label": "朗县","value":"542627","disabled": false},
{"label": "新城区","value":"610102","disabled": false},
{"label": "碑林区","value":"610103","disabled": false},
{"label": "莲湖区","value":"610104","disabled": false},
{"label": "灞桥区","value":"610111","disabled": false},
{"label": "未央区","value":"610112","disabled": false},
{"label": "雁塔区","value":"610113","disabled": false},
{"label": "阎良区","value":"610114","disabled": false},
{"label": "临潼区","value":"610115","disabled": false},
{"label": "长安区","value":"610116","disabled": false},
{"label": "蓝田县","value":"610122","disabled": false},
{"label": "周至县","value":"610124","disabled": false},
{"label": "户县","value":"610125","disabled": false},
{"label": "高陵县","value":"610126","disabled": false},
{"label": "王益区","value":"610202","disabled": false},
{"label": "印台区","value":"610203","disabled": false},
{"label": "耀州区","value":"610204","disabled": false},
{"label": "宜君县","value":"610222","disabled": false},
{"label": "渭滨区","value":"610302","disabled": false},
{"label": "金台区","value":"610303","disabled": false},
{"label": "陈仓区","value":"610304","disabled": false},
{"label": "凤翔县","value":"610322","disabled": false},
{"label": "岐山县","value":"610323","disabled": false},
{"label": "扶风县","value":"610324","disabled": false},
{"label": "眉县","value":"610326","disabled": false},
{"label": "陇县","value":"610327","disabled": false},
{"label": "千阳县","value":"610328","disabled": false},
{"label": "麟游县","value":"610329","disabled": false},
{"label": "凤县","value":"610330","disabled": false},
{"label": "太白县","value":"610331","disabled": false},
{"label": "秦都区","value":"610402","disabled": false},
{"label": "杨凌区","value":"610403","disabled": false},
{"label": "渭城区","value":"610404","disabled": false},
{"label": "三原县","value":"610422","disabled": false},
{"label": "泾阳县","value":"610423","disabled": false},
{"label": "乾县","value":"610424","disabled": false},
{"label": "礼泉县","value":"610425","disabled": false},
{"label": "永寿县","value":"610426","disabled": false},
{"label": "彬县","value":"610427","disabled": false},
{"label": "长武县","value":"610428","disabled": false},
{"label": "旬邑县","value":"610429","disabled": false},
{"label": "淳化县","value":"610430","disabled": false},
{"label": "武功县","value":"610431","disabled": false},
{"label": "兴平市","value":"610481","disabled": false},
{"label": "临渭区","value":"610502","disabled": false},
{"label": "华县","value":"610521","disabled": false},
{"label": "潼关县","value":"610522","disabled": false},
{"label": "大荔县","value":"610523","disabled": false},
{"label": "合阳县","value":"610524","disabled": false},
{"label": "澄城县","value":"610525","disabled": false},
{"label": "蒲城县","value":"610526","disabled": false},
{"label": "白水县","value":"610527","disabled": false},
{"label": "富平县","value":"610528","disabled": false},
{"label": "韩城市","value":"610581","disabled": false},
{"label": "华阴市","value":"610582","disabled": false},
{"label": "宝塔区","value":"610602","disabled": false},
{"label": "延长县","value":"610621","disabled": false},
{"label": "延川县","value":"610622","disabled": false},
{"label": "子长县","value":"610623","disabled": false},
{"label": "安塞县","value":"610624","disabled": false},
{"label": "志丹县","value":"610625","disabled": false},
{"label": "吴起县","value":"610626","disabled": false},
{"label": "甘泉县","value":"610627","disabled": false},
{"label": "富县","value":"610628","disabled": false},
{"label": "洛川县","value":"610629","disabled": false},
{"label": "宜川县","value":"610630","disabled": false},
{"label": "黄龙县","value":"610631","disabled": false},
{"label": "黄陵县","value":"610632","disabled": false},
{"label": "汉台区","value":"610702","disabled": false},
{"label": "南郑县","value":"610721","disabled": false},
{"label": "城固县","value":"610722","disabled": false},
{"label": "洋县","value":"610723","disabled": false},
{"label": "西乡县","value":"610724","disabled": false},
{"label": "勉县","value":"610725","disabled": false},
{"label": "宁强县","value":"610726","disabled": false},
{"label": "略阳县","value":"610727","disabled": false},
{"label": "镇巴县","value":"610728","disabled": false},
{"label": "留坝县","value":"610729","disabled": false},
{"label": "佛坪县","value":"610730","disabled": false},
{"label": "榆阳区","value":"610802","disabled": false},
{"label": "神木县","value":"610821","disabled": false},
{"label": "府谷县","value":"610822","disabled": false},
{"label": "横山县","value":"610823","disabled": false},
{"label": "靖边县","value":"610824","disabled": false},
{"label": "定边县","value":"610825","disabled": false},
{"label": "绥德县","value":"610826","disabled": false},
{"label": "米脂县","value":"610827","disabled": false},
{"label": "佳县","value":"610828","disabled": false},
{"label": "吴堡县","value":"610829","disabled": false},
{"label": "清涧县","value":"610830","disabled": false},
{"label": "子洲县","value":"610831","disabled": false},
{"label": "汉滨区","value":"610902","disabled": false},
{"label": "汉阴县","value":"610921","disabled": false},
{"label": "石泉县","value":"610922","disabled": false},
{"label": "宁陕县","value":"610923","disabled": false},
{"label": "紫阳县","value":"610924","disabled": false},
{"label": "岚皋县","value":"610925","disabled": false},
{"label": "平利县","value":"610926","disabled": false},
{"label": "镇坪县","value":"610927","disabled": false},
{"label": "旬阳县","value":"610928","disabled": false},
{"label": "白河县","value":"610929","disabled": false},
{"label": "商州区","value":"611002","disabled": false},
{"label": "洛南县","value":"611021","disabled": false},
{"label": "丹凤县","value":"611022","disabled": false},
{"label": "商南县","value":"611023","disabled": false},
{"label": "山阳县","value":"611024","disabled": false},
{"label": "镇安县","value":"611025","disabled": false},
{"label": "柞水县","value":"611026","disabled": false},
{"label": "城关区","value":"620102","disabled": false},
{"label": "七里河区","value":"620103","disabled": false},
{"label": "西固区","value":"620104","disabled": false},
{"label": "安宁区","value":"620105","disabled": false},
{"label": "红古区","value":"620111","disabled": false},
{"label": "永登县","value":"620121","disabled": false},
{"label": "皋兰县","value":"620122","disabled": false},
{"label": "榆中县","value":"620123","disabled": false},
{"label": "金川区","value":"620302","disabled": false},
{"label": "永昌县","value":"620321","disabled": false},
{"label": "白银区","value":"620402","disabled": false},
{"label": "平川区","value":"620403","disabled": false},
{"label": "靖远县","value":"620421","disabled": false},
{"label": "会宁县","value":"620422","disabled": false},
{"label": "景泰县","value":"620423","disabled": false},
{"label": "秦州区","value":"620502","disabled": false},
{"label": "麦积区","value":"620503","disabled": false},
{"label": "清水县","value":"620521","disabled": false},
{"label": "秦安县","value":"620522","disabled": false},
{"label": "甘谷县","value":"620523","disabled": false},
{"label": "武山县","value":"620524","disabled": false},
{"label": "张家川回族自治县","value":"620525","disabled": false},
{"label": "凉州区","value":"620602","disabled": false},
{"label": "民勤县","value":"620621","disabled": false},
{"label": "古浪县","value":"620622","disabled": false},
{"label": "天祝藏族自治县","value":"620623","disabled": false},
{"label": "甘州区","value":"620702","disabled": false},
{"label": "肃南裕固族自治县","value":"620721","disabled": false},
{"label": "民乐县","value":"620722","disabled": false},
{"label": "临泽县","value":"620723","disabled": false},
{"label": "高台县","value":"620724","disabled": false},
{"label": "山丹县","value":"620725","disabled": false},
{"label": "崆峒区","value":"620802","disabled": false},
{"label": "泾川县","value":"620821","disabled": false},
{"label": "灵台县","value":"620822","disabled": false},
{"label": "崇信县","value":"620823","disabled": false},
{"label": "华亭县","value":"620824","disabled": false},
{"label": "庄浪县","value":"620825","disabled": false},
{"label": "静宁县","value":"620826","disabled": false},
{"label": "肃州区","value":"620902","disabled": false},
{"label": "金塔县","value":"620921","disabled": false},
{"label": "瓜州县","value":"620922","disabled": false},
{"label": "肃北蒙古族自治县","value":"620923","disabled": false},
{"label": "阿克塞哈萨克族自治县","value":"620924","disabled": false},
{"label": "玉门市","value":"620981","disabled": false},
{"label": "敦煌市","value":"620982","disabled": false},
{"label": "西峰区","value":"621002","disabled": false},
{"label": "庆城县","value":"621021","disabled": false},
{"label": "环县","value":"621022","disabled": false},
{"label": "华池县","value":"621023","disabled": false},
{"label": "合水县","value":"621024","disabled": false},
{"label": "正宁县","value":"621025","disabled": false},
{"label": "宁县","value":"621026","disabled": false},
{"label": "镇原县","value":"621027","disabled": false},
{"label": "安定区","value":"621102","disabled": false},
{"label": "通渭县","value":"621121","disabled": false},
{"label": "陇西县","value":"621122","disabled": false},
{"label": "渭源县","value":"621123","disabled": false},
{"label": "临洮县","value":"621124","disabled": false},
{"label": "漳县","value":"621125","disabled": false},
{"label": "岷县","value":"621126","disabled": false},
{"label": "武都区","value":"621202","disabled": false},
{"label": "成县","value":"621221","disabled": false},
{"label": "文县","value":"621222","disabled": false},
{"label": "宕昌县","value":"621223","disabled": false},
{"label": "康县","value":"621224","disabled": false},
{"label": "西和县","value":"621225","disabled": false},
{"label": "礼县","value":"621226","disabled": false},
{"label": "徽县","value":"621227","disabled": false},
{"label": "两当县","value":"621228","disabled": false},
{"label": "临夏市","value":"622901","disabled": false},
{"label": "临夏县","value":"622921","disabled": false},
{"label": "康乐县","value":"622922","disabled": false},
{"label": "永靖县","value":"622923","disabled": false},
{"label": "广河县","value":"622924","disabled": false},
{"label": "和政县","value":"622925","disabled": false},
{"label": "东乡族自治县","value":"622926","disabled": false},
{"label": "积石山保安族东乡族撒拉族自治县","value":"622927","disabled": false},
{"label": "合作市","value":"623001","disabled": false},
{"label": "临潭县","value":"623021","disabled": false},
{"label": "卓尼县","value":"623022","disabled": false},
{"label": "舟曲县","value":"623023","disabled": false},
{"label": "迭部县","value":"623024","disabled": false},
{"label": "玛曲县","value":"623025","disabled": false},
{"label": "碌曲县","value":"623026","disabled": false},
{"label": "夏河县","value":"623027","disabled": false},
{"label": "城东区","value":"630102","disabled": false},
{"label": "城中区","value":"630103","disabled": false},
{"label": "城西区","value":"630104","disabled": false},
{"label": "城北区","value":"630105","disabled": false},
{"label": "大通回族土族自治县","value":"630121","disabled": false},
{"label": "湟中县","value":"630122","disabled": false},
{"label": "湟源县","value":"630123","disabled": false},
{"label": "平安县","value":"632121","disabled": false},
{"label": "民和回族土族自治县","value":"632122","disabled": false},
{"label": "乐都县","value":"632123","disabled": false},
{"label": "互助土族自治县","value":"632126","disabled": false},
{"label": "化隆回族自治县","value":"632127","disabled": false},
{"label": "循化撒拉族自治县","value":"632128","disabled": false},
{"label": "门源回族自治县","value":"632221","disabled": false},
{"label": "祁连县","value":"632222","disabled": false},
{"label": "海晏县","value":"632223","disabled": false},
{"label": "刚察县","value":"632224","disabled": false},
{"label": "同仁县","value":"632321","disabled": false},
{"label": "尖扎县","value":"632322","disabled": false},
{"label": "泽库县","value":"632323","disabled": false},
{"label": "河南蒙古族自治县","value":"632324","disabled": false},
{"label": "共和县","value":"632521","disabled": false},
{"label": "同德县","value":"632522","disabled": false},
{"label": "贵德县","value":"632523","disabled": false},
{"label": "兴海县","value":"632524","disabled": false},
{"label": "贵南县","value":"632525","disabled": false},
{"label": "玛沁县","value":"632621","disabled": false},
{"label": "班玛县","value":"632622","disabled": false},
{"label": "甘德县","value":"632623","disabled": false},
{"label": "达日县","value":"632624","disabled": false},
{"label": "久治县","value":"632625","disabled": false},
{"label": "玛多县","value":"632626","disabled": false},
{"label": "玉树县","value":"632721","disabled": false},
{"label": "杂多县","value":"632722","disabled": false},
{"label": "称多县","value":"632723","disabled": false},
{"label": "治多县","value":"632724","disabled": false},
{"label": "囊谦县","value":"632725","disabled": false},
{"label": "曲麻莱县","value":"632726","disabled": false},
{"label": "格尔木市","value":"632801","disabled": false},
{"label": "德令哈市","value":"632802","disabled": false},
{"label": "乌兰县","value":"632821","disabled": false},
{"label": "都兰县","value":"632822","disabled": false},
{"label": "天峻县","value":"632823","disabled": false},
{"label": "兴庆区","value":"640104","disabled": false},
{"label": "西夏区","value":"640105","disabled": false},
{"label": "金凤区","value":"640106","disabled": false},
{"label": "永宁县","value":"640121","disabled": false},
{"label": "贺兰县","value":"640122","disabled": false},
{"label": "灵武市","value":"640181","disabled": false},
{"label": "大武口区","value":"640202","disabled": false},
{"label": "惠农区","value":"640205","disabled": false},
{"label": "平罗县","value":"640221","disabled": false},
{"label": "利通区","value":"640302","disabled": false},
{"label": "盐池县","value":"640323","disabled": false},
{"label": "同心县","value":"640324","disabled": false},
{"label": "青铜峡市","value":"640381","disabled": false},
{"label": "原州区","value":"640402","disabled": false},
{"label": "西吉县","value":"640422","disabled": false},
{"label": "隆德县","value":"640423","disabled": false},
{"label": "泾源县","value":"640424","disabled": false},
{"label": "彭阳县","value":"640425","disabled": false},
{"label": "沙坡头区","value":"640502","disabled": false},
{"label": "中宁县","value":"640521","disabled": false},
{"label": "海原县","value":"640522","disabled": false},
{"label": "天山区","value":"650102","disabled": false},
{"label": "沙依巴克区","value":"650103","disabled": false},
{"label": "新市区","value":"650104","disabled": false},
{"label": "水磨沟区","value":"650105","disabled": false},
{"label": "头屯河区","value":"650106","disabled": false},
{"label": "达坂城区","value":"650107","disabled": false},
{"label": "米东区","value":"650109","disabled": false},
{"label": "乌鲁木齐县","value":"650121","disabled": false},
{"label": "独山子区","value":"650202","disabled": false},
{"label": "克拉玛依区","value":"650203","disabled": false},
{"label": "白碱滩区","value":"650204","disabled": false},
{"label": "乌尔禾区","value":"650205","disabled": false},
{"label": "吐鲁番市","value":"652101","disabled": false},
{"label": "鄯善县","value":"652122","disabled": false},
{"label": "托克逊县","value":"652123","disabled": false},
{"label": "哈密市","value":"652201","disabled": false},
{"label": "巴里坤哈萨克自治县","value":"652222","disabled": false},
{"label": "伊吾县","value":"652223","disabled": false},
{"label": "昌吉市","value":"652301","disabled": false},
{"label": "阜康市","value":"652302","disabled": false},
{"label": "呼图壁县","value":"652323","disabled": false},
{"label": "玛纳斯县","value":"652324","disabled": false},
{"label": "奇台县","value":"652325","disabled": false},
{"label": "吉木萨尔县","value":"652327","disabled": false},
{"label": "木垒哈萨克自治县","value":"652328","disabled": false},
{"label": "博乐市","value":"652701","disabled": false},
{"label": "精河县","value":"652722","disabled": false},
{"label": "温泉县","value":"652723","disabled": false},
{"label": "库尔勒市","value":"652801","disabled": false},
{"label": "轮台县","value":"652822","disabled": false},
{"label": "尉犁县","value":"652823","disabled": false},
{"label": "若羌县","value":"652824","disabled": false},
{"label": "且末县","value":"652825","disabled": false},
{"label": "焉耆回族自治县","value":"652826","disabled": false},
{"label": "和静县","value":"652827","disabled": false},
{"label": "和硕县","value":"652828","disabled": false},
{"label": "博湖县","value":"652829","disabled": false},
{"label": "阿克苏市","value":"652901","disabled": false},
{"label": "温宿县","value":"652922","disabled": false},
{"label": "库车县","value":"652923","disabled": false},
{"label": "沙雅县","value":"652924","disabled": false},
{"label": "新和县","value":"652925","disabled": false},
{"label": "拜城县","value":"652926","disabled": false},
{"label": "乌什县","value":"652927","disabled": false},
{"label": "阿瓦提县","value":"652928","disabled": false},
{"label": "柯坪县","value":"652929","disabled": false},
{"label": "阿图什市","value":"653001","disabled": false},
{"label": "阿克陶县","value":"653022","disabled": false},
{"label": "阿合奇县","value":"653023","disabled": false},
{"label": "乌恰县","value":"653024","disabled": false},
{"label": "喀什市","value":"653101","disabled": false},
{"label": "疏附县","value":"653121","disabled": false},
{"label": "疏勒县","value":"653122","disabled": false},
{"label": "英吉沙县","value":"653123","disabled": false},
{"label": "泽普县","value":"653124","disabled": false},
{"label": "莎车县","value":"653125","disabled": false},
{"label": "叶城县","value":"653126","disabled": false},
{"label": "麦盖提县","value":"653127","disabled": false},
{"label": "岳普湖县","value":"653128","disabled": false},
{"label": "伽师县","value":"653129","disabled": false},
{"label": "巴楚县","value":"653130","disabled": false},
{"label": "塔什库尔干塔吉克自治县","value":"653131","disabled": false},
{"label": "和田市","value":"653201","disabled": false},
{"label": "和田县","value":"653221","disabled": false},
{"label": "墨玉县","value":"653222","disabled": false},
{"label": "皮山县","value":"653223","disabled": false},
{"label": "洛浦县","value":"653224","disabled": false},
{"label": "策勒县","value":"653225","disabled": false},
{"label": "于田县","value":"653226","disabled": false},
{"label": "民丰县","value":"653227","disabled": false},
{"label": "伊宁市","value":"654002","disabled": false},
{"label": "奎屯市","value":"654003","disabled": false},
{"label": "伊宁县","value":"654021","disabled": false},
{"label": "察布查尔锡伯自治县","value":"654022","disabled": false},
{"label": "霍城县","value":"654023","disabled": false},
{"label": "巩留县","value":"654024","disabled": false},
{"label": "新源县","value":"654025","disabled": false},
{"label": "昭苏县","value":"654026","disabled": false},
{"label": "特克斯县","value":"654027","disabled": false},
{"label": "尼勒克县","value":"654028","disabled": false},
{"label": "塔城市","value":"654201","disabled": false},
{"label": "乌苏市","value":"654202","disabled": false},
{"label": "额敏县","value":"654221","disabled": false},
{"label": "沙湾县","value":"654223","disabled": false},
{"label": "托里县","value":"654224","disabled": false},
{"label": "裕民县","value":"654225","disabled": false},
{"label": "和布克赛尔蒙古自治县","value":"654226","disabled": false},
{"label": "阿勒泰市","value":"654301","disabled": false},
{"label": "布尔津县","value":"654321","disabled": false},
{"label": "富蕴县","value":"654322","disabled": false},
{"label": "福海县","value":"654323","disabled": false},
{"label": "哈巴河县","value":"654324","disabled": false},
{"label": "青河县","value":"654325","disabled": false},
{"label": "吉木乃县","value":"654326","disabled": false},
{"label": "中正区","value":"710101","disabled": false},
{"label": "大同区","value":"710102","disabled": false},
{"label": "中山区","value":"710103","disabled": false},
{"label": "松山区","value":"710104","disabled": false},
{"label": "大安区","value":"710105","disabled": false},
{"label": "万华区","value":"710106","disabled": false},
{"label": "信义区","value":"710107","disabled": false},
{"label": "士林区","value":"710108","disabled": false},
{"label": "北投区","value":"710109","disabled": false},
{"label": "内湖区","value":"710110","disabled": false},
{"label": "南港区","value":"710111","disabled": false},
{"label": "文山区","value":"710112","disabled": false},
{"label": "新兴区","value":"710201","disabled": false},
{"label": "前金区","value":"710202","disabled": false},
{"label": "芩雅区","value":"710203","disabled": false},
{"label": "盐埕区","value":"710204","disabled": false},
{"label": "鼓山区","value":"710205","disabled": false},
{"label": "旗津区","value":"710206","disabled": false},
{"label": "前镇区","value":"710207","disabled": false},
{"label": "三民区","value":"710208","disabled": false},
{"label": "左营区","value":"710209","disabled": false},
{"label": "楠梓区","value":"710210","disabled": false},
{"label": "小港区","value":"710211","disabled": false},
{"label": "仁爱区","value":"710301","disabled": false},
{"label": "信义区","value":"710302","disabled": false},
{"label": "中正区","value":"710303","disabled": false},
{"label": "中山区","value":"710304","disabled": false},
{"label": "安乐区","value":"710305","disabled": false},
{"label": "暖暖区","value":"710306","disabled": false},
{"label": "七堵区","value":"710307","disabled": false},
{"label": "中区","value":"710401","disabled": false},
{"label": "东区","value":"710402","disabled": false},
{"label": "南区","value":"710403","disabled": false},
{"label": "西区","value":"710404","disabled": false},
{"label": "北区","value":"710405","disabled": false},
{"label": "北屯区","value":"710406","disabled": false},
{"label": "西屯区","value":"710407","disabled": false},
{"label": "南屯区","value":"710408","disabled": false},
{"label": "中西区","value":"710501","disabled": false},
{"label": "东区","value":"710502","disabled": false},
{"label": "南区","value":"710503","disabled": false},
{"label": "北区","value":"710504","disabled": false},
{"label": "安平区","value":"710505","disabled": false},
{"label": "安南区","value":"710506","disabled": false},
{"label": "东区","value":"710601","disabled": false},
{"label": "北区","value":"710602","disabled": false},
{"label": "香山区","value":"710603","disabled": false},
{"label": "东区","value":"710701","disabled": false},
{"label": "西区","value":"710702","disabled": false},
  ]
}

export default util;
